-----------------------
-- NPSee             --
-- By: Wire @ Byriel --
-----------------------

TargetManager = class();

-- constants
TargetManager.Constant = {
  Message = {
    Interrupted = "INTERRUPTED",
    NoAbilityName = "No Abilty Name",
    Pet = "'s Pet"
  }
};

TargetManager.Defaults = {
  Spacing = 2,
  Interrupt = 0.06,
  CastBarSpacing = true,
  CastBarDisplay = true,
  PetDisplay = false,
  PvPDisplay = true,
  Stacking = false
};

function TargetManager:Constructor(application)
  
  -- parameters
  self:SetApplication(application);
  self:SetParent(self:GetApplication():GetContext());
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- target bar
  self:SetTargetBar(Wired.UI.CascadeBar(self:GetApplication():GetName() .. "_TargetsBar", self:GetApplication():GetContext(), {Left = 0, Top = (UIParent:GetHeight() * 0.50)}));
  self:GetTargetBar():SetTitle("Wire's " .. self:GetApplication():GetName() .. " Targets");
  
  -- defaults
  self:ApplyTargetManagerDefaults();
  
  -- idle
  self:GetIdle().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Idle",
    function()
      self:Idle();
    end,
    true
  ));
  
  -- units changed
  table.insert(Event.Unit.Available, {
    function(value)
      self:UnitsChanged(value);
    end,
    self:GetApplication():GetName(),
    "Units Changed Handler"
  });
  
  -- persistence
  
  -- cast bar display
  self.castBarDisplay = Wired.Persistence.PersistentVariable(
    "CastBarDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.CastBarDisplay
  );
  
  -- pet display
  self.petDisplay = Wired.Persistence.PersistentVariable(
    "PetDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.PetDisplay
  );
  
  -- PvP display
  self.PvPDisplay = Wired.Persistence.PersistentVariable(
    "PvPDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.PvPDisplay
  );
  
  -- stacking
  self.stacking = Wired.Persistence.PersistentVariable(
    "Stacking",
     self:GetParent():GetBase(),
     TargetManager.Defaults.Stacking
  );
  
  -- target persistence
  self.targetPersistence = Wired.Persistence.PersistentVariable(
    "Targets", self:GetParent():GetBase()
  );
end

-- application
function TargetManager:GetApplication()
  return self.application;
end

function TargetManager:SetApplication(value)
  self.application = value;
end

-- parent
function TargetManager:GetParent()
  return self.parent;
end

function TargetManager:SetParent(value)
  self.parent = value;
end

-- idle
function TargetManager:GetIdle()
  return self.idle;
end

function TargetManager:SetIdle(value)
  self.idle = value;
end

-- target bar
function TargetManager:GetTargetBar()
  return self.targetBar;
end

function TargetManager:SetTargetBar(value)
  self.targetBar = value;
  
  -- register bar
  self:GetApplication():RegisterCascadeBar(value);
end

-- cast bar display
function TargetManager:GetCastBarDisplay()
  return self.castBarDisplay:GetValue();
end

function TargetManager:SetCastBarDisplay(value)
  self.castBarDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- pet display
function TargetManager:GetPetDisplay()
  return self.petDisplay:GetValue();
end

function TargetManager:SetPetDisplay(value)
  self.petDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- PvP display
function TargetManager:GetPvPDisplay()
  return self.PvPDisplay:GetValue();
end

function TargetManager:SetPvPDisplay(value)
  self.PvPDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- stacking
function TargetManager:GetStacking()
  return self.stacking:GetValue();
end

function TargetManager:SetStacking(value)
  self.stacking:SetValue(value);
  
  -- redraw gauges
  self:RedrawGauges();
end

-- target persistence
function TargetManager:GetTargetPersistence()
  if (type(self.targetPersistence:GetValue()) ~= "table") then
    self.targetPersistence:SetValue({});
  end
  return self.targetPersistence:GetValue();
end

function TargetManager:SetTargetPersistence(value)
  self.targetPersistence:SetValue(value);
end

-- gauges
function TargetManager:GetGauges()
  if (type(self.gauges) ~= "table") then
    self.gauges = {};
  end
  return self.gauges;
end

function TargetManager:SetGauges(value)
  self.gauges = value;
end

-- defaults
function TargetManager:ApplyTargetManagerDefaults()
  
end

-- resource
function TargetManager:GetResource(details)
  local result = {};
  
  -- caster
  if (
    (details.calling == "cleric") or (details.calling == "mage")
  ) then
    result = {
      name = "Mana",
      color = Wired.UI.Color.Gauge.Mana,
      maximum = details.manaMax,
      value = details.mana
    };
    
  -- rogue
  elseif (details.calling == "rogue") then
    result = {
      name = "Energy",
      color = Wired.UI.Color.Gauge.Power,
      maximum = details.energyMax,
      value = details.energy
    };
    
  -- warrior
  elseif (details.calling == "warrior") then
    result = {
      name = "Power",
      color = Wired.UI.Color.Gauge.Energy,
      maximum = 100,
      value = details.power
    };
  end
  
  return result;
end

-- update gauge
function TargetManager:UpdateGauge(gauge, details, unit)
  
  -- redraw
  local redraw = false;
  
  -- mark
  local mark = "";
  if (details.mark) then
    mark = tostring(details.mark);
  end
  if (mark ~= gauge:GetMark()) then
    gauge:SetMark(mark);
  end
  
  -- resource
  local resource = self:GetResource(details);
  
  -- resource exists
  if (resource.value) then
    
    -- name
    if (resource.name ~= gauge:GetResource():GetTitle()) then
      gauge:GetResource():SetTitle(resource.name);
    end
    
    -- color
    if (resource.color ~= gauge:GetResource():GetTint()) then
      gauge:GetResource():SetTint(resource.color);
    end
    
    -- scale
    if (resource.max ~= gauge:GetResource():GetScale():GetMaximum()) then
      gauge:GetResource():GetScale():SetMaximum(resource.maximum);
    end
    
    -- value
    if (resource.value ~= gauge:GetResource():GetValue()) then
      gauge:GetResource():SetValue(resource.value);
    end
    
    -- show
    if (not gauge:GetResource():GetVisible()) then
      gauge:GetResource():SetVisible(true);
      
      -- redraw
      redraw = true;
    end
    
  -- no resource
  else
    
    -- hide
    if (gauge:GetResource():GetVisible()) then
      gauge:GetResource():SetVisible(false);
      
      -- redraw
      redraw = true;
    end
  end
  
  -- health
  
  -- name
  if (details.name ~= gauge:GetHealth():GetTitle()) then
    gauge:GetHealth():SetTitle(details.name);
  end
  
  -- calling
  local color = Wired.UI.Color.White;
  if (details.calling) then
    color = Wired.UI.Color.Calling[details.calling];
  end
  if (color ~= gauge:GetHealth():GetFontColor()) then
    gauge:GetHealth():SetFontColor(color);
  end
  
  -- tagged
  local color = nil;
  if (details.tagged == "other") then
    color = Wired.UI.Color.Grey;
  end
  if (color ~= gauge:GetHealth():GetTint()) then
    gauge:GetHealth():SetTint(color);
  end
  
  -- scale
  if (details.healthMax ~= gauge:GetHealth():GetScale():GetMaximum()) then
    gauge:GetHealth():GetScale():SetMaximum(details.healthMax);
  end
  
  -- value
  if (details.health ~= gauge:GetHealth():GetValue()) then
    gauge:GetHealth():SetValue(details.health);
  end
  
  -- cast bar
  if (self:GetCastBarDisplay()) then
    
    -- cast details
    local castDetails = Inspect.Unit.Castbar(unit);
    
    -- check interrupted
    if (not castDetails) then
      
      -- was visible
      if (gauge:GetCast():GetVisible()) then
        
        -- not active
        if (not gauge:GetCast():GetFlashAnimation():GetActive()) then
          
          -- interrupted
          if (
            (gauge:GetCast():GetValue() > TargetManager.Defaults.Interrupt) and
            (gauge:GetCast():GetValue() < gauge:GetCast():GetScale():GetMaximum())
          ) then
            
            -- interrupted settings
            gauge:GetCast():SetFlipped(false);
            gauge:GetCast():SetTint(Wired.UI.Color.Red);
            gauge:GetCast():SetTitle(TargetManager.Constant.Message.Interrupted);
            gauge:GetCast():SetValue(gauge:GetCast():GetScale():GetMaximum());
            
            -- start animation
            gauge:GetCast():StartFlashAnimation();
            
          -- finished casting
          else
            
            -- hide
            gauge:GetCast():SetVisible(false);
            
            -- redraw
            redraw = true;
          end
        end
      end
      
    -- update cast bar
    else
      
      -- active
      if (gauge:GetCast():GetFlashAnimation():GetActive()) then
        
        -- stop animation
        gauge:GetCast():StopFlashAnimation();
        
        -- reset opacity
        gauge:GetCast():SetAlpha(gauge:GetHealth():GetAlpha());
      end
      
      -- name
      local abilityName = castDetails.abilityName;
      if (abilityName ~= gauge:GetCast():GetTitle()) then
        if (not abilityName) then
          abilityName = TargetManager.Constant.Message.NoAbilityName;
        end
        gauge:GetCast():SetTitle(string.upper(abilityName));
      end
      
      -- flip
      if (gauge:GetCast():GetFlipped() ~= (not castDetails.channeled)) then
        gauge:GetCast():SetFlipped(not castDetails.channeled);
      end
      
      -- color
      local color = Wired.UI.Color.Gauge.Cast;
      if (castDetails.uninterruptible) then
        color = Wired.UI.Color.Gauge.Uninterruptible;
      end
      if (color ~= gauge:GetCast():GetTint()) then
        gauge:GetCast():SetTint(color);
      end
      
      -- scale
      if (castDetails.duration ~= gauge:GetCast():GetScale():GetMaximum()) then
        gauge:GetCast():GetScale():SetMaximum(castDetails.duration);
      end
      
      -- value
      if (castDetails.remaining ~= gauge:GetCast():GetValue()) then
        gauge:GetCast():SetValue(castDetails.remaining);
      end
      
      -- show
      if (not gauge:GetCast():GetVisible()) then
        gauge:GetCast():SetVisible(true);
        
        -- redraw
        redraw = true;
      end
    end
    
  -- no cast bar
  else
    
    -- hide
    if (gauge:GetCast():GetVisible()) then
      gauge:GetCast():SetVisible(false);
      
      -- redraw
      redraw = true;
    end
  end
  
  -- redraw
  if (redraw) then
    
    -- redraw gauges
    self:RedrawGauges();
  end
end

-- add unit
function TargetManager:AddUnit(unit, details)
  
  -- check for recycled gauge
  local gauges = 1;
  local gauge = nil;
  for id, recycledGauge in pairs(self:GetGauges()) do
    gauges = gauges + 1;
    if ((not gauge) and (not recycledGauge:GetVisible())) then
      gauge = recycledGauge;
      self:GetGauges()[id] = nil;
    end
  end
  
  -- create new
  if (not gauge) then
    gauge = Wired.UI.TargetGauge(
      "Gauge_" .. gauges,
      self:GetParent(),
      self:GetApplication()
    );
    
    -- persistence
    gauge.Wired.Event.Listener:Add(Wired.Event.Listener(
      "LockedChanged",
      function(sender)
        self:LockedChanged(sender);
      end,
      true
    ));
    
    -- finished
    gauge:GetCast():GetFlashAnimation().Wired.Event.Listener:Add(Wired.Event.Listener(
      "AnimationFinished",
      function()
        if (gauge:GetCast():GetVisible()) then
          gauge:GetCast():SetVisible(false);
        end
        self:RedrawGauges();
      end,
      true
    ));
  end
  
  -- hide cast
  gauge:GetCast():SetVisible(false);
  
  -- reset acquired
  gauge:SetAcquired(Inspect.Time.Frame());
  
  -- update gauge
  self:UpdateGauge(gauge, details, unit);
  
  -- reindex
  self:GetGauges()[unit] = gauge;
  
  -- load gauge
  local persistence = self:GetTargetPersistence()[details.name];
  if (persistence) then
    
    -- size
    gauge:SetFontSize(persistence.size);
    
    -- offset
    gauge:SetPosition(persistence.left, persistence.top);
    
    -- lock
    gauge:SetLocked(true);
    
  -- unlock
  else
    gauge:SetLocked(false);
  end
  
  -- show
  gauge:SetVisible(true);
end

-- units changed
function TargetManager:UnitsChanged(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- redraw
    local redraw = false;
    
    -- units
    for unit, v in pairs(value) do
      
      -- details
      local details = Inspect.Unit.Detail(unit);
      
      -- gauge exists
      if (self:GetGauges()[unit]) then
        
        -- disable unacquired
        if (not details) then
          self:GetGauges()[unit]:SetVisible(false);
          
          -- redraw
          redraw = true;
          
        -- enable live reacquired
        elseif (details.health) then
          if ((not self:GetGauges()[unit]:GetVisible()) and (details.health > 0)) then
            
            -- hide cast
            self:GetGauges()[unit]:GetCast():SetVisible(false);
            
            -- reset acquired
            self:GetGauges()[unit]:SetAcquired(Inspect.Time.Frame());
            
            -- show
            self:GetGauges()[unit]:SetVisible(true);
            
            -- redraw
            redraw = true;
          end
        end
        
      -- create for valid targets
      elseif (self:ValidTarget(details)) then
        
        -- add unit
        self:AddUnit(unit, details);
        
        -- redraw
        redraw = true;
      end
    end
    
    -- redraw
    if (redraw) then
      self:RedrawGauges();
    end
  end
end

-- valid target
function TargetManager:ValidTarget(details)
  local result = false;
  
  -- details
  if (details) then
    
    -- alive hostile
    if ((details.relation == "hostile") and (details.health > 0)) then
      
      -- pet
      local showPet = true;
      if (details.nameSecondary) then
        if (string.match(details.nameSecondary, TargetManager.Constant.Message.Pet)) then
          showPet = self:GetPetDisplay();
        end
      end
      
      -- npc or not player if not pvp display or not pet if not pet display
      if ((self:GetPvPDisplay() or (not details.player)) and showPet) then
        result = true;
      end
    end
  end
  return result;
end

-- idle
function TargetManager:Idle()

  -- redraw
  local redraw = false;
  
  -- gauges
  for unit, gauge in pairs(self:GetGauges()) do
    
    -- active
    if (gauge:GetVisible()) then
      
      -- details
      local details = Inspect.Unit.Detail(unit);
      
      -- disable
      if (not details) then
        gauge:SetVisible(false);
        
        -- redraw
        redraw = true;
        
      -- disable invalid target mobs
      elseif (not self:ValidTarget(details)) then
        gauge:SetVisible(false);
        
        -- redraw
        redraw = true;
          
      -- update valid target mobs
      else
        self:UpdateGauge(gauge, details, unit);
      end
    end
  end
  
  -- redraw
  if (redraw) then
    self:RedrawGauges();
  end
end

function TargetManager:RedrawGauges()
  
  -- sorted
  local sorted = {};
  
  -- gauges
  for k, gauge in pairs(self:GetGauges()) do
    
    -- only sort visible and unlocked
    if (gauge:GetVisible() and (not gauge:GetLocked())) then
      table.insert(sorted, gauge);
    end
  end
  
  -- sort by acquired then name
  table.sort(
    sorted,
    function(a, b)
      local result = a:GetAcquired() < b:GetAcquired();
      if (a:GetAcquired() == b:GetAcquired()) then
        result = a:GetHealth():GetTitle() < b:GetHealth():GetTitle();
      end
      return result;
    end
  );
  
  -- top
  local top = self:GetTargetBar();
  
  -- gauges
  for k, gauge in pairs(sorted) do
    
    -- resize
    if (gauge:GetFontSize() ~= self:GetTargetBar():GetSize()) then
      gauge:SetFontSize(self:GetTargetBar():GetSize());
    end
    
    -- reposition
    if (self:GetStacking()) then
      gauge:SetPosition(
        self:GetTargetBar():GetLeft(),
        top:GetTop() - gauge:GetHeight() - TargetManager.Defaults.Spacing
      );
    else
      gauge:SetPosition(
        self:GetTargetBar():GetLeft(),
        top:GetBottom() + TargetManager.Defaults.Spacing
      );
    end
    
    -- top
    top = gauge;
  end
end

-- locked changed
function TargetManager:LockedChanged(value)
  if (value:GetLocked()) then
    self:GetTargetPersistence()[value:GetHealth():GetTitle()] = {
      size = value:GetFontSize(),
      left = value:GetLeft(),
      top = value:GetTop(),
    };
  else
    self:GetTargetPersistence()[value:GetHealth():GetTitle()] = nil;
  end
  
  -- redraw
  self:RedrawGauges();
end
