-----------------------
-- OCD               --
-- By: Wire @ Byriel --
-----------------------

OCDOptions = class(Wired.UI.Window);

-- constants
OCDOptions.Constants = {
  Texture = {
    Soulwalk = Wired.UI.Texture("Rift", "offhand_skull_007a.dds"),
    PlanarCharges = Wired.UI.Texture("OCD", "Resources/planar-charges.png"),
    Vitality = Wired.UI.Texture("OCD", "Resources/vitality.png"),
    Role = {
      dps = Wired.UI.Texture("Wired", "UI/Resources/Role/dps.png"),
      heal = Wired.UI.Texture("Wired", "UI/Resources/Role/heal.png"),
      support = Wired.UI.Texture("Wired", "UI/Resources/Role/support.png"),
      tank = Wired.UI.Texture("Wired", "UI/Resources/Role/tank.png")
    }
  },
  Cell = {
    Duration = 6,
    Available = 6,
    Genre = 7,
    Ability = 7,
    Player = 20,
    Global = 3,
    GlobalDuration = 3
  },
  Label = {
    Raid = OCD:T("UI.O.M.Raid"),
    Genre = OCD:T("UI.O.M.Genre"),
    GlobalCooldown = OCD:T("UI.O.M.Cooldowns"),
    LocalCooldown = OCD:T("UI.O.M.Cooldowns"),
    Ability = OCD:T("UI.O.M.Global"),
    Available = OCD:T("UI.O.M.Player"),
    Global = OCD:T("UI.O.M.Raid"),
    Refresh = OCD:T("UI.O.M.Refresh")
  }
};

-- defaults
OCDOptions.Defaults = {
  Size = {
    Icon = 30,
    Font = {
      Title = 18
    }
  },
  Width = {
    SelectionPane = 400
  },
  List = {
    CellSpacing = 2
  },
  Spacing = 5,
  Color = {
    Font = {
      Title = Wired.UI.Color.Trion.Title,
      Default = Wired.UI.Color.White
    }
  }
};

function OCDOptions:Constructor(name, parent)
  Wired.UI.Window.Constructor(self, name, parent);
  
  -- global abilities
  self.globalAbilities = Wired.Persistence.PersistentVariable(
    "GlobalAbilities", self:GetBar():GetBase(), nil
  );
  
  -- list pane
  self.listPane = Wired.UI.Trion.Frame("ListPane", self:GetBase());
  self.listPane:SetPoint(
    {child = "TOPCENTER", parent = "TOPCENTER"},
    {
      x = 0,
      y = OCDOptions.Defaults.Spacing + self:GetBorderSize().top
    }
  );
  self.listPane:SetColor(Wired.UI.Color.Clear);
  self.listPane:SetVisible(true);  
  
  -- lists
  -- walk list
  self.walkList = self:CreateList(
    "Walk",
    self.listPane,
    {child = "BOTTOMLEFT", parent = "BOTTOMLEFT"},
    {x = 0, y = 0},
    OCDOptions.Constants.Cell.Player,
    function(sender, value)
      self:Soulwalk(sender, value);
    end,
    44
  );
  
  -- charge list
  self.chargeList = self:CreateList(
    "Charge",
    self.walkList,
    {child = "BOTTOMLEFT", parent = "BOTTOMRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Player,
    nil,
    44
  );
  
  -- vitality list
  self.vitalityList = self:CreateList(
    "Vitality",
    self.chargeList,
    {child = "BOTTOMLEFT", parent = "BOTTOMRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Player,
    nil,
    44
  );
  
  -- player list
  self.playerList = self:CreateList(
    "Player",
    self.vitalityList,
    {child = "BOTTOMLEFT", parent = "BOTTOMRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Player,
    function(sender, value)
      self:ShowPlayer(sender, value);
    end
  );
  
  -- available list
  self.availableList = self:CreateList(
    "Available",
    self.playerList,
    {child = "TOPLEFT", parent = "TOPRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Available,
    function(sender, value)
      self:Available(sender, value);
    end
  );
  
  -- duration list
  self.durationList = self:CreateList(
    "Duration",
    self.availableList,
    {child = "TOPLEFT", parent = "TOPRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Duration,
    function(sender, value)
      self:Duration(sender, value);
    end
  );
  
  -- global list
  self.globalList = self:CreateList(
    "Global",
    self.availableList,
    {child = "TOPCENTER", parent = "BOTTOMCENTER"},
    {x = 0, y = OCDOptions.Defaults.Spacing + OCDOptions.Defaults.Size.Icon},
    OCDOptions.Constants.Cell.Global,
    function(sender, value)
      self:Global(sender, value);
    end
  );
  
  -- global duration list
  self.globalDurationList = self:CreateList(
    "GlobalDuration",
    self.globalList,
    {child = "TOPLEFT", parent = "TOPRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.GlobalDuration,
    function(sender, value)
      self:GlobalDuration(sender, value);
    end
  );
  
  -- genre list
  self.genreList = self:CreateList(
    "Genre",
    self.globalList,
    {child = "TOPCENTER", parent = "BOTTOMCENTER"},
    {x = 0, y = OCDOptions.Defaults.Spacing + OCDOptions.Defaults.Size.Icon},
    OCDOptions.Constants.Cell.Genre,
    function(sender, value)
      self:Genre(sender, value);
    end
  );
  
  -- ability list
  self.abilityList = self:CreateList(
    "Ability",
    self.genreList,
    {child = "TOPLEFT", parent = "TOPRIGHT"},
    {x = OCDOptions.Defaults.Spacing, y = 0},
    OCDOptions.Constants.Cell.Ability,
    function(sender, value)
      self:ShowAbility(sender, value);
    end
  );
  
  -- create genre cells
  self:CreateGenreCells();
  
  -- role icons
  self:CreateRoleIcons(self.playerList);
  
  -- create global cells
  self:CreateGlobalCells();

  -- set genre
  self:Genre(self.genreList:GetCells()[1], 1);
  
  -- set global
  self:Global(self.globalList:GetCells()[1], 1);
  
  -- icons
  self:CreateIcon(self.chargeList, OCDOptions.Constants.Texture.PlanarCharges);
  self:CreateIcon(self.walkList, OCDOptions.Constants.Texture.Soulwalk);
  self:CreateIcon(self.vitalityList, OCDOptions.Constants.Texture.Vitality);
  
  -- labels
  self:CreateLabel(self.playerList, OCDOptions.Constants.Label.Raid);
  self:CreateLabel(self.genreList, OCDOptions.Constants.Label.Genre);
  self:CreateLabel(self.globalDurationList, OCDOptions.Constants.Label.GlobalCooldown);
  self:CreateLabel(self.durationList, OCDOptions.Constants.Label.LocalCooldown);
  self:CreateLabel(self.availableList, OCDOptions.Constants.Label.Available);
  self:CreateLabel(self.abilityList, OCDOptions.Constants.Label.Ability);
  self:CreateLabel(self.globalList, OCDOptions.Constants.Label.Global);
  
  -- refresh
  self.refresh = Wired.UI.Trion.Button("Refresh", self:GetBase());
  self.refresh:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -(self:GetBorderSize().right + OCDOptions.Defaults.Spacing), y = (self:GetBorderSize().top + OCDOptions.Defaults.Spacing)});
  self.refresh:SetTitle(OCDOptions.Constants.Label.Refresh);
  self.refresh:SetColor(Wired.UI.Color.Clear);
  self.refresh:SetVisible(true);
  
  -- left click
  self.refresh:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      self:Refresh();
    end
  ));
  
  -- size changed
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function()
      self:LayoutOCDOptions();
    end
  ));
  
  -- visible changed
  self.Wired.Event.Listener:Add(Wired.Event.Listener(
    "VisibleChanged",
    function()
      self:VisibleChanged();
    end,
    true
  ));
  
  -- self
  self:SetWidth(685);
  self:SetHeight(585);
end

-- layout
function OCDOptions:LayoutOCDOptions()
  self.listPane:SetWidth(
    self:GetWidth() -
    self:GetBorderSize().left -
    self:GetBorderSize().right - 
    (OCDOptions.Defaults.Spacing * 2)
  );
  self.listPane:SetHeight(
    self:GetHeight() -
    self:GetBorderSize().top -
    self:GetBorderSize().bottom -
    (OCDOptions.Defaults.Spacing * 2)
  );
end

-- raid
function OCDOptions:GetRaid()
  if (not self.raid) then
    self.raid = Raid();
  end
  return self.raid;
end

function OCDOptions:SetRaid(value)
  self.raid = value;
end

-- player
function OCDOptions:GetPlayer()
  return self.player;
end

function OCDOptions:SetPlayer(value)
  self.player = value;
end

-- ability
function OCDOptions:GetAbility()
  return self.ability;
end

function OCDOptions:SetAbility(value)
  self.ability = value;
end

-- genre
function OCDOptions:GetGenre()
  return self.genre;
end

function OCDOptions:SetGenre(value)
  self.genre = value;
end

-- abilities
function OCDOptions:GetAbilities()
  if (type(self.abilities) ~= "table") then
    self.abilities = {};
  end
  return self.abilities;
end

function OCDOptions:SetAbilities(value)
  self.abilities = value;
end

-- global abilities
function OCDOptions:GetGlobalAbilities()
  if (type(self.globalAbilities:GetValue()) ~= "table") then
    self.globalAbilities:SetValue({});
  end
  return self.globalAbilities:GetValue();
end

-- available
function OCDOptions:GetAvailable()
  if (type(self.available) ~= "table") then
    self.available = {};
  end
  return self.available;
end

function OCDOptions:SetAvailable(value)
  self.available = value;
end

-- role icons
function OCDOptions:GetRoleIcons()
  return self.roleIcons;
end

function OCDOptions:SetRoleIcons(value)
  self.roleIcons = value;
end

-- create list
function OCDOptions:CreateList(name, parent, anchor, offset, size, action, width)
  local result = Wired.UI.List(name .. "_List", parent);
  result:SetPoint(anchor, offset);
  result:SetOrientation(Wired.UI.List.Constants.Orientation.Vertical);
  result:SetCellSpacing(OCDOptions.Defaults.List.CellSpacing);
  result:SetSize(size);
  result:SetColor(Wired.UI.Color.Black);
  
  -- width
  if (width) then
    result:SetWidth(width);
  end
  
  -- show
  result:SetVisible(true);
  
  -- populate
  result:Populate(action);
  
  -- return
  return result;
end

-- create icon
function OCDOptions:CreateIcon(list, texture)
  local icon = Wired.UI.Trion.Texture(list:GetName() .. "_Icon", list);
  icon:SetPoint({child = "BOTTOMCENTER", parent = "TOPCENTER"}, {x = 0, y = 0});
  icon:SetTexture(texture);
  icon:SetWidth(OCDOptions.Defaults.Size.Icon);
  icon:SetHeight(OCDOptions.Defaults.Size.Icon);
  icon:SetColor(Wired.UI.Color.Clear);
  icon:SetVisible(true);
  
  -- register
  list.icon = result;
end

-- create label
function OCDOptions:CreateLabel(list, title)
  local label = Wired.UI.Trion.Text(list:GetName() .. "_Label", list);
  label:SetPoint({child = "BOTTOMCENTER", parent = "TOPCENTER"}, {x = 0, y = 0});
  label:SetFontSize(OCDOptions.Defaults.Size.Font.Title);
  label:SetFontColor(OCDOptions.Defaults.Color.Font.Title);
  label:SetText(title);
  label:SetVisible(true);
  
  -- register
  list.label = label;
end

-- create role icon
function OCDOptions:CreateRoleIcons(list)
  
  -- role icons
  self:SetRoleIcons({});
  
  -- cells
  for i = 1, list:GetSize() do
    
    -- cell
    local cell = list:GetCells()[i];
    
    -- icon
    local icon = Wired.UI.Trion.Texture("RoleIcon" .. i, cell);
    icon:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = list:GetPadding(), y = 0});
    icon:SetWidth(cell:GetHeight() - (list:GetPadding() * 2));
    icon:SetHeight(cell:GetHeight() - (list:GetPadding() * 2));
    icon:SetColor(Wired.UI.Color.Clear);
    icon:SetLayer(2);
    icon:SetVisible(false);
    
    -- save
    self:GetRoleIcons()[i] = icon;
  end
end

-- create global cells
function OCDOptions:CreateGlobalCells()
  
  -- cells
  for k, ability in pairs(CooldownManager.Constants.Abilities.Global) do
    
    -- localize variables
    local cell = self.globalList:GetCells()[k];
    
    -- title
    cell:SetTitle(ability:GetName());
    
    -- show
    cell:SetVisible(true);
  end
end

-- create genre cells
function OCDOptions:CreateGenreCells()
  
  -- cells
  for i = 1, OCDOptions.Constants.Cell.Genre do
    
    -- localize variables
    local cell = self.genreList:GetCells()[i];
    
    -- title
    local title = nil
    for k, v in pairs(LocalAbility.Constants.Genre) do
      if (v == i) then
        title = k;
      end
    end
    
    -- title
    if (title) then
      cell:SetTitle(title);
              
      --show
      cell:SetVisible(true);
      
    -- no ability
    else
      cell:SetVisible(false);
      cell:SetActive(false);
    end
  end
end

-- refresh
function OCDOptions:Refresh()
  
  -- refresh
  self:GetRaid():RefreshPlayers();
  
  -- players
  local players = self:GetRaid():GetPlayersBySlot();
  for i = 1, (Raid.Constants.Groups * Raid.Constants.GroupSize) do
    local cell = self.playerList:GetCells()[i];
    local chargeCell = self.chargeList:GetCells()[i];
    local walkCell = self.walkList:GetCells()[i];
    local vitalityCell = self.vitalityList:GetCells()[i];
    local player = players[i];
    if (player) then
      cell:SetTitle(player:GetName());
      cell:SetVisible(true);
      
      -- color
      local color = Wired.UI.Color.Trion.Disconnected;
      if (player:GetCalling()) then
        color = Wired.UI.Color.Calling[player:GetCalling()];
      end
      cell:SetOnColor(color);
      
      -- charge cell
      if (player:GetPlanarCharges()) then
        local charges = tostring(player:GetPlanarCharges());
        if (player:GetPlanarChargesMax()) then
          charges = charges .. " / " .. player:GetPlanarChargesMax();
        end
        chargeCell:SetTitle(charges);
        chargeCell:SetVisible(true);
      else
        chargeCell:SetVisible(false);
      end
      
      -- vitality cell
      if (player:GetVitality()) then
        vitalityCell:SetTitle(tostring(player:GetVitality() .. " %"));
        vitalityCell:SetVisible(true);
      else
        vitalityCell:SetVisible(false);
      end
      
      -- role
      local roleIcon = self:GetRoleIcons()[i];
      if (player:GetRole()) then
        roleIcon:SetTexture(OCDOptions.Constants.Texture.Role[player:GetRole()]);
        roleIcon:SetVisible(true);
      else
        roleIcon:SetVisible(false);
      end
      
      -- soul walk
      local cd = 0;
      local ability = CooldownManager.Constants.Abilities.Global[1];
      
      -- cooldown
      local cooldown = ability:GetCooldown()[1];
      if (not self:GetGlobalAbilities()[ability:GetName()]) then
        self:GetGlobalAbilities()[ability:GetName()] = {};
      end
      if (self:GetGlobalAbilities()[ability:GetName()].duration) then
        cooldown = self:GetGlobalAbilities()[ability:GetName()].duration;
      end
      
      -- player
      if (not player:GetAbilities()[ability:GetName()]) then
        player:GetAbilities()[ability:GetName()] = {};
      end
      
      -- time
      local time = player:GetAbilities()[ability:GetName()].time;
      
      -- cd
      if (time) then
        cd = math.floor(0.5 + math.max(
          0,
          cooldown - (Inspect.Time.Frame() - time)
        ));
      end
      
      -- walk cell
      walkCell:SetTitle(tostring(cd));
      walkCell:SetVisible(true);
    else
      cell:SetVisible(false);
      
      -- charge cell
      chargeCell:SetVisible(false);
      
      -- vitality cell
      vitalityCell:SetVisible(false);
      
      -- walk cell
      walkCell:SetVisible(false);
    end
  end
end

-- show player
function OCDOptions:ShowPlayer(cell, slot)
  
  -- player
  self:SetPlayer(self:GetRaid():GetPlayersBySlot()[slot]);
  if (self:GetPlayer()) then
    
    -- select
    self.playerList:Select(self.playerList:GetCells()[slot]);
    
    -- available
    self:SetAvailable();
    
    -- available
    local slot = 0;
    
    -- abilities
    for k, ability in pairs(CooldownManager.Constants.Abilities.Local) do
      
      -- enabled
      if (ability:GetEnabled()) then
      
        -- filter by calling
        if (ability:GetCalling() == self:GetPlayer():GetCalling()) then
          
          -- update slot
          slot = slot + 1;
          
          -- available
          self:GetAvailable()[slot] = ability;
        end
      end
    end
  end
    
  -- cells
  for i = 1, OCDOptions.Constants.Cell.Available do
    
    -- localize variables
    local cell = self.availableList:GetCells()[i];
    local ability = self:GetAvailable()[i];
    local active = false;
    
    -- exists
    if (ability and self:GetPlayer()) then
      
      -- active
      if (OCD.cooldownManager:GetTimer(ability, self:GetPlayer())) then
        if (OCD.cooldownManager:GetTimer(ability, self:GetPlayer()):GetVisible()) then
          active = true;
        end
      end
      
      -- show
      cell:SetTitle(ability:GetName());
      cell:SetVisible(true);
      
    -- no ability
    else
      cell:SetVisible(false);
    end
    
    -- active
    cell:SetActive(active);
  end
  
  -- reset available
  self:Available();
end

-- available
function OCDOptions:Available(cell, value)
  
  -- toggle active
  if (cell) then
    cell:SetActive(not cell:GetActive());
  end
  
  -- ability
  local ability = self:GetAvailable()[value];
  
  self:SetAbility(ability);
  
  -- exists
  if (self:GetPlayer() and ability) then
    if (not self:GetPlayer():GetAbilities()[ability:GetName()]) then
      self:GetPlayer():GetAbilities()[ability:GetName()] = {};
    end
    self:GetPlayer():GetAbilities()[ability:GetName()].enabled = self.availableList:GetCells()[value]:GetActive();
    
    -- toggle ability
    self:ToggleAbility(ability, self:GetPlayer());
  end
  
  -- show duration
  for i = 1, OCDOptions.Constants.Cell.Duration do
    
    -- cell
    local cell = self.durationList:GetCells()[i];
    
    -- duration
    local duration = nil;
    if (self:GetPlayer() and ability) then
      duration = ability:GetCooldown()[i];
    end
    
    -- exists
    if (duration) then
      
      -- active
      if (not self:GetPlayer():GetAbilities()[ability:GetName()].duration) then
        self:GetPlayer():GetAbilities()[ability:GetName()].duration = duration;
      end
      if (self:GetPlayer():GetAbilities()[ability:GetName()].duration == duration) then
        self.durationList:Select(cell);
      end
      
      -- cell
      cell:SetTitle(tostring(duration));
      cell:SetVisible(true);
      
    -- no duration
    else
      cell:SetVisible(false);
      cell:SetActive(false);
    end
  end
end

-- duration
function OCDOptions:Duration(cell, value)
  
  -- select
  self.durationList:Select(cell);
  
  -- set duration
  if (self:GetAbility()) then
    self:GetPlayer():GetAbilities()[self:GetAbility():GetName()].duration = self:GetAbility():GetCooldown()[value];
  end
end

-- genre
function OCDOptions:Genre(cell, value)
  
  -- select
  self.genreList:Select(cell);
  
  -- genre
  self:SetGenre(value);
    
  -- abilities
  local slot = 0;
  
  -- reset abilities
  self:SetAbilities({});
  
  -- abilities
  for k, ability in pairs(CooldownManager.Constants.Abilities.Local) do
    
    -- filter by calling
    if (ability:GetGenre() == self:GetGenre()) then
      
      -- update slot
      slot = slot + 1;
      
      -- abilities
      self:GetAbilities()[slot] = ability;
    end
  end
  
  -- cells
  for i = 1, OCDOptions.Constants.Cell.Ability do
    
    -- localize variables
    local cell = self.abilityList:GetCells()[i];
    local ability = self:GetAbilities()[i];
    
    -- ability
    if (ability) then
      cell:SetTitle(ability:GetName());
      
      -- active
      cell:SetActive(ability:GetEnabled());
      
      --show
      cell:SetVisible(true);
      
    -- no ability
    else
      cell:SetVisible(false);
      cell:SetActive(false);
    end
  end
end

-- show ability
function OCDOptions:ShowAbility(cell, value);
  
  -- ability
  local ability = self:GetAbilities()[value];
  
  -- toggle active
  cell:SetActive(not cell:GetActive());
  
  -- toggle enabled
  ability:SetEnabled(cell:GetActive());
  
  -- not active
  if (not cell:GetActive()) then
    
    -- update display
    cell:SetMouseOver(false);
    
    -- hide timers
    for abilityName, players in pairs(OCD.cooldownManager:GetTimersByAbility()) do
      if (abilityName == ability:GetName()) then
        for playerName, timer in pairs(players) do
          
          -- hide timer
          timer:SetActive(false);
          timer:SetVisible(false);
          
          -- redraw timers
          OCD.cooldownManager:RedrawTimers();
        end
      end
    end
  end
  
  -- reset player
  self:ShowPlayer();
end

-- toggle ability
function OCDOptions:ToggleAbility(ability, player)
  
  -- timer exists
  local found = false;
  local timer = OCD.cooldownManager:GetTimer(ability, player);
  if (timer) then
    if (timer:GetVisible()) then
      
      -- found
      found = true;
      
      -- disable
      timer:SetActive(false);
      timer:SetVisible(false);
      
      -- redraw timers
      OCD.cooldownManager:RedrawTimers();
    end
  end
  
  -- create
  if (not found) then
  
    -- create timer
    OCD.cooldownManager:StartTimer(ability, player);
    timer = OCD.cooldownManager:GetTimer(ability, player);
    
    -- push timer to end and stop
    timer:SetValue(0);
    timer:SetActive(false);
    
    -- redraw
    OCD.cooldownManager:RedrawTimers();
  end
end

-- soulwalk
function OCDOptions:Soulwalk(cell, value)

  -- player
  local player = self:GetRaid():GetPlayersBySlot()[value];
  if (player) then
    
    -- ability
    local ability = CooldownManager.Constants.Abilities.Global[1];
    if (not player:GetAbilities()[ability:GetName()]) then
      player:GetAbilities()[ability:GetName()] = {};
    end
    
    -- duration
    local cooldown = ability:GetCooldown()[1];
    if (self:GetGlobalAbilities()[ability:GetName()].duration) then
      cooldown = self:GetGlobalAbilities()[ability:GetName()].duration;
    end
    
    local time = player:GetAbilities()[ability:GetName()].time;
    local current = Inspect.Time.Frame();
    if (time) then
      if ((current - time) < cooldown) then
        player:GetAbilities()[ability:GetName()].time = current - cooldown;
      else
        player:GetAbilities()[ability:GetName()].time = current;
      end
    else
      player:GetAbilities()[ability:GetName()].time = current;
    end
    
    -- refresh
    self:Refresh();
  end
end

-- global
function OCDOptions:GetGlobal()
  return self.global;
end

function OCDOptions:SetGlobal(value)
  self.global = value;
end

-- global
function OCDOptions:Global(cell, value)
          
  -- toggle active
  cell:SetActive(not cell:GetActive());
  
  -- global
  self:SetGlobal(value);
  
  -- ability
  local ability = CooldownManager.Constants.Abilities.Global[value];
  
  -- exists
  if (ability and cell) then
  
    -- toggle active
    self.globalList:Select(cell);
    
    -- duration
    local globalDuration = ability:GetCooldown()[1];
    if (not self:GetGlobalAbilities()[ability:GetName()]) then
      self:GetGlobalAbilities()[ability:GetName()] = {};
    end
    if (self:GetGlobalAbilities()[ability:GetName()].duration) then
      globalDuration = self:GetGlobalAbilities()[ability:GetName()].duration;
    end
    
    -- show duration
    for i = 1, OCDOptions.Constants.Cell.GlobalDuration do
      local duration = ability:GetCooldown()[i];
      local cell = self.globalDurationList:GetCells()[i];
      
      -- duration
      if (duration) then
        cell:SetTitle(tostring(duration));
        cell:SetVisible(true);
        
        -- match duration
        if (duration == globalDuration) then
          self.globalDurationList:Select(cell);
        end
        
      -- no duration
      else
        cell:SetVisible(false);
        cell:SetActive(false);
      end
    end
  end
end

-- global duration
function OCDOptions:GlobalDuration(cell, value)
  
  -- select
  self.globalDurationList:Select(cell);
  
  -- global
  if (self:GetGlobal()) then
  
    -- ability
    local ability = CooldownManager.Constants.Abilities.Global[self:GetGlobal()];
    if (ability) then
      
      -- active
      if (not self:GetGlobalAbilities()[ability:GetName()]) then
        self:GetGlobalAbilities()[ability:GetName()] = {};
      end
      self:GetGlobalAbilities()[ability:GetName()].duration = ability:GetCooldown()[value];
    end
  end
end

-- visible changed
function OCDOptions:VisibleChanged()
  
  -- refresh
  if (self:GetVisible()) then
    self:Refresh();
  end
end
