-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

-- class
RaidKiller.Options.Tab.Mechanics = class(Wired.UI.Tab.Frame);

-- constants
RaidKiller.Options.Tab.Mechanics.Constants = {
  Name = "Mechanics Tab",
  Profile = {
    Ratio = 180 / 320,
    Path = "Resources/Profile/",
    Extension = ".jpg",
    Zone = "Zone",
    Encounter = "Encounter"
  },
  List = {
    Size = 30
  },
  Message = {
    Tested = RK:T("UI.O.M.Tested"),
    Untested = RK:T("UI.O.M.Untested"),
    Enable = RK:T("UI.O.M.Enable"),
    Disable = RK:T("UI.O.M.Disable"),
    Trigger = RK:T("UI.O.M.Trigger"),
    Edit = RK:T("UI.O.M.Edit"),
    Label = {
      Players = RK:T("UI.O.M.L.Players"),
      Difficulty = RK:T("UI.O.M.L.Difficulty"),
      Zone = RK:T("UI.O.M.L.Zone"),
      Encounter = RK:T("UI.O.M.L.Encounter"),
      Mechanics = RK:T("UI.O.M.L.Mechanics"),
      Profile = RK:T("UI.O.M.L.Profile"),
      Details = RK:T("UI.O.M.L.Details"),
      Objectives = RK:T("UI.O.M.L.Objectives"),
      History = RK:T("UI.O.M.L.History")
    },
    Component = {
      Hitpoints = RK:T("UI.O.M.C.Hitpoints"),
      Enrage = RK:T("UI.O.M.C.Enrage"),
      NoDetails = RK:T("UI.O.M.C.NoDetails"),
      NoObjectives = RK:T("UI.O.M.C.NoObjectives"),
      NoHistory = RK:T("UI.O.M.C.NoHistory"),
      Difficulty = RK:T("UI.O.M.C.Difficulty"),
      Attempts = RK:T("UI.O.M.C.Attempts"),
      Completions = RK:T("UI.O.M.C.Completions"),
      LastAttempt = RK:T("UI.O.M.C.LastAttempt"),
      BestAttempt = RK:T("UI.O.M.C.BestAttempt"),
      LastCompletion = RK:T("UI.O.M.C.LastCompletion"),
      BestCompletion = RK:T("UI.O.M.C.BestCompletion")
    }
  }
};

-- defaults
RaidKiller.Options.Tab.Mechanics.Defaults = {
  Title = RK:T("UI.O.T.Mechanics"),
  Color = {
    Tested = Wired.UI.Color.Green,
    Untested = Wired.UI.Color.Red
  },
  Profile = {
    Font = {
      Color = Wired.UI.Color.White,
      Size = {
        Zone = 48,
        Encounter = 18,
        Mechanic = {
          Label = 18,
          Description = 14
        }
      },
      Alpha = 0.75
    }
  }
};

-- constructor
function RaidKiller.Options.Tab.Mechanics:Constructor(parent, application)
  Wired.UI.Tab.Frame.Constructor(self, RaidKiller.Options.Tab.Mechanics.Constants.Name, parent:GetFrame());
  
  -- parameters
  self:SetApplication(application);
  
  -- values
  self:SetTitle(RaidKiller.Options.Tab.Mechanics.Defaults.Title);
  self:SetPadding(0);
  self:SetColor(Wired.UI.Color.Clear);
  
  -- zone profile pane
  self.zoneProfilePane = Wired.UI.Trion.Frame("ZoneProfilePane", self);
  self.zoneProfilePane:SetColor(Wired.UI.Color.Clear);
  self.zoneProfilePane:SetLayer(1);
  self.zoneProfilePane:SetVisible(true);
    
  -- zone profile left
  self.zoneProfileLeft = Wired.UI.Trion.Texture("ZoneProfileLeft", self.zoneProfilePane);
  self.zoneProfileLeft:SetLayer(2);
  self.zoneProfileLeft:SetVisible(true);
  
  -- zone profile center
  self.zoneProfileCenter = Wired.UI.Trion.Texture("ZoneProfileCenter", self.zoneProfilePane);
  self.zoneProfileCenter:SetLayer(2);
  self.zoneProfileCenter:SetVisible(true);
  
  -- zone profile right
  self.zoneProfileRight = Wired.UI.Trion.Texture("ZoneProfileRight", self.zoneProfilePane);
  self.zoneProfileRight:SetLayer(2);
  self.zoneProfileRight:SetVisible(true);
  
  -- zone profile label
  self.zoneProfileLabel = Wired.UI.Trion.Text("ZoneProfileLabel", self.zoneProfilePane);
  self.zoneProfileLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.zoneProfileLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Zone);
  self.zoneProfileLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.zoneProfileLabel:SetLayer(3);
  self.zoneProfileLabel:SetVisible(true);
  
  -- encounter profile label
  self.encounterProfileLabel = Wired.UI.Trion.Text("EncounterProfileLabel", self);
  self.encounterProfileLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.encounterProfileLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Encounter);
  self.encounterProfileLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.encounterProfileLabel:SetLayer(1);
  self.encounterProfileLabel:SetText("Encounter");
  self.encounterProfileLabel:SetVisible(true);
  
  -- encounter profile
  self.encounterProfile = Wired.UI.Trion.Texture("EncounterProfile", self);
  self.encounterProfile:SetLayer(1);
  self.encounterProfile:SetVisible(true);
  
  -- encounter tab list
  self.encounterTabList = Wired.UI.Tab.List("EncounterTabList", self);
  self.encounterTabList:SetColor(Wired.UI.Color.Clear);
  self.encounterTabList:SetLayer(2);
  self.encounterTabList:SetPadding(0);
  self.encounterTabList:SetVisible(true);
  
  -- encounter profile pane
  self.encounterProfilePane = Wired.UI.Tab.Frame("EncounterProfilePane", self.encounterTabList:GetFrame());
  self.encounterProfilePane:SetColor(Wired.UI.Color.Clear);
  self.encounterProfilePane:SetLayer(1);
  self.encounterProfilePane:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Profile);
  self.encounterProfilePane:SetVisible(false);
  
  -- add tab
  self.encounterTabList:AddTab(self.encounterProfilePane);
  
  -- encounter details pane
  self.encounterDetailsPane = Wired.UI.Tab.Frame("EncounterDetailsPane", self.encounterTabList:GetFrame());
  self.encounterDetailsPane:SetColor(Wired.UI.Color.Clear);
  self.encounterDetailsPane:SetLayer(1);
  self.encounterDetailsPane:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Details);
  self.encounterDetailsPane:SetVisible(false);
  
  -- add tab
  self.encounterTabList:AddTab(self.encounterDetailsPane);
  
  -- encounter details label
  self.encounterDetailsLabel = Wired.UI.Trion.Text("EncounterDetailsLabel", self.encounterDetailsPane);
  self.encounterDetailsLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.encounterDetailsLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Description);
  self.encounterDetailsLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.encounterDetailsLabel:SetLayer(1);
  self.encounterDetailsLabel:SetText("Details");
  self.encounterDetailsLabel:SetColor(Wired.UI.Color.Black);
  self.encounterDetailsLabel:SetVisible(true);
  
  -- encounter objectives pane
  self.encounterObjectivesPane = Wired.UI.Tab.ScrollableFrame("EncounterObjectivesPane", self.encounterTabList:GetFrame());
  self.encounterObjectivesPane:SetColor(Wired.UI.Color.Clear);
  self.encounterObjectivesPane:SetLayer(1);
  self.encounterObjectivesPane:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Objectives);
  self.encounterObjectivesPane:SetVisible(false);
  
  -- add tab
  self.encounterTabList:AddTab(self.encounterObjectivesPane);
  
  -- encounter objectives label
  self.encounterObjectivesLabel = Wired.UI.Trion.Text("EncounterObjectivesLabel", self.encounterObjectivesPane:GetMask());
  self.encounterObjectivesLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.encounterObjectivesLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Description);
  self.encounterObjectivesLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.encounterObjectivesLabel:SetLayer(1);
  self.encounterObjectivesLabel:SetText("Objectives");
  self.encounterObjectivesLabel:SetColor(Wired.UI.Color.Black);
  self.encounterObjectivesLabel:SetVisible(true);
  
  -- add label
  self.encounterObjectivesPane:SetComponent(self.encounterObjectivesLabel);
  
  -- encounter history pane
  self.encounterHistoryPane = Wired.UI.Tab.Frame("EncounterHistoryPane", self.encounterTabList:GetFrame());
  self.encounterHistoryPane:SetColor(Wired.UI.Color.Clear);
  self.encounterHistoryPane:SetLayer(1);
  self.encounterHistoryPane:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.History);
  self.encounterHistoryPane:SetVisible(false);
  
  -- add tab
  self.encounterTabList:AddTab(self.encounterHistoryPane);
  
  -- encounter history label
  self.encounterHistoryLabel = Wired.UI.Trion.Text("EncounterHistoryLabel", self.encounterHistoryPane);
  self.encounterHistoryLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.encounterHistoryLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Description);
  self.encounterHistoryLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.encounterHistoryLabel:SetLayer(1);
  self.encounterHistoryLabel:SetText("History");
  self.encounterHistoryLabel:SetColor(Wired.UI.Color.Black);
  self.encounterHistoryLabel:SetVisible(true);
  
  -- mechanic list
  self.mechanicList = Wired.UI.Scroll.List("MechanicList", self);
  self.mechanicList:SetLayer(2);
  self.mechanicList:SetDisplaySize(9);
  self.mechanicList:SetVisible(true);
  
  -- mechanic list label
  self.mechanicListLabel = Wired.UI.Trion.Text("MechanicListLabel", self);
  self.mechanicListLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.mechanicListLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.mechanicListLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.mechanicListLabel:SetText(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Mechanics);
  self.mechanicListLabel:SetLayer(1);
  self.mechanicListLabel:SetVisible(true);
  
  -- selection pane
  self.selectionPane = Wired.UI.Trion.Frame("SelectionPane", self);
  self.selectionPane:SetColor(Wired.UI.Color.Clear);
  self.selectionPane:SetLayer(3);
  self.selectionPane:SetVisible(true);
  
  -- player drop list
  self.playerDropList = Wired.UI.Scroll.DropList("PlayerDropList", self.selectionPane);
  self.playerDropList:SetLayer(5);
  self.playerDropList:SetWidth(184);
  self.playerDropList:SetHeight(24);
  self.playerDropList:SetVisible(true);
  
  -- player drop label
  self.playerDropLabel = Wired.UI.Trion.Text("PlayerDropLabel", self.selectionPane);
  self.playerDropLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.playerDropLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.playerDropLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.playerDropLabel:SetHeight(24);
  self.playerDropLabel:SetText(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Players);
  self.playerDropLabel:SetLayer(3);
  self.playerDropLabel:SetVisible(true);
  
  -- difficulty drop list
  self.difficultyDropList = Wired.UI.Scroll.DropList("DifficultyDropList", self.selectionPane);
  self.difficultyDropList:SetLayer(5);
  self.difficultyDropList:SetWidth(184);
  self.difficultyDropList:SetHeight(24);
  self.difficultyDropList:SetVisible(true);
  
  -- difficulty drop label
  self.difficultyDropLabel = Wired.UI.Trion.Text("DifficultyDropLabel", self.selectionPane);
  self.difficultyDropLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.difficultyDropLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.difficultyDropLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.difficultyDropLabel:SetHeight(24);
  self.difficultyDropLabel:SetText(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Difficulty);
  self.difficultyDropLabel:SetLayer(3);
  self.difficultyDropLabel:SetVisible(true);
  
  -- zone drop list
  self.zoneDropList = Wired.UI.Scroll.DropList("ZoneDropList", self.selectionPane);
  self.zoneDropList:SetLayer(4);
  self.zoneDropList:SetWidth(184);
  self.zoneDropList:SetHeight(24);
  self.zoneDropList:SetVisible(true);
  
  -- zone drop label
  self.zoneDropLabel = Wired.UI.Trion.Text("ZoneDropLabel", self.selectionPane);
  self.zoneDropLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.zoneDropLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.zoneDropLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.zoneDropLabel:SetHeight(24);
  self.zoneDropLabel:SetText(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Zone);
  self.zoneDropLabel:SetLayer(3);
  self.zoneDropLabel:SetVisible(true);
  
  -- encounter drop list
  self.encounterDropList = Wired.UI.Scroll.DropList("EncounterDropList", self.selectionPane);
  self.encounterDropList:SetLayer(4);
  self.encounterDropList:SetWidth(184);
  self.encounterDropList:SetHeight(24);
  self.encounterDropList:SetVisible(true);
  
  -- encounter drop label
  self.encounterDropLabel = Wired.UI.Trion.Text("EncounterDropLabel", self.selectionPane);
  self.encounterDropLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.encounterDropLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.encounterDropLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.encounterDropLabel:SetHeight(24);
  self.encounterDropLabel:SetText(RaidKiller.Options.Tab.Mechanics.Constants.Message.Label.Encounter);
  self.encounterDropLabel:SetLayer(3);
  self.encounterDropLabel:SetVisible(true);
  
  -- mechanic pane
  self.mechanicPane = Wired.UI.Trion.Frame("MechanicPane", self);
  self.mechanicPane:SetColor(Wired.UI.Color.Clear);
  self.mechanicPane:SetLayer(1);
  self.mechanicPane:SetVisible(true);
  
  -- mechanic drop
  self.mechanicDrop = Wired.UI.Trion.Frame("MechanicDrop", self);
  self.mechanicDrop:SetColor(Wired.UI.Color.Black);
  self.mechanicDrop:SetLayer(0);
  self.mechanicDrop:SetAlpha(0.50);
  self.mechanicDrop:SetVisible(true);
  
  -- mechanic label
  self.mechanicLabel = Wired.UI.Trion.Text("MechanicLabel", self.mechanicPane);
  self.mechanicLabel:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.mechanicLabel:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Label);
  self.mechanicLabel:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.mechanicLabel:SetLayer(2);
  self.mechanicLabel:SetText("Qy");
  self.mechanicLabel:SetVisible(true);
  
  -- mechanic description
  self.mechanicDescription = Wired.UI.Trion.Text("MechanicDescription", self.mechanicPane);
  self.mechanicDescription:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.mechanicDescription:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Description);
  self.mechanicDescription:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.mechanicDescription:SetWrapping(true);
  self.mechanicDescription:SetLayer(2);
  self.mechanicDescription:SetVisible(true);
  
  -- mechanic tested
  self.mechanicTested = Wired.UI.Trion.Text("MechanicTested", self.mechanicPane);
  self.mechanicTested:SetFontColor(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Color);
  self.mechanicTested:SetFontSize(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Size.Mechanic.Description);
  self.mechanicTested:SetAlpha(RaidKiller.Options.Tab.Mechanics.Defaults.Profile.Font.Alpha);
  self.mechanicTested:SetLayer(2);
  self.mechanicTested:SetVisible(true);
  
  -- mechanic trigger
  self.mechanicTrigger = Wired.UI.Trion.Button("MechanicTrigger", self.mechanicPane);
  self.mechanicTrigger:SetColor(Wired.UI.Color.Clear);
  self.mechanicTrigger:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Trigger);
  self.mechanicTrigger:SetLayer(2);
  self.mechanicTrigger:SetVisible(true);
  
  -- action pane
  self.actionPane = Wired.UI.Trion.Frame("ActionPane", self.mechanicPane);
  self.actionPane:SetColor(Wired.UI.Color.Clear);
  self.actionPane:SetLayer(2);
  self.actionPane:SetVisible(true);
  
  -- action drop list
  self.actionDropList = Wired.UI.Scroll.DropList("ActionDropList", self.actionPane);
  self.actionDropList:SetLayer(3);
  self.actionDropList:SetVisible(true);
  
  -- action edit
  self.actionEdit = Wired.UI.Trion.Button("ActionEdit", self.actionPane);
  self.actionEdit:SetColor(Wired.UI.Color.Clear);
  self.actionEdit:SetTitle(RaidKiller.Options.Tab.Mechanics.Constants.Message.Edit);
  self.actionEdit:SetLayer(3);
  self.actionEdit:SetVisible(false);
  
  -- action color picker
  self.actionColor = Wired.UI.ColorPicker.Picker("ActionColor", self.actionPane);
  self.actionColor:SetWindow(self:GetColorPickerWindow());
  self.actionColor:SetVisible(true);
  
  -- action enable
  self.actionEnable = Wired.UI.Trion.Button("ActionEnable", self.actionPane);
  self.actionEnable:SetColor(Wired.UI.Color.Clear);
  self.actionEnable:SetLayer(3);
  self.actionEnable:SetVisible(true);
  
  -- link lists
  self.zoneDropList.nextList = self.encounterDropList;
  self.encounterDropList.nextList = self.mechanicList;
  self.mechanicList.nextList = self.actionDropList;
  
  -- layout
  self:LayoutMechanicsTab();
  
  -- events
  
  -- players changed
  self.playerDropList.Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- data
      local data = self:FilterZones();
      
      -- update
      self:UpdateSelection(self.zoneDropList, data, self.zoneDropList:GetList():GetList());
    end,
    true
  ));
  
  -- difficulty changed
  self.difficultyDropList.Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- data
      local data = self:FilterZones();
      
      -- update
      self:UpdateSelection(self.zoneDropList, data, self.zoneDropList:GetList():GetList());
    end,
    true
  ));
  
  -- zone changed
  self.zoneDropList.Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- difficulty
      local difficulty = nil;
      if (self.difficultyDropList:GetSelected()) then
        difficulty = self.difficultyDropList:GetSelected():GetTitle();
      end
      
      -- zone
      local zone = self.zoneDropList:GetSelected().category;
      
      -- encounters
      local encounters = {};
      if (zone) then
        
        -- difficulties
        local difficulties = zone:GetDifficulties();
        local d = difficulties[difficulty];
        if (d) then
          encounters = d:GetEncounters();
        end
      end
      
      -- update
      self:UpdateSelection(self.encounterDropList, encounters, self.encounterDropList:GetList():GetList());
      
      -- update zone
      self:UpdateZoneProfile(self.zoneDropList:GetSelected().category);
    end,
    true
  ));
  
  -- encounter changed
  self.encounterDropList.Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- encounter
      local encounter = self.encounterDropList:GetSelected().category;
      
      -- mechanics
      local mechanics = {};
      if (encounter) then
        mechanics = encounter:GetMechanics();
      end
      
      -- update
      self:UpdateSelection(self.mechanicList, mechanics, self.mechanicList:GetList());
      
      -- update encounter
      self:UpdateEncounterProfile(self.encounterDropList:GetSelected().category);
    end,
    true
  ));
  
  -- mechanic changed
  self.mechanicList:GetList().Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- data
      local data = {};
      if (self.mechanicList:GetSelected().category) then
        data = self.mechanicList:GetSelected().category:GetActions();
      end
      
      -- index actions
      data = self:IndexActions(data);
      
      -- update
      self:UpdateSelection(self.actionDropList, data, self.actionDropList:GetList():GetList());
      
      -- update mechanic
      self:UpdateMechanic();
    end,
    true
  ));
  
  -- mechanic changed
  self.actionDropList.Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      
      -- update action
      self:UpdateAction();
    end,
    true
  ));
  
  -- mechanic triggered
  self.mechanicTrigger:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      
      -- enabled
      if (self.mechanicTrigger:GetEnabled()) then
        
        -- trigger mechanic
        self:TriggerMechanic();
      end
    end
  ));
  
  -- action enabled
  self.actionEnable:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      
      -- enable action
      self:EnableAction();
    end
  ));
  
  -- action color changed
  self.actionColor.Wired.Event.Listener:Add(Wired.Event.Listener(
    "ColorChanged",
    function()
      
      -- change color
      self:ChangeActionColor();
    end,
    true
  ));
  
  -- content
  local content = self:GetApplication():GetMechanicContent():GetZones();
  
  -- populate zone filters
  self:PopulateZoneFilters(content);
  
  -- difficulty cell
  for k, cell in pairs(self.difficultyDropList:GetCells()) do
    if (cell:GetTitle() == RK:T("D.Standard")) then
      self.difficultyDropList:Select(cell);
    end
  end
  
  -- player cell
  for k, cell in pairs(self.playerDropList:GetCells()) do
    if (cell:GetTitle() == "20") then
      self.playerDropList:Select(cell);
    end
  end
  
  -- encounter tab
  self.encounterTabList:Select(self.encounterProfilePane);
end

-- application
function RaidKiller.Options.Tab.Mechanics:GetApplication()
  return self.application;
end

function RaidKiller.Options.Tab.Mechanics:SetApplication(value)
  self.application = value;
end

-- color picker window
function RaidKiller.Options.Tab.Mechanics:GetColorPickerWindow()
  if (not self.colorPickerWindow) then
    self.colorPickerWindow = Wired.UI.ColorPicker.Window("ColorPickerWindow", self:GetApplication():GetContext());
  end
  return self.colorPickerWindow;
end

function RaidKiller.Options.Tab.Mechanics:SetColorPickerWindow(value)
  self.colorPickerWindow = value;
end

-- layout
function RaidKiller.Options.Tab.Mechanics:LayoutMechanicsTab()
  
  -- selection pane
  
  -- label width
  local maxLabelWidth = math.max(
    self.playerDropLabel:GetWidth(),
    self.difficultyDropLabel:GetWidth(),
    self.zoneDropLabel:GetWidth(),
    self.encounterDropLabel:GetWidth()
  );
  
  -- list width
  local maxListWidth = math.max(
    self.playerDropList:GetWidth(),
    self.difficultyDropList:GetWidth(),
    self.zoneDropList:GetWidth(),
    self.encounterDropList:GetWidth()
  );
  
  -- difficulty drop list
  self.difficultyDropList:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -self:GetPadding(), y = self:GetPadding()});
  
  -- difficulty drop label
  self.difficultyDropLabel:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -(self:GetPadding() + maxListWidth + self:GetSpacing()), y = self:GetPadding()});
  
  -- player drop list
  self.playerDropList:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -(self:GetPadding() + maxListWidth + maxLabelWidth + (self:GetSpacing() * 2)), y = self:GetPadding()});
  
  -- player drop label
  self.playerDropLabel:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -(self:GetPadding() + (maxListWidth * 2) + maxLabelWidth + (self:GetSpacing() * 3)), y = self:GetPadding()});
  
  -- encounter drop list
  self.encounterDropList:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -self:GetPadding(), y = -self:GetPadding()});
  
  -- encounter drop label
  self.encounterDropLabel:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -(self:GetPadding() + maxListWidth + self:GetSpacing()), y = -self:GetPadding()});
  
  -- zone drop list
  self.zoneDropList:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -(self:GetPadding() + maxListWidth + maxLabelWidth + (self:GetSpacing() * 2)), y = -self:GetPadding()});
  
  -- zone drop label
  self.zoneDropLabel:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -(self:GetPadding() + (maxListWidth * 2) + maxLabelWidth + (self:GetSpacing() * 3)), y = -self:GetPadding()});
  
  -- selection pane
  self.selectionPane:SetWidth((maxLabelWidth * 2) + (maxListWidth * 2) + (self:GetPadding() * 2) + (self:GetSpacing() * 3));
  self.selectionPane:SetHeight(self.difficultyDropList:GetHeight() + self.zoneDropList:GetHeight() + (self:GetPadding() * 2) + self:GetSpacing());
  
  -- zone profile pane
  
  -- profile width
  local profileWidth = math.ceil((self.selectionPane:GetWidth() - (self:GetPadding() * 2)) / 3);
  
  -- zone profile pane
  self.zoneProfilePane:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = self:GetPadding(), y = self:GetPadding()});
  self.zoneProfilePane:SetWidth(self.selectionPane:GetWidth());
  self.zoneProfilePane:SetHeight(math.ceil(profileWidth * RaidKiller.Options.Tab.Mechanics.Constants.Profile.Ratio));
  
  -- zone profile left
  self.zoneProfileLeft:SetPoint({child = "LEFTCENTER", parent = "LEFTCENTER"}, {x = self:GetPadding(), y = 0});
  self.zoneProfileLeft:SetWidth(profileWidth);
  self.zoneProfileLeft:SetHeight(self.zoneProfilePane:GetHeight());
  
  -- zone profile center
  self.zoneProfileCenter:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self.zoneProfileCenter:SetWidth(profileWidth);
  self.zoneProfileCenter:SetHeight(self.zoneProfilePane:GetHeight());
  
  -- zone profile right
  self.zoneProfileRight:SetPoint({child = "RIGHTCENTER", parent = "RIGHTCENTER"}, {x = -self:GetPadding(), y = 0});
  self.zoneProfileRight:SetWidth(profileWidth);
  self.zoneProfileRight:SetHeight(self.zoneProfilePane:GetHeight());
  
  -- zone profile label
  self.zoneProfileLabel:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- selection pane
  self.selectionPane:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = self:GetPadding(), y = self.zoneProfilePane:GetBottom() + self:GetSpacing()});
  
  -- encounter profile label
  self.encounterProfileLabel:SetPoint({child = "TOPCENTER", parent = "TOPLEFT"}, {x = 0, y = self.selectionPane:GetBottom() + self:GetSpacing()});
  
  -- mechanic list label
  self.mechanicListLabel:SetPoint({child = "TOPCENTER", parent = "TOPRIGHT"}, {x = 0, y = self.selectionPane:GetBottom() + self:GetSpacing()});
  
  -- encounter profile
  self.encounterProfile:SetWidth(320);
  self.encounterProfile:SetHeight(180);
  
  -- encounter tab list
  self.encounterTabList:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = self:GetPadding(), y = self.encounterProfileLabel:GetBottom() + self:GetSpacing()});
  self.encounterTabList:SetWidth(self.encounterProfile:GetWidth() + (self.encounterTabList:GetPadding() * 2));
  self.encounterTabList:SetHeight(self.encounterProfile:GetHeight() + self.encounterTabList:GetSpacing() + self.encounterTabList:GetList():GetHeight() + (self.encounterTabList:GetPadding() * 2));
  
  -- encounter profile pane
  self.encounterProfilePane:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self.encounterProfilePane:SetWidth(320);
  self.encounterProfilePane:SetHeight(180);
  
  -- encounter details pane
  self.encounterDetailsPane:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self.encounterDetailsPane:SetWidth(320);
  self.encounterDetailsPane:SetHeight(180);
  
  -- encounter details label
  self.encounterDetailsLabel:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- encounter objectives pane
  self.encounterObjectivesPane:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self.encounterObjectivesPane:SetWidth(320);
  self.encounterObjectivesPane:SetHeight(180);
  
  -- encounter objectives label
  self.encounterObjectivesLabel:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- encounter history pane
  self.encounterHistoryPane:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self.encounterHistoryPane:SetWidth(320);
  self.encounterHistoryPane:SetHeight(180);
  
  -- encounter history label
  self.encounterHistoryLabel:SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- encounter profile
  self.encounterProfile:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = self:GetPadding(), y = self.encounterTabList:GetBottom() - self.encounterProfile:GetHeight()});
  
  -- mechanic list
  self.mechanicList:SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -self:GetPadding(), y = self.mechanicListLabel:GetBottom() + self:GetSpacing()});
  self.mechanicList:SetWidth(self.selectionPane:GetWidth() - (self.encounterTabList:GetWidth() + self:GetSpacing() + (self:GetPadding() * 2)));
  
  -- fix this
  
  -- encounter profile label
  self.encounterProfileLabel:SetPoint({child = "TOPCENTER", parent = "TOPLEFT"}, {x = (self:GetPadding() + (self.encounterTabList:GetWidth() / 2)), y = self.selectionPane:GetBottom() + self:GetSpacing()});
  
  -- mechanic list label
  self.mechanicListLabel:SetPoint({child = "TOPCENTER", parent = "TOPRIGHT"}, {x = -(self:GetPadding() + (self.mechanicList:GetWidth() / 2)), y = self.selectionPane:GetBottom() + self:GetSpacing()});
  
  -- fix this
  
  -- self
  self:SetWidth(self.selectionPane:GetWidth());
  self:SetHeight(self:GetParent():GetHeight());
  
  -- mechanic pane
  self.mechanicPane:SetPoint({child = "BOTTOMCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = -self:GetPadding()});
  self.mechanicPane:SetWidth(self:GetWidth() - (self:GetPadding() * 2));
  self.mechanicPane:SetHeight(self:GetHeight() - (self.encounterProfilePane:GetBottom() + self:GetSpacing() + self:GetPadding()));
  
  -- mechanic drop
  self.mechanicDrop:SetPoint({child = "BOTTOMCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = -self:GetPadding()});
  self.mechanicDrop:SetWidth(self.mechanicPane:GetWidth());
  self.mechanicDrop:SetHeight(self.mechanicPane:GetHeight());
  
  -- mechanic label
  self.mechanicLabel:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = self:GetPadding()});
  
  -- mechanic description
  self.mechanicDescription:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = self:GetPadding() + self.mechanicLabel:GetHeight() + self:GetSpacing()});
  self.mechanicDescription:SetWidth(self.mechanicPane:GetWidth());
  
  -- mechanic tested
  self.mechanicTested:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = self:GetPadding(), y = self:GetPadding()});
  
  -- mechanic trigger
  self.mechanicTrigger:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -self:GetPadding(), y = -self:GetPadding()});
  
  -- action drop list
  self.actionDropList:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = self:GetPadding(), y = 0});
  self.actionDropList:SetWidth(134);
  self.actionDropList:SetHeight(24);
  
  -- action edit
  --self.actionEdit:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = self.actionDropList:GetWidth() + self:GetPadding() + self:GetSpacing(), y = 0});
  
  -- action color
  self.actionColor:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = self.actionDropList:GetWidth() + self:GetPadding() + self:GetSpacing(), y = 0});
  
  -- action enable
  self.actionEnable:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = self.actionDropList:GetWidth() + self.actionColor:GetBase():GetWidth() + self:GetPadding() + (self:GetSpacing() * 2), y = 0});
  
  -- action pane
  self.actionPane:SetPoint({child = "BOTTOMLEFT", parent = "BOTTOMLEFT"}, {x = self:GetPadding(), y = -self:GetPadding()});
  self.actionPane:SetWidth(self.actionDropList:GetWidth() + self.actionColor:GetBase():GetWidth() + self.actionEnable:GetWidth() + (self:GetSpacing() * 2) + (self:GetPadding() * 2));
  self.actionPane:SetHeight(math.max(self.actionColor:GetBase():GetHeight(), self.actionDropList:GetHeight(), self.actionEnable:GetHeight()));
end

-- populate zone filters
function RaidKiller.Options.Tab.Mechanics:PopulateZoneFilters(content)
  
  --[[
  -- players
  local players = {};
  local playersSize = 0;
  
  -- difficulties
  local difficulties = {};
  local difficultiesSize = 0;
  
  -- zones
  for k, zone in pairs(content) do
    
    -- difficulties
    for l, difficulty in pairs(zone:GetDifficulties()) do
      if (not players[difficulty:GetPlayers()]) then
        players[difficulty:GetPlayers()] = difficulty:GetPlayers();
        playersSize = playersSize + 1;
      end
      if (not difficulties[difficulty:GetOrder()]) then
        difficulties[difficulty:GetOrder()] = difficulty:GetName();
        difficultiesSize = difficultiesSize + 1;
      end
    end
  end
  
  -- sort
  table.sort(players);
  table.sort(difficulties);
  --]]
  
  -- lists
  local lists = {
    players = {
      list = self.playerDropList,
      data = {
        "1", "2", "5", "10", "20"
      }
    },
    difficulties = {
      list = self.difficultyDropList,
      data = {
        RK:T("D.Chronicle"), RK:T("D.Standard"), RK:T("D.Expert"), RK:T("D.Master")
      }
    }
  };
  
  -- lists
  for k, x in pairs(lists) do
    
    -- populate
    x.list:SetSize(#x.data);
    x.list:Populate(function(sender, value)
      x.list:Select(sender);
    end);
    
    -- cells
    for i = 1, x.list:GetSize() do
      
      -- cell
      local cell = x.list:GetCells()[i];
      cell:SetTitle(x.data[i]);
      cell:SetVisible(true);
    end
    
    -- size list display
    x.list:SetSize(0);
    x.list:SetSize(#x.data);
  end
  
  -- index zones
  self.indexedZones = {};
  for k, zone in pairs(self:GetApplication():GetMechanicContent():GetZones()) do
    for l, difficulty in pairs(zone:GetDifficulties()) do
      
      -- index by difficulty
      if (type(self.indexedZones[difficulty:GetName()]) ~= "table") then
        self.indexedZones[difficulty:GetName()] = {};
      end
      
      -- index by players
      if (type(self.indexedZones[difficulty:GetName()][tostring(difficulty:GetPlayers())]) ~= "table") then
        self.indexedZones[difficulty:GetName()][tostring(difficulty:GetPlayers())] = {};
      end
      
      -- insert
      table.insert(self.indexedZones[difficulty:GetName()][tostring(difficulty:GetPlayers())], zone);
    end
  end
end

-- filter zones
function RaidKiller.Options.Tab.Mechanics:FilterZones()
  
  -- result
  local result = {};
  
  -- filters
  
  -- players
  local players = nil;
  if (self.playerDropList:GetSelected()) then
    players = self.playerDropList:GetSelected():GetTitle();
  end
  
  -- difficulty
  local difficulty = nil;
  if (self.difficultyDropList:GetSelected()) then
    difficulty = self.difficultyDropList:GetSelected():GetTitle();
  end
  
  -- selected
  if (difficulty) then
    if (self.indexedZones) then
      if (self.indexedZones[difficulty]) then
        if (players) then
          if (self.indexedZones[difficulty][players]) then
            result = self.indexedZones[difficulty][players];
          end
        end
      end
    end
  end
  
  -- return
  return result;
end

-- update selection
function RaidKiller.Options.Tab.Mechanics:UpdateSelection(listDisplay, unsorted, list)
  
  -- index data
  local data = {};
  for k, v in pairs(unsorted) do
    
    -- display
    local display = true;
    if (type(v.GetHidden) == "function") then
      display = not v:GetHidden();
    end
    if (display) then
      table.insert(data, v);
    end
  end
  
  -- sort data
  table.sort(data, function(a, b)
    
    -- sort by name
    local result = a:GetName() > b:GetName();
    
    -- sort by order
    if (a:GetOrder() and b:GetOrder()) then
      result = a:GetOrder() > b:GetOrder();
    end
    
    -- return
    return result;
  end);
  
  -- hide
  --listDisplay:SetVisible(false);
  
  -- populate
  if (#list:GetCells() == 0) then
    list:SetSize(RaidKiller.Options.Tab.Mechanics.Constants.List.Size);
    list:Populate(function(sender, value)
      list:Select(sender);
    end);
  end
  
  -- cells
  for i = 1, RaidKiller.Options.Tab.Mechanics.Constants.List.Size do
    
    -- cell
    local cell = list:GetCells()[i];
    
    -- category
    local category = data[i];
    if (category) then
      cell.category = category;
      cell:SetTitle(category:GetName());
      cell:SetVisible(true);
    else
      cell.category = nil;
      cell:SetVisible(false);
    end
  end
  
  -- size list display
  list:SetSize(0);
  list:SetSize(#data);
  
  -- cells changed
  list.Wired.Event:CellsChanged();
  
  -- data
  if (#data > 0) then
    if (not listDisplay:GetVisible()) then
      listDisplay:SetVisible(true);
    end
  else
    if (listDisplay:GetVisible()) then
      listDisplay:SetVisible(false);
      
      -- turn off sublists
      local nextList = listDisplay;
      while (nextList.nextList) do
        nextList.nextList:SetVisible(false);
        nextList = nextList.nextList;
      end
    end
  end
end

-- texture
function RaidKiller.Options.Tab.Mechanics:GetTexture(zone, encounter, value)
  
  -- path
  local path = RaidKiller.Options.Tab.Mechanics.Constants.Profile.Path;
  
  -- zone
  if (value) then
    if (zone:GetProfile()) then
      path = path .. zone:GetProfile() .. "/" .. value;
    else
      path = path .. RaidKiller.Options.Tab.Mechanics.Constants.Profile.Zone;
    end
    
  -- encounter
  else
    if (encounter:GetProfile()) then
      path = path .. zone:GetProfile() .. "/" .. encounter:GetProfile();
    else
      path = path .. RaidKiller.Options.Tab.Mechanics.Constants.Profile.Encounter;
    end
  end
  
  -- extension
  path = path .. RaidKiller.Options.Tab.Mechanics.Constants.Profile.Extension;
  
  return Wired.UI.Texture("RaidKiller", path);
end

-- update zone profile
function RaidKiller.Options.Tab.Mechanics:UpdateZoneProfile(value)
  
  -- exists
  if (value) then
    
    -- textures
    self.zoneProfileLeft:SetTexture(self:GetTexture(value, nil, 1));
    self.zoneProfileCenter:SetTexture(self:GetTexture(value, nil, 2));
    self.zoneProfileRight:SetTexture(self:GetTexture(value, nil, 3));
    
    -- name
    self.zoneProfileLabel:SetText(value:GetName());
    
    -- show
    self.zoneProfilePane:SetVisible(true);
  else
    self.zoneProfilePane:SetVisible(false);
  end
end

-- update encounter profile
function RaidKiller.Options.Tab.Mechanics:UpdateEncounterProfile(value)
  
  -- exists
  if (value) then
    
    -- profile
    self.encounterProfile:SetTexture(self:GetTexture(value:GetDifficulty():GetZone(), value, nil));
    
    -- profile label
    self.encounterProfileLabel:SetText(value:GetName());
    
    -- details label
    
    -- enrage
    local details = nil;
    if (value:GetEnrage()) then
      
      -- new line
      if (details) then
        details = details .. "\n\n";
      else
        details = "";
      end
      
      details = details .. Wired.UI.Format.Time(value:GetEnrage()) .. " " .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.Enrage;
    end
    
    -- hitpoints
    if (value:GetHitpoints()) then
      
      -- new line
      if (details) then
        details = details .. "\n\n";
      else
        details = "";
      end
      
      -- hitpoints
      details = details .. Wired.UI.Format.Hitpoints(value:GetHitpoints()) .. " " .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.Hitpoints;
    end
    
    -- apply
    if (not details) then
      details = RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.NoDetails;
    end
    self.encounterDetailsLabel:SetText(details);
    
    -- ojectives label
    local objectives = nil;
    
    -- phases
    for k, phase in pairs(value:GetState():GetPhasesByOrder()) do
      
      -- new line
      if (objectives) then
        objectives = objectives .. "\n\n";
      else
        objectives = "";
      end
      
      -- phase
      objectives = objectives .. phase:GetName();
      
      -- objectives
      for l, objective in pairs(phase:GetObjectivesByOrder()) do
        objectives = objectives .. "\n - " .. objective:GetMessage();
        if (objective:GetGoal() > 1) then
          objectives = objectives .. " (" .. objective:GetGoal() .. ")";
        end
      end
    end
    
    -- apply
    if (not objectives) then
      objectives = RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.NoObjectives;
    end
    self.encounterObjectivesLabel:SetText(objectives);
    
    -- history
    local history = nil;
    local historyData = value:GetState():GetHistory();
    
    -- attempted
    if (historyData.Attempts.Total > 0) then
      
      -- retarded data check
      if (not historyData.Attempts.Last) then
        historyData.Attempts.Last = 0;
      end
      if (not historyData.Attempts.Best) then
        historyData.Attempts.Best = 0;
      end
      if (not historyData.Completions.Last) then
        historyData.Completions.Last = 0;
      end
      if (not historyData.Completions.Best) then
        historyData.Completions.Best = 0;
      end
      
      -- difficulty
      local difficulty = (
        historyData.Completions.Total /
        historyData.Attempts.Total
      );
      difficulty = Wired.Math.Round((1 - difficulty) * 10, 1);
      
      -- difficulty
      history = RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.Difficulty .. ": " .. difficulty;
      
      -- attempts
      history = history .. "\n\n" .. historyData.Attempts.Total .. " " .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.Attempts;
      
      -- completions
      history = history .. "\n" .. historyData.Completions.Total .. " " .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.Completions;
      
      -- last attempt
      history = history .. "\n\n" .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.LastAttempt .. ": " .. Wired.UI.Format.Time(historyData.Attempts.Last);
      
      -- completed
      if (historyData.Completions.Total > 0) then
        
        -- last completion
        history = history .. "\n\n" .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.LastCompletion .. ": " .. Wired.UI.Format.Time(historyData.Completions.Last);
        
        -- best completion
        history = history .. "\n" .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.BestCompletion .. ": " .. Wired.UI.Format.Time(historyData.Completions.Best);
        
      -- not completed
      else
        
        -- best attempt
        history = history .. "\n" .. RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.BestAttempt .. ": " .. Wired.UI.Format.Time(historyData.Attempts.Best);
      end
    end
    
    -- apply
    if (not history) then
      history = RaidKiller.Options.Tab.Mechanics.Constants.Message.Component.NoHistory;
    end
    self.encounterHistoryLabel:SetText(history);
    
    -- show
    self.encounterTabList:SetVisible(true);
    self.encounterProfile:SetVisible(true);
  else
    self.encounterTabList:SetVisible(false);
    self.encounterProfile:SetVisible(false);
  end
end

-- update mechanic
function RaidKiller.Options.Tab.Mechanics:UpdateMechanic()
  
  -- exists
  local value = self.mechanicList:GetSelected().category;
  if (value) then
    
    -- name
    self.mechanicLabel:SetText(value:GetName());
    
    -- description
    self.mechanicDescription:SetText(value:GetDescription());
    
    -- tested
    local testedColor = RaidKiller.Options.Tab.Mechanics.Defaults.Color.Untested;
    local tested = RaidKiller.Options.Tab.Mechanics.Constants.Message.Untested;
    if (value:GetTested()) then
      testedColor = RaidKiller.Options.Tab.Mechanics.Defaults.Color.Tested;
      tested = RaidKiller.Options.Tab.Mechanics.Constants.Message.Tested;
    end
    self.mechanicTested:SetFontColor(testedColor);
    self.mechanicTested:SetText("[" .. tested .. "]");
    
    -- trigger
    local enabled = false;
    for k, action in pairs(value:GetActions()) do
      if (action:GetEnabled()) then
        enabled = true;
      end
    end
    self.mechanicTrigger:SetEnabled(enabled);
    
    -- show
    self.mechanicPane:SetVisible(true);
  else
    self.mechanicPane:SetVisible(false);
  end
end

-- update action
function RaidKiller.Options.Tab.Mechanics:UpdateAction()
  
  -- exists
  local value = self.actionDropList:GetSelected().category;
  if (value) then
    
    -- color
    self.actionColor:SetColor(value:GetColor());
    
    -- enable
    local enable = RaidKiller.Options.Tab.Mechanics.Constants.Message.Enable;
    if (value:GetEnabled()) then
      enable = RaidKiller.Options.Tab.Mechanics.Constants.Message.Disable;
    end
    self.actionEnable:SetTitle(enable);
    
    -- show
    --self.actionEdit:SetVisible(true);
    self.actionColor:SetVisible(true);
    self.actionEnable:SetVisible(true);
  else
    
    -- hide
    --self.actionEdit:SetVisible(false);
    self.actionColor:SetVisible(false);
    self.actionEnable:SetVisible(false);
  end
end

-- trigger mechanic
function RaidKiller.Options.Tab.Mechanics:TriggerMechanic()
  
  -- mechanic
  local mechanic = self.mechanicList:GetSelected().category;
  
  -- exists
  if (mechanic) then
    
    -- output
    --print("Triggering: " .. mechanic:GetName());
    
    -- activate
    mechanic:Activate();
  end
end

-- displayable action
function RaidKiller.Options.Tab.Mechanics:DisplayableAction(value)
  return (
    (value:GetType() == RaidKiller.Mechanic.Action.Object.Constants.Types.Alert) or
    (value:GetType() == RaidKiller.Mechanic.Action.Object.Constants.Types.Flash) or
    (value:GetType() == RaidKiller.Mechanic.Action.Object.Constants.Types.Message) or
    (value:GetType() == RaidKiller.Mechanic.Action.Object.Constants.Types.Timer)
  );
end

-- index actions
function RaidKiller.Options.Tab.Mechanics:IndexActions(value)
  
  -- result
  local result = {};
  
  -- actions
  local index = 0;
  for k, action in pairs(value) do
    
    -- displayable
    if (self:DisplayableAction(action)) then
      
      -- index
      index = index + 1;
      result[index] = action;
    end
  end
  
  -- return
  return result;
end

-- enable action
function RaidKiller.Options.Tab.Mechanics:EnableAction()
  
  -- action
  local action = self.actionDropList:GetSelected().category;
  
  -- exists
  if (action) then
    
    -- output
    --print("Enabling: " .. action:GetName());
    
    -- enable
    action:SetEnabled(not action:GetEnabled());
    
    -- update
    self:UpdateMechanic();
    self:UpdateAction();
  end
end

-- change action color
function RaidKiller.Options.Tab.Mechanics:ChangeActionColor()
  
  -- action
  local action = self.actionDropList:GetSelected().category;
  
  -- exists
  if (action) then
    
    -- output
    --print("Changing Color: " .. action:GetName());
    
    -- enable
    action:SetColor(self.actionColor:GetColor());
  end
end
