-----------------------
-- OCD               --
-- By: Wire @ Byriel --
-----------------------

Raid = class();

Raid.Constants = {
  Inspect = {
    Slot = "group"
  },
  Groups = 4,
  GroupSize = 5
};

function Raid:Constructor()
  
  -- initialized
  self:SetInitialized(false);
end

-- initialized
function Raid:GetInitialized()
  return self.initialized;
end

function Raid:SetInitialized(value)
  self.initialized = value;
end

-- players by slot
function Raid:GetPlayersBySlot()
  if (type(self.playersBySlot) ~= "table") then
    self.playersBySlot = {};
  end
  return self.playersBySlot;
end

-- players by name
function Raid:GetPlayersByName()
  if (type(self.playersByName) ~= "table") then
    self.playersByName = {};
  end
  return self.playersByName;
end

function Raid:AddPlayer(slot, value)
  
  -- add player
  self:GetPlayers()[slot] = value;
  
  -- register
  value:SetRaid(self);
end

-- refresh players
function Raid:RefreshPlayers()
  
  -- players
  for slot = 1, (Raid.Constants.Groups * Raid.Constants.GroupSize) do
    
    -- group id
    local slotID = slot
    if (slotID < 10) then
      slotID = "0" .. slotID;
    end
    
    -- player
    local player = self:GetPlayersBySlot()[slot];
    if (player) then
      
      -- reset slot
      if (player:GetSlot() == slot) then
        player:SetSlot();
      end
    end
    self:GetPlayersBySlot()[slot] = nil;
    
    -- details
    local details = Inspect.Unit.Detail(Raid.Constants.Inspect.Slot .. slotID);
    if (details) then
      
      -- create player
      if (not self:GetPlayersByName()[details.name]) then
        self:GetPlayersByName()[details.name] = Player(details);
        
        -- register
        self:GetPlayersByName()[details.name]:SetRaid(self);
        
      -- update details
      else
        self:GetPlayersByName()[details.name]:SetDetails(details);
      end
      
      -- add player
      player = self:GetPlayersByName()[details.name];
      
      -- register
      player:SetSlot(slot);
      
      -- update slot
      self:GetPlayersBySlot()[slot] = player;
    end
  end
  
  -- hide timers
  for k, player in pairs(self:GetPlayersByName()) do
    if (not player:GetSlot()) then
      if (OCD.cooldownManager:GetTimersByPlayer()[player:GetName()]) then
        for j, timer in pairs(OCD.cooldownManager:GetTimersByPlayer()[player:GetName()]) do
          timer:SetVisible(false);
        end
      end
    end
  end
  
  -- redraw
  OCD.cooldownManager:RedrawTimers();
end
