-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

RaidKiller.Category.Encounter = class(RaidKiller.Category.Object);

function RaidKiller.Category.Encounter:Constructor(data)
  RaidKiller.Category.Object.Constructor(self, data);
  
  -- parameters
  self:SetHitpoints(data.hitpoints);
  self:SetEnrage(data.enrage);
  self:SetMechanics(data.mechanics);
  
  -- create enrage mechanic
  self:CreateEnrageMechanic();
  
  -- create objective mechanic
  self:CreateObjectiveMechanic(data);
  
  -- state
  self:SetState(RaidKiller.State.Encounter(data));
  
  -- create phase alerts
  self:CreatePhaseAlerts();
end

-- difficulty
function RaidKiller.Category.Encounter:GetDifficulty()
  return self.difficulty;
end

function RaidKiller.Category.Encounter:SetDifficulty(value)
  self.difficulty = value;
end

-- mechanics
function RaidKiller.Category.Encounter:GetMechanics()
  return self:GetChildren();
end

function RaidKiller.Category.Encounter:SetMechanics(value)
  self:SetChildren(value);
  
  -- register
  for k, child in pairs(self:GetChildren()) do
    child:SetEncounter(self);
  end
end

function RaidKiller.Category.Encounter:AddMechanic(value)
  
  -- index
  self:AddChild(value);
  
  -- register
  value:SetEncounter(self);
end

-- hitpoints
function RaidKiller.Category.Encounter:GetHitpoints()
  return self.hitpoints;
end

function RaidKiller.Category.Encounter:SetHitpoints(value)
  self.hitpoints = value;
end

-- enrage
function RaidKiller.Category.Encounter:GetEnrage()
  return self.enrage;
end

function RaidKiller.Category.Encounter:SetEnrage(value)
  self.enrage = value;
end

-- state
function RaidKiller.Category.Encounter:GetState()
  return self.state;
end

function RaidKiller.Category.Encounter:SetState(value)
  self.state = value;
  
  -- register
  self:GetState():SetEncounter(self);
  self:GetState():SetParent(self);
end

-- create enrage mechanic
function RaidKiller.Category.Encounter:CreateEnrageMechanic()
  
  -- enrage
  if (self:GetEnrage()) then
    
    -- description
    local description = RK:T("G.MD.Enrage");
    description = description[1] .. self:GetName() .. description[2] .. Wired.UI.Format.Time(self:GetEnrage()) .. description[3];
    
    -- mechanic
    self:AddMechanic(RaidKiller.Mechanic.Object({
      order = 0,
      name = RK:T("G.M.Enrage"),
      description = description,
      tested = true,
      triggers = {
        RaidKiller.Mechanic.Trigger.Engage({
          encounter = self:GetName(),
          settings = {
            duration = self:GetEnrage()
          }
        })
      },
      actions = {
        RaidKiller.Mechanic.Action.Timer({
          enabled = true,
          message = RK:T("G.L.Enrage")
        })
      }
    }));
  end
end

-- create objective mechanic
function RaidKiller.Category.Encounter:CreateObjectiveMechanic(data)
  
  -- no phases
  if (not data.phases) then
    
    -- kill
    local objective = RaidKiller.State.Objective.Kill({
      goal = 1,
      order = 1,
      required = true,
      unit = self:GetName()
    });
    
    -- phases
    data.phases = {
      RaidKiller.State.Phase({
        order = 1,
        name = self:GetName(),
        objectives = {
          objective
        }
      })
    };
    
    -- objective mechanic
    self:AddMechanic(RaidKiller.Mechanic.Object({
      name = RK:T("G.M.Phasing"),
      hidden = true,
      triggers = {
        RaidKiller.Mechanic.Trigger.Death({
          units = {
            self:GetName()
          },
          settings = {
            targetObjective = 1,
            targetPhase = 1,
          }
        })
      },
      actions = {
        RaidKiller.Mechanic.Action.Objective({
        })
      }
    }));
  end
end

-- create phase alerts
function RaidKiller.Category.Encounter:CreatePhaseAlerts()
  
  -- triggers
  local triggers = {};
  
  -- phases
  local phases = 0;
  for k, phase in pairs(self:GetState():GetPhasesByOrder()) do
    
    -- skip first phase
    if (phase:GetOrder() > 1) then
      
      -- phases
      phases = phases + 1;
      
      -- add phase trigger
      table.insert(
        triggers, 
        RaidKiller.Mechanic.Trigger.Phase({
          phase = phase:GetOrder(),
          entered = true,
          settings = {
            message = phase:GetName() .. " " .. RK:T("G.L.Started")
          }
        })
      );
    end
  end
  
  -- phase alert mechanic
  if (phases > 0) then
    self:AddMechanic(RaidKiller.Mechanic.Object({
      order = 0,
      name = RK:T("G.M.Phase"),
      description = RK:T("G.MD.Phase"),
      tested = true,
      triggers = triggers,
      actions = {
        RaidKiller.Mechanic.Action.Alert({
          enabled = true,
          color = Wired.UI.Color.Green
        }),
        RaidKiller.Mechanic.Action.Flash({
          color = Wired.UI.Color.Green
        })
      }
    }));
  end
end
