-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

-- class
RaidKiller.Mechanic.Action.Alert = class(RaidKiller.Mechanic.Action.Object);

-- constants
RaidKiller.Mechanic.Action.Alert.Constants = {
  Persistence = {
    Color = "Color",
    Message = "Message",
    Position = "Position",
    Size = "Size"
  }
};

function RaidKiller.Mechanic.Action.Alert:Constructor(data)
  RaidKiller.Mechanic.Action.Object.Constructor(self, data);
  
  -- constants
  self:SetType(RaidKiller.Mechanic.Action.Object.Constants.Types.Alert);
  self:SetName(RaidKiller.Mechanic.Action.Object.Constants.Names.Alert);
  
  -- parameters
  self:SetColor(data.color);
  self:SetMessage(data.message);
end

-- color
function RaidKiller.Mechanic.Action.Alert:GetColor()
  local value = self:GetPersistenceValue(RaidKiller.Mechanic.Action.Alert.Constants.Persistence.Color);
  local result = nil;
  if (value) then
    if (value.Red and value.Green and value.Blue and value.Alpha) then
      result = Wired.UI.Color(
        value.Red,
        value.Green,
        value.Blue,
        value.Alpha
      );
    end
  end
  return result;
end

function RaidKiller.Mechanic.Action.Alert:SetColor(value)
  local color = {};
  if (value) then
    color = {
      Red = value:GetRed(),
      Green = value:GetGreen(),
      Blue = value:GetBlue(),
      Alpha = value:GetAlpha()
    };
  end
  self:SetPersistenceValue(RaidKiller.Mechanic.Action.Alert.Constants.Persistence.Color, color);
end

-- message
function RaidKiller.Mechanic.Action.Alert:GetMessage()
  
  -- result
  local result = self:GetPersistenceValue(RaidKiller.Mechanic.Action.Alert.Constants.Persistence.Message);
  
  -- mechanic name
  if ((not result) and self:GetMechanic()) then
    result = self:GetMechanic():GetName();
  end
  
  -- return
  return result;
end

function RaidKiller.Mechanic.Action.Alert:SetMessage(value)
  self:SetPersistenceValue(RaidKiller.Mechanic.Action.Alert.Constants.Persistence.Message, value);
end

-- settings
function RaidKiller.Mechanic.Action.Alert:GetSettings()
  return {
    color = self:GetColor(),
    message = self:GetMessage()
  };
end

-- execute
function RaidKiller.Mechanic.Action.Alert:Execute(settings)
  
  -- localize
  local alert = self:GetManager():GetAlert();
  local bar = self:GetManager():GetAlertBar();
  
  -- position
  alert:SetPoint(
    {child = "TOPCENTER", parent = "TOPLEFT"},
    {x = bar:GetLeft() + (bar:GetWidth() / 2), y = bar:GetBottom()}
  );
  
  -- start
  alert:Start(settings);
end
