-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

-- class
RaidKiller.Mechanic.Action.Timer = class(RaidKiller.Mechanic.Action.Object);

-- constants
RaidKiller.Mechanic.Action.Timer.Constants = {
  Persistence = {
    Color = "Color",
    Message = "Message",
    Position = "Position",
    Size = "Size"
  },
  Duration = 10
};

-- defaults
RaidKiller.Mechanic.Action.Timer.Defaults = {
  Height = 25
};

-- constructor
function RaidKiller.Mechanic.Action.Timer:Constructor(data)
  RaidKiller.Mechanic.Action.Object.Constructor(self, data);
  
  -- constants
  self:SetType(RaidKiller.Mechanic.Action.Object.Constants.Types.Timer);
  self:SetName(RaidKiller.Mechanic.Action.Object.Constants.Names.Timer);
  
  -- parameters
  self:SetColor(data.color);
  self:SetMessage(data.message);
  self:SetContinuous(data.continuous);
  self:SetDuration(data.duration);
  self:SetGroup(data.group);
end

-- persistence

-- color
function RaidKiller.Mechanic.Action.Timer:GetColor()
  local value = self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Color);
  local result = nil;
  if (value) then
    if (value.Red and value.Green and value.Blue and value.Alpha) then
      result = Wired.UI.Color(
        value.Red,
        value.Green,
        value.Blue,
        value.Alpha
      );
    end
  end
  return result;
end

function RaidKiller.Mechanic.Action.Timer:SetColor(value)
  local color = {};
  if (value) then
    color = {
      Red = value:GetRed(),
      Green = value:GetGreen(),
      Blue = value:GetBlue(),
      Alpha = value:GetAlpha()
    };
  end
  self:SetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Color, color);
end

-- message
function RaidKiller.Mechanic.Action.Timer:GetMessage()
  
  -- persistence
  local result = self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Message);
  
  -- mechanic name
  if ((not result) and self:GetMechanic()) then
    result = self:GetMechanic():GetName();
  end
  
  -- return
  return result;
end

function RaidKiller.Mechanic.Action.Timer:SetMessage(value)
  self:SetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Message, value);
end

-- position
function RaidKiller.Mechanic.Action.Timer:GetPosition()
  if (type(self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Position)) ~= "table") then
    self:SetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Position, {});
  end
  return self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Position);
end

-- size
function RaidKiller.Mechanic.Action.Timer:GetSize()
  if (type(self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Size)) ~= "table") then
    self:SetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Size, {});
  end
  return self:GetPersistenceValue(RaidKiller.Mechanic.Action.Timer.Constants.Persistence.Size);
end

-- values

-- continuous
function RaidKiller.Mechanic.Action.Timer:GetContinuous()
  return self.continuous;
end

function RaidKiller.Mechanic.Action.Timer:SetContinuous(value)
  self.continuous = value;
end

-- duration
function RaidKiller.Mechanic.Action.Timer:GetDuration()
  return self.duration;
end

function RaidKiller.Mechanic.Action.Timer:SetDuration(value)
  self.duration = value;
end

-- group
function RaidKiller.Mechanic.Action.Timer:GetGroup()
  return self.group;
end

function RaidKiller.Mechanic.Action.Timer:SetGroup(value)
  self.group = value;
end

-- timers
function RaidKiller.Mechanic.Action.Timer:GetTimers()
  if (type(self.timers) ~= "table") then
    self.timers = {};
  end
  return self.timers;
end

function RaidKiller.Mechanic.Action.Timer:SetTimers(value)
  self.timers = value;
end

-- settings
function RaidKiller.Mechanic.Action.Timer:GetSettings()
  return {
    color = self:GetColor(),
    continuous = self:GetContinuous(),
    duration = self:GetDuration(),
    group = self:GetGroup(),
    message = self:GetMessage(),
    position = self:GetPosition(),
    size = self:GetSize()
  };
end

-- timer title
function RaidKiller.Mechanic.Action.Timer:GetTimerTitle(settings)
  
  -- message
  local result = settings.message;
  
  -- player
  if (settings.group and settings.player) then
    result = result .. ": " .. settings.player;
  end
  
  -- return
  return result;
end

-- save timer
function RaidKiller.Mechanic.Action.Timer:SaveTimer(timer)
  
  -- locked
  if (timer:GetLocked()) then
    
    -- size
    self:GetSize()[timer:GetTitle()] = timer:GetFontSize();
    
    -- position
    self:GetPosition()[timer:GetTitle()] = {
      Left = timer:GetBase():GetLeft(),
      Top = timer:GetBase():GetTop()
    };
    
  -- unlocked
  else
    self:GetSize()[timer:GetTitle()] = nil;
    self:GetPosition()[timer:GetTitle()] = nil;
  end
end

-- update timer
function RaidKiller.Mechanic.Action.Timer:UpdateTimer(timer, settings)
  
  -- apply settings
  timer:SetTint(settings.color);
  timer:SetContinuous(settings.continuous);
  timer:SetFlipped(true);
  timer:SetInverted(not settings.group);
  timer:SetPercentage(false);
  
  -- mark
  if (settings.mark) then
    timer:SetMark(settings.mark);
  end
  
  -- duration
  local duration = settings.duration;
  if (not duration) then
    duration = RaidKiller.Mechanic.Action.Timer.Constants.Duration;
  end
  
  -- scale
  timer:SetScale(Wired.Math.Scale({minimum = 0, maximum = duration}));
  
  -- value
  timer:SetValue(duration);
end

-- create timer
function RaidKiller.Mechanic.Action.Timer:CreateTimer(title, settings)
  
  -- create timer
  local timer = Wired.UI.Timer(
    "Timer_" .. title,
    self:GetManager():GetParent(),
    self:GetManager():GetApplication()
  );
  
  -- title
  timer:SetTitle(title);
  
  -- update timer
  self:UpdateTimer(timer, settings);
  
  -- load
  
  -- size
  local size = self:GetSize()[title];
  if (size) then
    timer:SetFontSize(size);
  end
  
  -- position
  local position = self:GetPosition()[title];
  if (position) then
    timer:SetPosition(position.Left, position.Top);
  end
  
  -- lock
  if (size or position) then
    timer:SetLocked(true);
  end
  
  -- events
  
  -- click
  timer:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "StationaryClick",
    function()
      
      -- deactivate
      if (timer:GetActive()) then
        timer:SetActive(false);
      end
      
      -- hide
      if (timer:GetVisible()) then
        timer:SetVisible(false);
      end
    end,
    true
  ));
  
  -- visible
  timer.Wired.Event.Listener:Add(Wired.Event.Listener(
    "VisibleChanged",
    function(sender, value)
      self:GetManager():TimerVisibleChanged(self, timer);
    end,
    true
  ));
  
  -- locked
  timer.Wired.Event.Listener:Add(Wired.Event.Listener(
    "LockedChanged",
    function()
      
      -- redraw
      self:GetManager():RedrawTimers();
      
      -- save
      self:SaveTimer(timer);
    end,
    true
  ));
  
  -- finished
  timer.Wired.Event.Listener:Add(Wired.Event.Listener(
    "TimerFinished",
    function()
      self:GetManager():TimerFinished(self, timer);
    end,
    true
  ));
  
  -- return
  return timer;
end

-- execute
function RaidKiller.Mechanic.Action.Timer:Execute(settings)
  
  -- title
  local title = self:GetTimerTitle(settings);
  
  -- timer
  local timer = self:GetTimers()[title];
  
  -- create timer
  if (not timer) then
    timer = self:CreateTimer(title, settings);
    
    -- index by title
    self:GetTimers()[title] = timer;
    
  -- update timer
  else
    self:UpdateTimer(timer, settings);
  end
  
  -- show
  timer:SetVisible(true);
  
  -- activate
  timer:SetActive(true);
end
