-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

RaidKiller.Mechanic.Manager = class();

-- constants
RaidKiller.Mechanic.Manager.Constants = {
  Texture = {
    EncounterActive = Wired.UI.Texture("RaidKiller", "Resources/Pin/engaged-off.png"),
    EncounterActiveGlow = Wired.UI.Texture("RaidKiller", "Resources/Pin/glow.png")
  }
};

-- defaults
RaidKiller.Mechanic.Manager.Defaults = {
  TimerSpacing = 2,
  Chronicle = false,
  Stacking = false,
  Timeout = 15,
  Tray = {
    Offset = {
      Top = 98
    }
  },
  Animation = {
    Glow = {
      Minimum = 0.25,
      Maximum = 0.90,
      Duration = 0.25
    }
  }
};

function RaidKiller.Mechanic.Manager:Constructor(application)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetApplication(application);
  self:SetParent(self:GetApplication():GetContext());
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- persistence
  self.Wired.Persistence:Enable("Mechanic Manager");
  self.Wired.Persistence:SetParent(self:GetParent():GetBase());
  
  -- stacking
  self.stacking = Wired.Persistence.PersistentVariable(
    "Stacking",
     self:GetParent():GetBase(),
     RaidKiller.Mechanic.Manager.Defaults.Stacking
  );
  
  -- chronicle
  self.chronicle = Wired.Persistence.PersistentVariable(
    "Chronicle",
     self:GetParent():GetBase(),
     RaidKiller.Mechanic.Manager.Defaults.Chronicle
  );
  
  -- components
  
  -- flash
  self:SetFlash(Wired.UI.Animation.Flash(self:GetApplication()));
  
  -- alert
  self:SetAlert(Wired.UI.Animation.Alert(self:GetApplication()));
  
  -- timer bar
  self:SetTimerBar(Wired.UI.CascadeBar(self:GetApplication():GetName() .. "_TimerBar", self:GetApplication():GetContext(), {Left = (UIParent:GetWidth() - 300), Top = (UIParent:GetHeight() * 0.50)}));
  self:GetTimerBar():SetTitle("Wire's " .. self:GetApplication():GetName() .. " Timers");
  
  -- monitor bar
  self:SetMonitorBar(Wired.UI.CascadeBar(self:GetApplication():GetName() .. "_MonitorBar", self:GetApplication():GetContext(), {Left = (UIParent:GetWidth() - 300), Top = 300}));
  self:GetMonitorBar():SetTitle("Wire's " .. self:GetApplication():GetName() .. " Monitor");
  self:GetMonitorBar():SetRatio(9);
  
  -- alert bar
  self:SetAlertBar(Wired.UI.CascadeBar(self:GetApplication():GetName() .. "_AlertBar", self:GetApplication():GetContext(), {Left = (UIParent:GetWidth() * 0.50) - 150, Top = (UIParent:GetHeight() * 0.15)}));
  self:GetAlertBar():SetTitle("Wire's " .. self:GetApplication():GetName() .. " Alerts");
  
  -- tray
  self:SetTray(Wired.UI.Tray(self:GetApplication():GetName() .. "_Tray", self:GetApplication():GetContext()));
  self:GetTray():SetVisible(false);
  
  -- encounter active display
  self:SetEncounterActiveDisplay(Wired.UI.Trion.Texture("EncounterActiveDisplay", self:GetTray()));
  self:GetEncounterActiveDisplay():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 6, y = 0});
  self:GetEncounterActiveDisplay():SetLayer(1);
  self:GetEncounterActiveDisplay():SetWidth(14 * (39 / 50));
  self:GetEncounterActiveDisplay():SetHeight(14);
  self:GetEncounterActiveDisplay():SetColor(Wired.UI.Color.Clear);
  self:GetEncounterActiveDisplay():SetTexture(RaidKiller.Mechanic.Manager.Constants.Texture.EncounterActive);
  self:GetEncounterActiveDisplay():SetVisible(true);
  
  -- encounter active display glow
  self:SetEncounterActiveDisplayGlow(Wired.UI.Trion.Texture("EncounterActiveDisplayGlow", self:GetTray()));
  self:GetEncounterActiveDisplayGlow():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 2, y = 0});
  self:GetEncounterActiveDisplayGlow():SetLayer(-1);
  self:GetEncounterActiveDisplayGlow():SetWidth(22 * (39 / 50));
  self:GetEncounterActiveDisplayGlow():SetHeight(22);
  self:GetEncounterActiveDisplayGlow():SetColor(Wired.UI.Color.Clear);
  self:GetEncounterActiveDisplayGlow():SetTexture(RaidKiller.Mechanic.Manager.Constants.Texture.EncounterActiveGlow);
  self:GetEncounterActiveDisplayGlow():SetVisible(true);
  
  -- encounter phase display
  self:SetEncounterPhaseDisplay(Wired.UI.TextGlow("EncounterPhaseDisplay", self:GetTray()));
  self:GetEncounterPhaseDisplay():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetEncounterPhaseDisplay():SetFontSize(14);
  self:GetEncounterPhaseDisplay():SetColor(Wired.UI.Color.Clear);
  self:GetEncounterPhaseDisplay():SetFontColor(Wired.UI.Color.White);
  self:GetEncounterPhaseDisplay():SetText("Phase");
  --self:GetEncounterPhaseDisplay():ScaleFontSizeVertically(self:GetTray());
  self:GetEncounterPhaseDisplay():SetVisible(true);
  
  -- encounter duration display
  self:SetEncounterDurationDisplay(Wired.UI.TextGlow("EncounterDurationDisplay", self:GetTray()));
  self:GetEncounterDurationDisplay():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = -2, y = 0});
  self:GetEncounterDurationDisplay():SetFontSize(14);
  self:GetEncounterDurationDisplay():SetColor(Wired.UI.Color.Clear);
  self:GetEncounterDurationDisplay():SetFontColor(Wired.UI.Color.White);
  self:GetEncounterDurationDisplay():SetText("00:00");
  --self:GetEncounterDurationDisplay():ScaleFontSizeVertically(self:GetTray());
  self:GetEncounterDurationDisplay():SetVisible(true);
  
  -- objective display
  self:SetObjectiveDisplay(Wired.UI.Trion.Text("ObjectiveDisplay", self:GetApplication():GetContext()));
  self:GetObjectiveDisplay():SetFontColor(Wired.UI.Color.White);
  self:GetObjectiveDisplay():SetFontSize(14);
  self:GetObjectiveDisplay():SetVisible(false);
  
  -- events
  
  -- idle
  self:GetIdle().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Idle",
    function()
      self:Idle();
    end,
    true
  ));
  
  -- triggers
  
  -- buff
  table.insert(Event.Buff.Add, {
    function(unit, buffs)
      self:Buff(unit, buffs);
    end,
    self:GetApplication():GetName(),
    "Mechanic Manager Buff Handler"
  });
  
  -- cast
  table.insert(Event.Unit.Castbar, {
    function(value)
      self:Cast(value);
    end,
    self:GetApplication():GetName(),
    "Mechanic Manager Cast Handler"
  });
  
  -- notify
  table.insert(Event.Chat.Notify, {
    function(value)
      self:Notify(value);
    end,
    self:GetApplication():GetName(),
    "Mechanic Manager Notify Handler"
  });
  
  -- speak
  table.insert(Event.Chat.Npc, {
    function(value)
      self:Speak(value);
    end,
    self:GetApplication():GetName(),
    "Mechanic Manager Speak Handler"
  });
  
  -- encounter
  
  -- damage
  table.insert(Event.Combat.Damage, {
    function(value)
      self:Damage(value);
    end,
    self:GetApplication():GetName(),
    "Damage Handler"
  });
  
  -- health max
  table.insert(Event.Unit.Detail.HealthMax, {
    function(value)
      self:HealthMax(value);
    end,
    self:GetApplication():GetName(),
    "Health Max Handler"
  });
  
  -- health
  table.insert(Event.Unit.Detail.Health, {
    function(value)
      self:Health(value);
    end,
    self:GetApplication():GetName(),
    "Health Handler"
  });
  
  -- death
  table.insert(Event.Combat.Death, {
    function(value)
      self:Death(value);
    end,
    self:GetApplication():GetName(),
    "Death Handler"
  });
end

-- application
function RaidKiller.Mechanic.Manager:GetApplication()
  return self.application;
end

function RaidKiller.Mechanic.Manager:SetApplication(value)
  self.application = value;
end

-- parent
function RaidKiller.Mechanic.Manager:GetParent()
  return self.parent;
end

function RaidKiller.Mechanic.Manager:SetParent(value)
  self.parent = value;
end

-- timer bar
function RaidKiller.Mechanic.Manager:GetTimerBar()
  return self.timerBar;
end

function RaidKiller.Mechanic.Manager:SetTimerBar(value)
  self.timerBar = value;
  
  -- register bar
  self:GetApplication():RegisterCascadeBar(value);
end

-- monitor bar
function RaidKiller.Mechanic.Manager:GetMonitorBar()
  return self.monitorBar;
end

function RaidKiller.Mechanic.Manager:SetMonitorBar(value)
  self.monitorBar = value;
  
  -- register bar
  self:GetApplication():RegisterCascadeBar(value);
end

-- alert bar
function RaidKiller.Mechanic.Manager:GetAlertBar()
  return self.alertBar;
end

function RaidKiller.Mechanic.Manager:SetAlertBar(value)
  self.alertBar = value;
  
  -- register bar
  self:GetApplication():RegisterCascadeBar(value);
end

-- idle
function RaidKiller.Mechanic.Manager:GetIdle()
  return self.idle;
end

function RaidKiller.Mechanic.Manager:SetIdle(value)
  self.idle = value;
end

-- flash
function RaidKiller.Mechanic.Manager:GetFlash()
  return self.flash;
end

function RaidKiller.Mechanic.Manager:SetFlash(value)
  self.flash = value;
end

-- alert
function RaidKiller.Mechanic.Manager:GetAlert()
  return self.alert;
end

function RaidKiller.Mechanic.Manager:SetAlert(value)
  self.alert = value;
end

-- tray
function RaidKiller.Mechanic.Manager:GetTray()
  return self.tray;
end

function RaidKiller.Mechanic.Manager:SetTray(value)
  self.tray = value;
end

-- encounter active display
function RaidKiller.Mechanic.Manager:GetEncounterActiveDisplay()
  return self.encounterActiveDisplay;
end

function RaidKiller.Mechanic.Manager:SetEncounterActiveDisplay(value)
  self.encounterActiveDisplay = value;
end

-- encounter active display glow
function RaidKiller.Mechanic.Manager:GetEncounterActiveDisplayGlow()
  return self.encounterActiveDisplayGlow;
end

function RaidKiller.Mechanic.Manager:SetEncounterActiveDisplayGlow(value)
  self.encounterActiveDisplayGlow = value;
end

-- encounter duration display
function RaidKiller.Mechanic.Manager:GetEncounterDurationDisplay()
  return self.encounterDurationDisplay;
end

function RaidKiller.Mechanic.Manager:SetEncounterDurationDisplay(value)
  self.encounterDurationDisplay = value;
end

-- encounter phase display
function RaidKiller.Mechanic.Manager:GetEncounterPhaseDisplay()
  return self.encounterPhaseDisplay;
end

function RaidKiller.Mechanic.Manager:SetEncounterPhaseDisplay(value)
  self.encounterPhaseDisplay = value;
end

-- tray glow animation
function RaidKiller.Mechanic.Manager:GetTrayGlowAnimation()
  if (not self.trayGlowAnimation) then
    self:CreateTrayGlowAnimation();
  end
  return self.trayGlowAnimation;
end

function RaidKiller.Mechanic.Manager:SetTrayGlowAnimation(value)
  self.trayGlowAnimation = value;
end

-- objective display
function RaidKiller.Mechanic.Manager:GetObjectiveDisplay()
  return self.objectiveDisplay;
end

function RaidKiller.Mechanic.Manager:SetObjectiveDisplay(value)
  self.objectiveDisplay = value;
end

-- create tray glow animation
function RaidKiller.Mechanic.Manager:CreateTrayGlowAnimation(value)
  
  -- sequence
  self:SetTrayGlowAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Glow",
    cycles = 1,
    continuous = true
  }));
  
  -- steps
  self:GetTrayGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade In",
    value = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Minimum,
    minimum = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Minimum,
    maximum = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Maximum,
    duration = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {
        self:GetEncounterActiveDisplayGlow(),
        self:GetEncounterDurationDisplay():GetTopGlow(),
        self:GetEncounterDurationDisplay():GetBottomGlow(),
        self:GetEncounterPhaseDisplay():GetTopGlow(),
        self:GetEncounterPhaseDisplay():GetBottomGlow()
      }
    })}
  }));
  self:GetTrayGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade Out",
    value = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Maximum,
    minimum = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Minimum,
    maximum = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Maximum,
    duration = RaidKiller.Mechanic.Manager.Defaults.Animation.Glow.Duration,
    direction = -1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {
        self:GetEncounterActiveDisplayGlow(),
        self:GetEncounterDurationDisplay():GetTopGlow(),
        self:GetEncounterDurationDisplay():GetBottomGlow(),
        self:GetEncounterPhaseDisplay():GetTopGlow(),
        self:GetEncounterPhaseDisplay():GetBottomGlow()
      }
    })}
  }));
end

-- show encounter display
function RaidKiller.Mechanic.Manager:ShowEncounterDisplay()
  
  -- phase
  local phase = nil;
  if (self:GetState()) then
    phase = self:GetState():GetPhase();
  end
  
  -- phase
  local objectives = nil;
  if (phase) then
    
    -- anchor
    local anchor = self:GetMonitorBar();
    self:GetTray():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = anchor:GetBase():GetLeft(), y = anchor:GetBase():GetBottom() + 2});
    anchor = self:GetTray();
    self:GetObjectiveDisplay():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = anchor:GetBase():GetLeft(), y = anchor:GetBase():GetBottom() + 2});
    
    -- objectives
    for l, objective in pairs(phase:GetObjectivesByOrder()) do
      
      -- new line
      if (objectives) then
        objectives = objectives .. "\n";
      else
        objectives = "";
      end
      objectives = objectives .. (objective:GetMessage() .. " (" .. objective:GetProgress() .. " / " .. objective:GetGoal() .. ")");
    end
  end
  
  -- objectives
  if (objectives) then
    self:GetObjectiveDisplay():SetText(objectives);
    self:GetObjectiveDisplay():SetVisible(true);
    self:GetTray():SetVisible(true);
    
  -- no objectives
  else
    self:GetObjectiveDisplay():SetVisible(false);
    self:GetTray():SetVisible(false);
  end
end

-- update encounter display
function RaidKiller.Mechanic.Manager:UpdateEncounterDisplay()
  
  -- engaged
  if (self:GetState()) then
    
    -- duration
    local duration = self:GetState():GetDuration();
    if (duration ~= self:GetDuration()) then
      self:SetDuration(duration);
      
      -- update display
      self:GetEncounterDurationDisplay():SetText(Wired.UI.Format.Time(duration));
      --self:GetEncounterDurationDisplay():SetWidth(self:GetEncounterDurationDisplay():GetFullWidth());
    end
  end
end

-- stacking
function RaidKiller.Mechanic.Manager:GetStacking()
  return self.stacking:GetValue();
end

function RaidKiller.Mechanic.Manager:SetStacking(value)
  self.stacking:SetValue(value);
  
  -- redraw timers
  self:RedrawTimers();
end

-- chronicle
function RaidKiller.Mechanic.Manager:GetChronicle()
  return self.chronicle:GetValue();
end

function RaidKiller.Mechanic.Manager:SetChronicle(value)
  self.chronicle:SetValue(value);
end

-- player details
function RaidKiller.Mechanic.Manager:GetPlayerDetails()
  if (not self.playerDetails) then
    self.playerDetails = Inspect.Unit.Detail("player");
  end
  return self.playerDetails;
end

function RaidKiller.Mechanic.Manager:SetPlayerDetails(value)
  self.playerDetails = value;
end

-- mechanic persistence
function RaidKiller.Mechanic.Manager:GetMechanicPersistence()
  if (type(self.mechanicPersistence:GetValue()) ~= "table") then
    self.mechanicPersistence:SetValue({});
  end
  return self.mechanicPersistence:GetValue();
end

function RaidKiller.Mechanic.Manager:SetMechanicPersistence(value)
  self.mechanicPersistence:SetValue(value);
end

-- encounter state
function RaidKiller.Mechanic.Manager:GetState()
  return self.state;
end

function RaidKiller.Mechanic.Manager:SetState(value)
  self.state = value;
end

-- get primary unit states
function RaidKiller.Mechanic.Manager:GetPrimaryUnitStates()
  if (type(self.primaryUnitStates) ~= "table") then
    self.primaryUnitStates = {};
  end
  return self.primaryUnitStates;
end

-- add primary unit state
function RaidKiller.Mechanic.Manager:AddPrimaryUnitState(value)
  self:GetPrimaryUnitStates()[value:GetUnit()] = value;
end

-- unit states by unit
function RaidKiller.Mechanic.Manager:GetUnitStatesByUnit()
  if (type(self.unitStatesByUnit) ~= "table") then
    self.unitStatesByUnit = {};
  end
  return self.unitStatesByUnit;
end

function RaidKiller.Mechanic.Manager:SetUnitStatesByUnit(value)
  self.unitStatesByUnit = value;
end

-- unit states by ID
function RaidKiller.Mechanic.Manager:GetUnitStatesByID()
  if (type(self.unitStatesByID) ~= "table") then
    self.unitStatesByID = {};
  end
  return self.unitStatesByID;
end

function RaidKiller.Mechanic.Manager:SetUnitStatesByID(value)
  self.unitStatesByID = value;
end

-- clear unit states
function RaidKiller.Mechanic.Manager:ClearUnitStates()
  self:SetUnitStatesByUnit();
  self:SetUnitStatesByID();
end

-- mechanics
function RaidKiller.Mechanic.Manager:GetMechanics()
  if (type(self.mechanics) ~= "table") then
    self.mechanics = {};
  end
  return self.mechanics;
end

function RaidKiller.Mechanic.Manager:SetMechanics(value)
  self.mechanics = value;
end

-- clear mechanics
function RaidKiller.Mechanic.Manager:ClearMechanics()
  self:SetMechanics();
end

-- buff triggers
function RaidKiller.Mechanic.Manager:GetBuffTriggers()
  if (type(self.buffTriggers) ~= "table") then
    self.buffTriggers = {};
  end
  return self.buffTriggers;
end

function RaidKiller.Mechanic.Manager:SetBuffTriggers(value)
  self.buffTriggers = value;
end

-- cast triggers
function RaidKiller.Mechanic.Manager:GetCastTriggers()
  if (type(self.castTriggers) ~= "table") then
    self.castTriggers = {};
  end
  return self.castTriggers;
end

function RaidKiller.Mechanic.Manager:SetCastTriggers(value)
  self.castTriggers = value;
end

-- notify triggers
function RaidKiller.Mechanic.Manager:GetNotifyTriggers()
  if (type(self.notifyTriggers) ~= "table") then
    self.notifyTriggers = {};
  end
  return self.notifyTriggers;
end

function RaidKiller.Mechanic.Manager:SetNotifyTriggers(value)
  self.notifyTriggers = value;
end

-- speak triggers
function RaidKiller.Mechanic.Manager:GetSpeakTriggers()
  if (type(self.speakTriggers) ~= "table") then
    self.speakTriggers = {};
  end
  return self.speakTriggers;
end

function RaidKiller.Mechanic.Manager:SetSpeakTriggers(value)
  self.speakTriggers = value;
end

-- timer triggers
function RaidKiller.Mechanic.Manager:GetTimerTriggers()
  if (type(self.timerTriggers) ~= "table") then
    self.timerTriggers = {};
  end
  return self.timerTriggers;
end

function RaidKiller.Mechanic.Manager:SetTimerTriggers(value)
  self.timerTriggers = value;
end

-- engage triggers
function RaidKiller.Mechanic.Manager:GetEngageTriggers()
  if (type(self.engageTriggers) ~= "table") then
    self.engageTriggers = {};
  end
  return self.engageTriggers;
end

function RaidKiller.Mechanic.Manager:SetEngageTriggers(value)
  self.engageTriggers = value;
end

-- death triggers
function RaidKiller.Mechanic.Manager:GetDeathTriggers()
  if (type(self.deathTriggers) ~= "table") then
    self.deathTriggers = {};
  end
  return self.deathTriggers;
end

function RaidKiller.Mechanic.Manager:SetDeathTriggers(value)
  self.deathTriggers = value;
end

-- phase triggers
function RaidKiller.Mechanic.Manager:GetPhaseTriggers()
  if (type(self.phaseTriggers) ~= "table") then
    self.phaseTriggers = {};
  end
  return self.phaseTriggers;
end

function RaidKiller.Mechanic.Manager:SetPhaseTriggers(value)
  self.phaseTriggers = value;
end

-- percentage triggers
function RaidKiller.Mechanic.Manager:GetPercentageTriggers()
  if (type(self.percentageTriggers) ~= "table") then
    self.percentageTriggers = {};
  end
  return self.percentageTriggers;
end

function RaidKiller.Mechanic.Manager:SetPercentageTriggers(value)
  self.percentageTriggers = value;
end

-- clear triggers
function RaidKiller.Mechanic.Manager:ClearTriggers()
  self:SetBuffTriggers();
  self:SetCastTriggers();
  self:SetNotifyTriggers();
  self:SetSpeakTriggers();
  self:SetTimerTriggers();
  self:SetEngageTriggers();
  self:SetDeathTriggers();
  self:SetPhaseTriggers();
  self:SetPercentageTriggers();
end

-- last action time
function RaidKiller.Mechanic.Manager:GetLastActionTime()
  return self.lastActionTime;
end

function RaidKiller.Mechanic.Manager:SetLastActionTime(value)
  self.lastActionTime = value;
end

-- duration
function RaidKiller.Mechanic.Manager:GetDuration()
  return self.duration;
end

function RaidKiller.Mechanic.Manager:SetDuration(value)
  self.duration = value;
end

-- add content
function RaidKiller.Mechanic.Manager:AddMechanicContent(value)
  
  -- zone
  for k, zone in pairs(value:GetZones()) do
    
    -- register manager
    zone:SetManager(self);
    
    -- persistence
    zone.Wired.Persistence:SetParent(self);
    
    -- difficulties
    for l, difficulty in pairs(zone:GetDifficulties()) do
      
      -- register manager
      difficulty:SetManager(self);
      
      -- encounters
      for m, encounter in pairs(difficulty:GetEncounters()) do
        
        -- register manager
        encounter:SetManager(self);
        encounter:GetState():SetManager(self);
        
        -- mechanics
        for n, mechanic in pairs(encounter:GetMechanics()) do
          
          -- register manager
          mechanic:SetManager(self);
          
          -- triggers
          for o, trigger in pairs(mechanic:GetTriggers()) do
            
            -- register manager
            trigger:SetManager(self);
          end
          
          -- actions
          for o, action in pairs(mechanic:GetActions()) do
            
            -- register manager
            action:SetManager(self);
          end
        end
        
        -- unit states
        for n, state in pairs(encounter:GetState():GetUnitStates()) do
          
          -- register manager
          state:SetManager(self);
          
          -- primary
          if (state:GetPrimary()) then
            self:AddPrimaryUnitState(state);
          end
        end
      end
    end
  end
end

-- activate trigger
function RaidKiller.Mechanic.Manager:ActivateTrigger(trigger, settings)
  
  -- debug
  self:Debug("Attempting to activate: " .. trigger:GetMechanic():GetName());
  
  -- phase restricted
  local restricted = false;
  if (trigger:GetPhaseRestriction()) then
    
    -- debug
    self:Debug("Phase Restriction: " .. trigger:GetPhaseRestriction());
    
    -- match phase
    if (trigger:GetPhaseRestriction() ~= self:GetState():GetPhase()) then
      restricted = true;
    end
  end
  
  -- unrestricted
  if (not restricted) then
    
    -- debug
    self:Debug("Activating: " .. trigger:GetMechanic():GetName());
    
    -- activate
    trigger:GetMechanic():Activate(trigger:CopySettings(settings), trigger);
    
  -- restricted
  else
    
    -- debug
    self:Debug("Failed to activate: " .. trigger:GetMechanic():GetName());
  end
end

-- triggers

-- buff
function RaidKiller.Mechanic.Manager:Buff(unit, buffs)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- details
      local details = Inspect.Unit.Detail(unit);
      if (details) then
        
        -- buff details
        local buffDetails = Inspect.Buff.Detail(unit, buffs);
        if (buffDetails) then
          for k, buff in pairs(buffDetails) do
            
            -- triggers
            for l, trigger in pairs(self:GetBuffTriggers()) do
              
              -- activate
              local activate = false;
              
              -- match buff name
              if (buff.name == trigger:GetAbility()) then
                
                -- match mob name or any unit
                if (trigger:GetUnit()) then
                  if (
                    (details.name == trigger:GetUnit()) or
                    (RaidKiller.Mechanic.Trigger.Object.Constants.Any == trigger:GetUnit())
                  ) then
                    
                    -- activate
                    activate = true;
                  end
                  
                -- no unit
                else
                  
                  -- match role
                  if (trigger:GetRole()) then
                    if (details.role == trigger:GetRole()) then
                      
                      -- activate
                      activate = true;
                    end
                    
                  -- match player name
                  elseif (self:GetPlayerDetails()) then
                    if (details.name == self:GetPlayerDetails().name) then
                      
                      -- activate
                      activate = true;
                    end
                  end
                end
              end
              
              -- stacks
              if (trigger:GetStacks()) then
                
                -- match stacks
                if (buff.stack ~= trigger:GetStacks()) then
                  
                  -- deactivate
                  activate = false;
                end
              end
              
              -- activate
              if (activate) then
                
                -- stacks
                local stacks = "";
                if (buff.stack) then
                  if (buff.stack > 1) then
                    stacks = tostring(buff.stack);
                  end
                end
                
                -- settings
                local settings = {
                  player = details.name
                };
                
                -- bypass duration
                if (not trigger:GetBypassDuration()) then
                  settings.duration = buff.remaining;
                end
                
                -- bypass stacks
                if (not trigger:GetBypassStacks()) then
                  settings.mark = stacks;
                end
                
                -- activate
                self:ActivateTrigger(trigger, settings);
              end
            end
          end
        end
      end
    end
  end
end

-- cast
function RaidKiller.Mechanic.Manager:Cast(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- bars
      local bars = Inspect.Unit.Castbar(value);
      if (bars) then
        for unit, bar in pairs(bars) do
          
          -- details
          local details = Inspect.Unit.Detail(unit);
          if (details) then
            
            -- triggers
            for k, trigger in pairs(self:GetCastTriggers()) do
              
              -- activate
              local activate = false;
              
              -- match ability
              if (
                (bar.abilityName == trigger:GetAbility()) and
                
                -- match unit or any unit
                (
                  (not trigger:GetUnit()) or
                  (details.name == trigger:GetUnit())
                )
              ) then
                
                -- check cycle
                if (trigger:GetCycle()) then
                  
                  -- check leeway
                  local time = Inspect.Time.Frame();
                  if (trigger:GetStart()) then
                    if ((time - trigger:GetStart()) >= trigger:GetCycle().timeout) then
                      trigger:SetIteration(0);
                    end
                  end
                  trigger:SetStart(time);
                  
                  -- iterate
                  if (not trigger:GetIteration()) then
                    trigger:SetIteration(0);
                  end
                  trigger:SetIteration(trigger:GetIteration() + 1);
                  
                  -- check mod
                  for i, mod in pairs(trigger:GetCycle().mod) do
                    if (mod == math.mod(trigger:GetIteration(), trigger:GetCycle().length)) then
                      
                      -- activate
                      activate = true;
                    end
                  end
                  
                -- no cycle
                else
                  
                  -- activate
                  activate = true;
                end
              end
              
              -- activate
              if (activate) then
                self:ActivateTrigger(trigger);
              end
            end
          end
        end
      end
    end
  end
end

-- notify
function RaidKiller.Mechanic.Manager:Notify(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- message
      local message = nil
      if (value) then
        message = value.message
      end
      
      -- debug
      self:Debug("Notify Message:");
      self:Debug(message);
      
      -- details
      local details = nil
      if (message) then
        details = self:GetPlayerDetails();
      end
      
      -- parse
      if (details) then
        
        -- triggers
        for k, trigger in pairs(self:GetNotifyTriggers()) do
          
          -- activate
          local activate = false;
          
          -- match message
          if (string.find(trigger:MatchMessage(details.name), message)) then
            
            -- activate
            activate = true;
          end
          
          -- activate
          if (activate) then
            self:ActivateTrigger(trigger);
          end
        end
      end
    end
  end
end

-- speak
function RaidKiller.Mechanic.Manager:Speak(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- message
      local message = nil
      if (value) then
        message = value.message
      end
      
      -- debug
      self:Debug("Speak Message:");
      self:Debug(message);
      
      -- details
      local details = nil
      if (message) then
        details = self:GetPlayerDetails();
      end
      
      -- parse
      if (details) then
        
        -- triggers
        for k, trigger in pairs(self:GetSpeakTriggers()) do
          
          -- activate
          local activate = false;
          
          -- match message
          if (string.find(trigger:MatchMessage(details.name), message)) then
            
            -- match unit
            if (trigger:GetUnit()) then
              if (value.fromName == trigger:GetUnit()) then
                
                -- activate
                activate = true;
              end
            else
              
              -- activate
              activate = true;
            end
          end
          
          -- activate
          if (activate) then
            self:ActivateTrigger(trigger);
          end
        end
      end
    end
  end
end

-- encounter

-- idle
function RaidKiller.Mechanic.Manager:Idle()
  
  -- engaged
  if (self:GetState()) then
    
    -- elapsed
    local elapsed = (
      self:GetIdle():GetPast() -
      self:GetLastActionTime()
    );
    
    -- timeout
    if (elapsed >= self:GetState():GetTimeout()) then
      
      -- failed
      self:GetState():Failed();
    end
    
    -- update encounter display
    self:UpdateEncounterDisplay();
  end
end

-- start encounter
function RaidKiller.Mechanic.Manager:StartEncounter(state)
  
  -- time
  local time = Inspect.Time.Frame();
  
  -- last action timed out
  if (
    (not self:GetLastActionTime()) or
    (
      (time - self:GetLastActionTime()) >
      RaidKiller.Mechanic.Manager.Defaults.Timeout
    )
  ) then
    
    -- update last action time
    self:SetLastActionTime(time);
    
    -- encounter state
    self:SetState(state);
    
    -- clear mechanics
    self:ClearMechanics();
    
    -- clear triggers
    self:ClearTriggers();
    
    -- clear unit states
    self:ClearUnitStates();
    
    -- unit states
    for k, unitState in pairs(self:GetState():GetUnitStates()) do
      
      -- reset
      unitState:Reset();
      
      -- index by unit
      self:GetUnitStatesByUnit()[unitState:GetUnit()] = unitState;
    end
    
    -- mechanics
    for k, mechanic in pairs(self:GetState():GetEncounter():GetMechanics()) do
      
      -- index by name
      self:GetMechanics()[mechanic:GetName()] = mechanic;
      
      -- triggers
      for l, trigger in pairs(mechanic:GetTriggers()) do
        
        -- buff trigger
        if (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Buff) then
          
          -- index
          table.insert(self:GetBuffTriggers(), trigger);
          
        -- cast trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Cast) then
          
          -- index
          table.insert(self:GetCastTriggers(), trigger);
          
        -- notify trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Notify) then
          
          -- index
          table.insert(self:GetNotifyTriggers(), trigger);
          
        -- speak trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Speak) then
          
          -- index
          table.insert(self:GetSpeakTriggers(), trigger);
          
        -- timer trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Timer) then
          
          -- index
          table.insert(self:GetTimerTriggers(), trigger);
          
        -- engage trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Engage) then
          
          -- index
          table.insert(self:GetEngageTriggers(), trigger);
          
        -- death trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Death) then
          
          -- index
          table.insert(self:GetDeathTriggers(), trigger);
          
        -- phase trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Phase) then
          
          -- index
          table.insert(self:GetPhaseTriggers(), trigger);
          
        -- percentage trigger
        elseif (trigger:GetType() == RaidKiller.Mechanic.Trigger.Object.Constants.Types.Percentage) then
          
          -- reset reached
          trigger:SetReached(false);
          
          -- index by unit
          if (type(self:GetPercentageTriggers()[trigger:GetUnit()]) ~= "table") then
            self:GetPercentageTriggers()[trigger:GetUnit()] = {};
          end
          table.insert(self:GetPercentageTriggers()[trigger:GetUnit()], trigger);
        end
      end
    end
    
    -- engaged
    self:GetState():Engaged();
    
    -- update encounter display
    self:UpdateEncounterDisplay();
    
    -- show encounter display
    self:ShowEncounterDisplay();
    
    -- animation
    self:GetTrayGlowAnimation():Start();
  end
end

-- stop encounter
function RaidKiller.Mechanic.Manager:StopEncounter()
  
  -- last action time
  self:SetLastActionTime(Inspect.Time.Frame());
  
  -- animation
  self:GetTrayGlowAnimation():Stop();
  
  -- timers
  for k, timer in pairs(self:GetActiveTimers()) do
    
    -- deactivate
    if (timer:GetVisible()) then
      timer:SetActive(false);
      timer:SetVisible(false);
    end
  end
  
  -- encounter state
  self:SetState();
  
  -- update encounter display
  self:UpdateEncounterDisplay();
  
  -- show encounter display
  self:ShowEncounterDisplay();
end

-- lookup unit state
function RaidKiller.Mechanic.Manager:LookupUnitState(ID)
  
  -- details
  local details = nil;
  if (ID) then
    details = Inspect.Unit.Detail(ID);
  end
  
  -- name
  local unit = nil;
  if (details) then
    if (details.name) then
      unit = details.name;
    end
  end
  
  -- unit
  if (unit) then
    
    -- engaged
    if (self:GetState()) then
      
      -- unit state
      local unitState = self:GetUnitStatesByUnit()[unit];
      if (unitState) then
        
        -- new ID
        if (ID ~= unitState:GetID()) then
          
          -- remove old index
          if (unitState:GetID()) then
            self:GetUnitStatesByID()[unitState:GetID()] = nil;
          end
          
          -- update ID
          unitState:SetID(ID);
          
          -- index by ID
          self:GetUnitStatesByID()[ID] = unitState;
          
          -- set health
          unitState:SetHealthMax(details.healthMax);
          unitState:SetHealth(details.health);
        end
        
        -- update last action time
        self:SetLastActionTime(Inspect.Time.Frame());
      end
      
    -- not engaged
    else
      
      -- primary
      local primary = self:GetPrimaryUnitStates()[unit];
      if (primary) then
        
        -- chronicle
        local chronicle = false;
        if (primary:GetChronicle()) then
          if (details.healthMax) then
            if (details.healthMax < RaidKiller.State.Unit.Constants.Chronicle) then
              chronicle = true;
            end
          end
        end
        
        -- showing chronicles or not a chronicle
        if (self:GetChronicle() or (not chronicle)) then
          self:StartEncounter(primary:GetState());
        else
          
          -- debug
          self:Debug("Bypassing chronicle encounter.");
        end
      end
    end
  end
end

-- damage
function RaidKiller.Mechanic.Manager:Damage(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
      
    -- value
    if (value) then
      
      -- lookup caster
      if (value.caster) then
        self:LookupUnitState(value.caster);
      end
      
      -- lookup target
      if (value.target) then
        self:LookupUnitState(value.target);
      end
    end
  end
end

-- check unit state
function RaidKiller.Mechanic.Manager:UpdateUnitState(data)
  
  -- unit state
  local unitState = self:GetUnitStatesByID()[data.ID];
  if (unitState) then
    
    -- health max
    if (data.healthMax) then
      unitState:SetHealthMax(data.healthMax);
    end
    
    -- health
    if (data.health) then
      unitState:SetHealth(data.health);
    end
  end
end

-- health max
function RaidKiller.Mechanic.Manager:HealthMax(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- health max
      for ID, healthMax in pairs(value) do
        if (healthMax) then
          self:UpdateUnitState({ID = ID, healthMax = healthMax});
        end
      end
    end
  end
end

-- health
function RaidKiller.Mechanic.Manager:Health(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
    
      -- health
      for ID, health in pairs(value) do
        if (health) then
          self:UpdateUnitState({ID = ID, health = health});
        end
      end
    end
  end
end

-- death
function RaidKiller.Mechanic.Manager:Death(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- value
      local details = nil;
      if (value) then
        if (value.target) then
          details = Inspect.Unit.Detail(value.target);
        end
      end
      
      -- name
      local unit = nil;
      if (details) then
        if (details.name) then
          unit = details.name;
        end
      end
      
      -- unit state
      local unitState = nil;
      if (unit) then
        unitState = self:GetUnitStatesByUnit()[unit];
      end
      
      -- exists
      if (unitState) then
        
        -- died
        unitState:Died();
      end
    end
  end
end

-- triggers

-- timer
function RaidKiller.Mechanic.Manager:TimerFinished(action, timer)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- engaged
    if (self:GetState()) then
      
      -- triggers
      for j, trigger in pairs(self:GetTimerTriggers()) do
        
        -- activate
        local activate = false;
        
        -- match mechanic
        if (action:GetMechanic():GetName() == trigger:GetTargetMechanic()) then
          
          -- activate
          activate = true;
          
          -- match duration
          if (trigger:GetDuration()) then
            if (timer:GetScale():GetMaximum() ~= trigger:GetDuration()) then
              
              -- activate
              activate = false;
            end
          end
          
          -- match title
          if (trigger:GetMessage()) then
            if (timer:GetTitle() ~= trigger:GetMessage()) then
              
              -- activate
              activate = false;
            end
          end
        end
        
        -- activate
        if (activate) then
          self:ActivateTrigger(trigger);
        end
      end
    end
  end
end

-- engaged
function RaidKiller.Mechanic.Manager:Engaged()
  
  -- triggers
  for k, trigger in pairs(self:GetEngageTriggers()) do
    
    -- activate
    self:ActivateTrigger(trigger);
  end
end

-- health percentage changed
function RaidKiller.Mechanic.Manager:HealthPercentageChanged(value)

  -- triggers
  local triggers = self:GetPercentageTriggers()[value:GetUnit()];
  if (triggers) then
    for k, trigger in pairs(triggers) do
      
      -- reached percent
      if (
        (not trigger:GetReached()) and
        (trigger:GetPercent() >= value:GetHealthPercentage())
      ) then
        
        -- reached
        trigger:SetReached(true);
        
        -- activate
        self:ActivateTrigger(
          trigger,
          {
            percent = value:GetHealthPercentage()
          }
        );
      end
    end
  end
end

-- died
function RaidKiller.Mechanic.Manager:Died(value)
  
  -- triggers
  for k, trigger in pairs(self:GetDeathTriggers()) do
    
    -- activate
    local activate = false;
    
    -- check
    local check = false;
    
    -- units
    for l, unit in pairs(trigger:GetUnits()) do
      
      -- match unit
      if (unit == value:GetUnit()) then
        check = true;
      end
    end
    
    -- check
    if (check) then
      
      -- activate
      activate = true;
      
      -- units
      for l, unit in pairs(trigger:GetUnits()) do
        
        -- dead
        local dead = false;
        
        -- state
        local state = self:GetUnitStatesByUnit()[unit];
        if (state) then
          
          -- died
          if (state:GetDeaths() > 0) then
            
            -- match deaths
            if (
              (not trigger:GetDeaths()) or
              (state:GetDeaths() == trigger:GetDeaths())
            ) then
              
              -- activate
              dead = true;
            end
          end
        end
        
        -- deactivate
        if (not dead) then
          activate = false;
        end
      end
    end
    
    -- activate
    if (activate) then
      self:ActivateTrigger(trigger);
    end
  end
end

-- phase entered
function RaidKiller.Mechanic.Manager:PhaseEntered(phase)
  
  -- debug
  self:Debug("Phase Entered: " .. phase:GetName());
  for k, objective in pairs(phase:GetObjectivesByOrder()) do
    self:Debug("Objective: " .. objective:GetMessage() .. " " .. objective:GetProgress() .. " / " .. objective:GetGoal());
  end
  
  -- encounter phase display
  self:GetEncounterPhaseDisplay():SetText(phase:GetName());
  --self:GetEncounterPhaseDisplay():SetWidth(self:GetEncounterPhaseDisplay():GetFullWidth());
  
  -- show encounter display
  self:ShowEncounterDisplay();
  
  -- triggers
  for k, trigger in pairs(self:GetPhaseTriggers()) do
    
    -- match phase
    if ((trigger:GetPhase() == phase:GetOrder()) and trigger:GetEntered()) then
      
      -- activate
      self:ActivateTrigger(trigger);
    end
  end
end

-- phase exited
function RaidKiller.Mechanic.Manager:PhaseExited(phase)
  
  -- debug
  self:Debug("Phase Exited: " .. phase:GetName());
  
  -- triggers
  for k, trigger in pairs(self:GetPhaseTriggers()) do
    
    -- match phase
    if ((trigger:GetPhase() == phase:GetOrder()) and trigger:GetExited()) then
      
      -- activate
      self:ActivateTrigger(trigger);
    end
  end
end

-- objective progressed
function RaidKiller.Mechanic.Manager:ObjectiveProgressed(objective)
  
  -- debug
  self:Debug("Objective Progressed: " .. objective:GetMessage() .. " " .. objective:GetProgress() .. " / " .. objective:GetGoal());
  
  -- show encounter display
  self:ShowEncounterDisplay();
end

-- active timers
function RaidKiller.Mechanic.Manager:GetActiveTimers()
  if (type(self.activeTimers) ~= "table") then
    self.activeTimers = {};
  end
  return self.activeTimers;
end

function RaidKiller.Mechanic.Manager:SetActiveTimers(value)
  self.activeTimers = value;
end

function RaidKiller.Mechanic.Manager:RedrawTimers()
  
  -- sorted
  local sorted = {};
  
  -- timers
  for k, timer in pairs(self:GetActiveTimers()) do
    
    -- only draw visible and unlocked
    if (timer:GetVisible() and (not timer:GetLocked())) then
      table.insert(sorted, timer);
    end
  end
  
  -- sort by duration then name
  table.sort(
    sorted,
    function(a, b)
      local result = a:GetValue() > b:GetValue();
      if (a:GetValue() == b:GetValue()) then
        result = a:GetTitle() < b:GetTitle();
      end
      return result;
    end
  );
  
  -- top
  local top = self:GetTimerBar();
  
  -- timers
  for k, timer in pairs(sorted) do
    
    -- resize
    if (timer:GetFontSize() ~= self:GetTimerBar():GetSize()) then
      timer:SetFontSize(self:GetTimerBar():GetSize());
    end
    
    -- move
    if (self:GetStacking()) then
      timer:SetPosition(
        self:GetTimerBar():GetLeft(),
        top:GetTop() - timer:GetHeight() - RaidKiller.Mechanic.Manager.Defaults.TimerSpacing
      );
    else
      timer:SetPosition(
        self:GetTimerBar():GetLeft(),
        top:GetBottom() + RaidKiller.Mechanic.Manager.Defaults.TimerSpacing
      );
    end
    
    -- next
    top = timer;
  end
end

-- timer visible changed
function RaidKiller.Mechanic.Manager:TimerVisibleChanged(action, timer)
  
  -- visible
  if (timer:GetVisible()) then
    self:GetActiveTimers()[timer:GetTitle()] = timer;
  else
    self:GetActiveTimers()[timer:GetTitle()] = nil;
  end
  
  -- redraw
  self:RedrawTimers();
end

-- debug
function RaidKiller.Mechanic.Manager:Debug(value)
  self:GetApplication():Debug(value);
end
