-----------------------
-- RaidKiller        --
-- By: Wire @ Byriel --
-----------------------

-- class
RaidKiller.State.Phase = class();

-- constructor
function RaidKiller.State.Phase:Constructor(data)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(data.name);
  self:SetObjectives(data.objectives);
  self:SetOrder(data.order);
end

-- name
function RaidKiller.State.Phase:GetName()
  return self.name;
end

function RaidKiller.State.Phase:SetName(value)
  self.name = value;
end

-- objectives
function RaidKiller.State.Phase:GetObjectivesByMessage()
  if (type(self.objectivesByMessage) ~= "table") then
    self.objectivesByMessage = {};
  end
  return self.objectivesByMessage;
end

function RaidKiller.State.Phase:GetObjectivesByOrder()
  if (type(self.objectivesByOrder) ~= "table") then
    self.objectivesByOrder = {};
  end
  return self.objectivesByOrder;
end

function RaidKiller.State.Phase:SetObjectives(value)
  
  -- objectives
  for k, objective in pairs(value) do
    
    -- index by message
    self:GetObjectivesByMessage()[objective:GetMessage()] = objective;
    
    -- index by order
    self:GetObjectivesByOrder()[objective:GetOrder()] = objective;
    
    -- register
    objective:SetPhase(self);
  end
end

-- order
function RaidKiller.State.Phase:GetOrder()
  return self.order;
end

function RaidKiller.State.Phase:SetOrder(value)
  self.order = value;
end

-- manager
function RaidKiller.State.Phase:GetManager()
  return self.manager;
end

function RaidKiller.State.Phase:SetManager(value)
  self.manager = value;
  
  -- update objectives
  for k, objective in pairs(self:GetObjectivesByOrder()) do
    objective:SetManager(self:GetManager());
  end
end

-- encounter state
function RaidKiller.State.Phase:GetState()
  return self.state;
end

function RaidKiller.State.Phase:SetState(value)
  self.state = value;
end

-- objective completed
function RaidKiller.State.Phase:ObjectiveCompleted(value)
  
  -- complete
  local complete = true;
  
  -- objectives
  for k, objective in pairs(self:GetObjectivesByOrder()) do
    
    -- check complete
    if (objective:GetRequired() and (not objective:GetComplete())) then
      complete = false;
    end
  end
  
  -- update state
  if (complete) then
    self:GetState():PhaseCompleted(self);
  end
end

-- reset
function RaidKiller.State.Phase:Reset()
 
  -- objectives
  for k, objective in pairs(self:GetObjectivesByOrder()) do
    objective:Reset();
  end
end
