-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.IconTray = class();

-- constants
Wired.IconTray.Constants = {
  Texture = {
    Pin = {
      On = Wired.UI.Texture("Wired", "Resources/Pin/on.png"),
      Off = Wired.UI.Texture("Wired", "Resources/Pin/off.png"),
    }
  }
};

-- defaults
Wired.IconTray.Defaults = {
  Open = false,
  Position = {
    Left = UIParent:GetWidth() / 2,
    Top = UIParent:GetHeight() / 10
  },
  Cell = {
    Padding = 0,
    Spacing = 2,
    Size = {
      width = 32,
      height = 32
    }
  },
  Animation = {
    Duration = 0.25
  }
};

function Wired.IconTray:Constructor(name, parent, application)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  self:SetApplication(application);
  self:SetIdle(self:GetApplication():GetIdle());
    
  -- components
  
  -- pin
  self:SetPin(ApplicationTrayIcon(self:GetName() .. "_Pin", self:GetParent(), self:GetApplication()));
  self:GetPin():SetTextures(Wired.IconTray.Constants.Texture.Pin);
  self:GetPin():GetBase():SetLayer(2);
  self:GetPin():GetBase().Wired.UI.Drag:Enable(true);
  
  -- tray
  self:SetTray(Wired.UI.Trion.Mask(self:GetName() .. "_Tray", self:GetParent()));
  self:GetTray():SetLayer(1);
  self:GetTray():SetColor(Wired.UI.Color.Clear);
  self:GetTray():SetWidth(self:GetPin():GetBase():GetWidth() / 2);
  self:GetTray():SetHeight(self:GetPin():GetBase():GetHeight());
  self:GetTray():SetVisible(true);
  
  -- list
  self:SetList(Wired.UI.List(self:GetName() .. "_List", self:GetTray()));
  self:GetList():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = 0, y = 0});
  self:GetList():SetOrientation(Wired.UI.List.Constants.Orientation.Horizontal);
  self:GetList():SetSize(0);
  self:GetList():SetColor(Wired.UI.Color.Clear);
  self:GetList():SetVisible(true);
  
  -- apply defaults
  self:ApplyIconTrayDefaults();
  
  -- events
  
  -- pin moved
  self:GetPin():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function(sender, value)
      self:UpdatePosition();
    end
  ));
  
  -- pin clicked
  self:GetPin():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "StationaryClick",
    function(sender, value)
      
      -- toggle open
      self:SetOpen(not self:GetOpen());
      
      -- update drag
      self:UpdateDrag();
      
      -- animation
      if (self:GetOpen()) then
        self:GetOpenAnimation():Start();
      else
        self:GetCloseAnimation():Start();
      end
    end,
    true
  ));
  
  -- right click
  self:GetPin():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      WiredApplicationManager:EditLayout();
    end
  ));
  
  -- application registered
  WiredApplicationManager.Wired.Event.Listener:Add(Wired.Event.Listener(
    "ApplicationRegistered",
    function(sender, value)
      self:CreateIcon(value);
    end,
    true
  ));
  
  -- persistence
  
  -- open
  self.open = Wired.Persistence.PersistentVariable(
    self:GetName() .. "_Open", self:GetParent():GetBase(), Wired.IconTray.Defaults.Open
  );
  
  -- position
  self.position = Wired.Persistence.PersistentVariable(
    self:GetName() .. "_Position", self:GetParent():GetBase(), Wired.IconTray.Defaults.Position
  );
  
  -- load
  self:Load();
end

-- name
function Wired.IconTray:GetName()
  return self.name;
end

function Wired.IconTray:SetName(value)
  self.name = value;
end

-- parent
function Wired.IconTray:GetParent()
  return self.parent;
end

function Wired.IconTray:SetParent(value)
  self.parent = value;
end

-- application
function Wired.IconTray:GetApplication()
  return self.application;
end

function Wired.IconTray:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.IconTray:GetIdle()
  return self.idle;
end

function Wired.IconTray:SetIdle(value)
  self.idle = value;
end

-- pin
function Wired.IconTray:GetPin()
  return self.pin;
end

function Wired.IconTray:SetPin(value)
  self.pin = value;
end

-- tray
function Wired.IconTray:GetTray()
  return self.tray;
end

function Wired.IconTray:SetTray(value)
  self.tray = value;
end

-- list
function Wired.IconTray:GetList()
  return self.list;
end

function Wired.IconTray:SetList(value)
  self.list = value;
end

-- open
function Wired.IconTray:GetOpen()
  return self.open:GetValue();
end

function Wired.IconTray:SetOpen(value)
  self.open:SetValue(value);
end

-- position
function Wired.IconTray:GetPosition()
  return self.position:GetValue();
end

function Wired.IconTray:SetPosition(value)
  self.position:SetValue(value);
end

-- open animation
function Wired.IconTray:GetOpenAnimation()
  if (not self.openAnimation) then
    self:CreateOpenAnimation();
  end
  return self.openAnimation;
end

function Wired.IconTray:SetOpenAnimation(value)
  self.openAnimation = value;
end

-- close animation
function Wired.IconTray:GetCloseAnimation()
 if (not self.closeAnimation) then
    self:CreateCloseAnimation();
  end
  return self.closeAnimation;
end

function Wired.IconTray:SetCloseAnimation(value)
  self.closeAnimation = value;
end

-- defaults
function Wired.IconTray:ApplyIconTrayDefaults(value)
  
  -- list
  self:GetList():SetCellSize(Wired.IconTray.Defaults.Cell.Size);
  self:GetList():SetPadding(Wired.IconTray.Defaults.Cell.Padding);
end

-- create open animation
function Wired.IconTray:CreateOpenAnimation(value)
  
  -- animation
  self:SetOpenAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Open",
    cycles = 1
  }));
  
  -- steps
  self:GetOpenAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Grow",
    value = (self:GetList():GetCellSize().width / 2),
    minimum = (self:GetList():GetCellSize().width / 2),
    maximum = self:GetList():GetWidth() + (self:GetList():GetCellSize().width / 2) + self:GetList():GetCellSpacing(),
    duration = Wired.IconTray.Defaults.Animation.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self:GetTray()},
      width = true
    })}
  }));
end

-- create close animation
function Wired.IconTray:CreateCloseAnimation(value)
  
  -- animation
  self:SetCloseAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Close",
    cycles = 1
  }));
  
  -- steps
  self:GetCloseAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Shrink",
    value = self:GetList():GetWidth() + (self:GetList():GetCellSize().width / 2) + self:GetList():GetCellSpacing(),
    minimum = (self:GetList():GetCellSize().width / 2),
    maximum = self:GetList():GetWidth() + (self:GetList():GetCellSize().width / 2) + self:GetList():GetCellSpacing(),
    duration = Wired.IconTray.Defaults.Animation.Duration,
    direction = -1,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self:GetTray()},
      width = true
    })}
  }));
end

function Wired.IconTray:CreateIcon(value)
  local icon = ApplicationTrayIcon(value:GetName() .. "_TrayIcon", self:GetList(), value);
  
  -- textures
  local textures = value:GetIconTrayTextures();
  if (not textures) then
    textures = Wired.IconTray.Constants.Texture.Pin;
  end
  icon:SetTextures(textures);
  
  -- left click
  icon:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      
      -- found
      local found = false;
      
      -- options
      local options = icon:GetApplication():GetOptions();
      if (options) then
        
        -- toggle
        options:SetVisible(not options:GetVisible());
        
        -- found
        found = true;
      end
      
      -- universe
      local universe = icon:GetApplication().universe;
      if (universe) then
        
        -- toggle
        universe:SetVisible(not universe:GetVisible());
        
        -- found
        found = true;
      end
      
      -- found
      if (not found) then
        print("No options for " .. icon:GetApplication():GetName());
      end
    end
  ));
  
  -- right click
  icon:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      WiredApplicationManager:EditLayout();
    end
  ));
  
  -- list
  self:GetList():AddCell(icon);
  self:GetList():SetSize(self:GetList():GetSize() + 1);
  
  -- tray
  if (self:GetOpen()) then
    self:GetTray():SetWidth(
      self:GetList():GetWidth() +
      (self:GetList():GetCellSize().width / 2) +
      self:GetList():GetCellSpacing()
    );
  else
    self:GetTray():SetWidth(
      self:GetList():GetCellSize().width / 2
    );
  end
end

-- update position
function Wired.IconTray:UpdatePosition()
  
  -- pin
  self:SetPosition({
    Left = self:GetPin():GetBase():GetLeft(),
    Top = self:GetPin():GetBase():GetTop()
  });
  
  -- tray
  self:GetTray():SetPosition(
    (
      self:GetPin():GetBase():GetLeft() +
      (self:GetList():GetCellSize().width / 2)
    ),
    self:GetPin():GetBase():GetTop()
  );
end

function Wired.IconTray:UpdateDrag()
  self:GetPin():GetBase().Wired.UI.Drag.data.enabled = not self:GetOpen();
end

-- persistence

-- load
function Wired.IconTray:Load()
  
  -- position
  self:GetPin():SetPosition(
    self:GetPosition().Left,
    self:GetPosition().Top
  );
  
  -- drag
  self:UpdateDrag();
end
