-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Animation.Alert = class(Wired.UI.TextShadow);

-- defaults
Wired.UI.Animation.Alert.Defaults = {
  Text = "Alert",
  Layer = 100,
  Animation = {
    Duration = 2.00,
    Color = {
      Minimum = 0.75,
      Maximum = 1.00,
      Duration = 0.25
    },
    Opacity = {
      Minimum = 0.00,
      Maximum = 1.00,
      Duration = 0.25
    },
    Pause = {
      Minimum = 0.00,
      Maximum = 1.00,
      Duration = 1.50
    },
    Size = {
      Minimum = 30,
      Maximum = 80,
      Duration = 0.25
    },
    FontSize = {
      Minimum = 8,
      Maximum = 64,
      Duration = 0.25
    }
  }
};

function Wired.UI.Animation.Alert:Constructor(application)
  Wired.UI.TextShadow.Constructor(self, "Alert", application:GetContext());
  
  -- parameters
  self:SetApplication(application);
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- glow
  self.glow = Wired.UI.Trion.Texture("Alert_Glow", application:GetContext());
  self.glow:SetColor(Wired.UI.Color.Clear);
  self.glow:SetVisible(false);
  self.glow.GetSize = function(sender)
    return self.glow:GetHeight();
  end
  self.glow.SetSize = function(sender, value)
    self.glow:SetWidth(self:GetWidth());
    self.glow:SetHeight(self:GetWidth() / 2.5);
  end
  
  -- animation
  self:CreateAlertAnimation();
  self:CreateGlowAnimation();
  
  -- defaults
  self:ApplyAlertDefaults();
end

-- application
function Wired.UI.Animation.Alert:GetApplication()
  return self.application;
end

function Wired.UI.Animation.Alert:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.UI.Animation.Alert:GetIdle()
  return self.idle;
end

function Wired.UI.Animation.Alert:SetIdle(value)
  self.idle = value;
end

-- duration
function Wired.UI.Animation.Alert:GetDuration()
  if (not self.duration) then
    self.duration = Wired.UI.Animation.Alert.Defaults.Animation.Duration;
  end
  return self.duration;
end

function Wired.UI.Animation.Alert:SetDuration(value)
  self.duration = value;
end

-- alert animation
function Wired.UI.Animation.Alert:GetAlertAnimation()
  return self.alertAnimation;
end

function Wired.UI.Animation.Alert:SetAlertAnimation(value)
  self.alertAnimation = value;
end

-- glow animation
function Wired.UI.Animation.Alert:GetGlowAnimation()
  return self.glowAnimation;
end

function Wired.UI.Animation.Alert:SetGlowAnimation(value)
  self.glowAnimation = value;
end

-- alert color
function Wired.UI.Animation.Alert:GetAlertColor()
  if (not self.alertColor) then
    self.alertColor = self:GetFontColor();
  end
  return self.alertColor;
end

function Wired.UI.Animation.Alert:SetAlertColor(value)
  self.alertColor = value;
  
  -- update animation steps
  for k, compositeStep in pairs(self:GetAlertAnimation():GetSteps()) do
    for l, step in pairs(compositeStep) do
      for l, effect in pairs(step:GetEffects()) do
        if (effect:GetType() == Wired.UI.Animation.Effect.Constants.Types.FontColor) then
          effect:SetColor(self:GetAlertColor());
        end
      end
    end
  end
end

-- defaults
function Wired.UI.Animation.Alert:ApplyAlertDefaults()
  
  -- text
  self:SetText(Wired.UI.Animation.Alert.Defaults.Text);
  
  -- layer
  self:SetLayer(Wired.UI.Animation.Alert.Defaults.Layer);
  
  -- position
  --[[
  self:SetPoint(
    {child = "CENTER", parent = "TOPLEFT"},
    {x = (UIParent:GetWidth() / 2), y = (UIParent:GetHeight() / 8)}
  );
  --]]
  --[[
  self.glow:SetPoint(
    {child = "CENTER", parent = "TOPLEFT"},
    {x = (UIParent:GetWidth() / 2), y = (UIParent:GetHeight() / 8)}
  );
  --]]
end

-- create alert animation
function Wired.UI.Animation.Alert:CreateAlertAnimation()
  
  -- sequence
  self:SetAlertAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Display",
    cycles = 1,
    --duration = {self:GetDuration()}
  }));
    
  -- steps
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Hide",
    value = false,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self}
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Font Size",
    composite = true,
    value = Wired.UI.Animation.Alert.Defaults.Animation.FontSize.Minimum,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self},
      fontSize = true
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Color",
    composite = true,
    value = Wired.UI.Animation.Alert.Defaults.Animation.Color.Maximum,
    effects = {Wired.UI.Animation.FontColorEffect({
      components = {self},
      color = self:GetAlertColor()
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Text Opacity",
    composite = true,
    value = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Minimum,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  --[[
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Pause For Glow",
    value = 0,
    minimum = 0,
    maximum = 1,
    duration = 0.25,
    direction = 1,
    effects = {Wired.UI.Animation.PauseEffect({
      components = {self}
    })}
  }));
  --]]
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Show",
    value = true,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self}
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Grow Font Size",
    value = Wired.UI.Animation.Alert.Defaults.Animation.FontSize.Minimum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.FontSize.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.FontSize.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.FontSize.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self},
      fontSize = true
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade In",
    composite = true,
    value = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Minimum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Darken",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Color.Maximum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Color.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Color.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Color.Duration,
    direction = -1,
    effects = {Wired.UI.Animation.FontColorEffect({
      components = {self},
      color = self:GetAlertColor()
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Pause",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Minimum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.PauseEffect({
      components = {self}
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Lighten",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Color.Minimum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Color.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Color.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Color.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.FontColorEffect({
      components = {self},
      color = self:GetAlertColor()
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Duration,
    direction = -1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  self:GetAlertAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Hide",
    value = false,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self}
    })}
  }));
end

-- create glow animation
function Wired.UI.Animation.Alert:CreateGlowAnimation()
  
  -- sequence
  self:SetGlowAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Glow",
    cycles = 1,
    --duration = {self:GetDuration()}
  }));
  
  -- steps
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Hide",
    value = false,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Texture",
    value = 1,
    effects = {Wired.UI.Animation.TextureEffect({
      components = {self.glow},
      textures = {
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/1.png")
      }
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Size",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Size.Maximum,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self.glow},
      size = true
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Opacity",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Show",
    value = true,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Spin",
    value = 1,
    minimum = 1,
    maximum = 17.99,
    duration = 1.00,
    direction = 1,
    effects = {Wired.UI.Animation.TextureEffect({
      components = {self.glow},
      textures = {
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/1.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/2.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/3.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/4.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/5.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/6.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/7.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/8.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/9.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/10.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/11.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/12.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/13.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/14.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/15.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/16.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/17.png")
      }
    })}
  }));
  --[[
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Grow",
    composite = true,
    value = 1,
    minimum = 1,
    maximum = 17.99,
    duration = 1.00,
    direction = 1,
    effects = {Wired.UI.Animation.SizeEffect({
      components = {self.glow},
      size = true
    })}
  }));
  --]]
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Hide",
    value = false,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Pause",
    value = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Minimum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Maximum,
    duration = Wired.UI.Animation.Alert.Defaults.Animation.Pause.Duration,
    direction = 1,
    effects = {Wired.UI.Animation.PauseEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset Texture",
    value = 1,
    effects = {Wired.UI.Animation.TextureEffect({
      components = {self.glow},
      textures = {
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/1.png")
      }
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Show",
    value = true,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self.glow}
    })}
  }));
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Spin Quarter",
    --composite = true,
    value = 1,
    minimum = 1,
    maximum = 7.99,
    duration = 0.50,
    direction = 1,
    effects = {Wired.UI.Animation.TextureEffect({
      components = {self.glow},
      textures = {
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/1.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/2.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/3.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/4.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/5.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/6.png"),
        Wired.UI.Texture("Wired", "UI/Animation/Resources/Spoke/7.png")
      }
    })}
  }));
  --[[
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade Out",
    composite = true,
    value = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    minimum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Minimum,
    maximum = Wired.UI.Animation.Alert.Defaults.Animation.Opacity.Maximum,
    duration = 0.50,
    direction = -1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self.glow}
    })}
  }));
  --]]
  self:GetGlowAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Hide",
    value = false,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self.glow}
    })}
  }));
end

-- start
function Wired.UI.Animation.Alert:Start(data)
  
  -- hide
  self:SetVisible(false);
  self.glow:SetVisible(false);
  
  -- update values
  self:SetAlertColor(data.color);
  self:SetText(data.message);
  
  -- start animation
  self:GetAlertAnimation():Start();
  --self:GetGlowAnimation():Start();
end
