-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Animation.Flash = class();

-- constants
Wired.UI.Animation.Flash.Constants = {
  Directions = {
    Top = "top",
    Bottom = "bottom",
    Left = "left",
    Right = "right"
  }
};

-- defaults
Wired.UI.Animation.Flash.Defaults = {
  Border = {
    Size = 25,
    Color = Wired.UI.Color.Red
  },
  Animation = {
    Cycles = 3,
    Minimum = 0.00,
    Maximum = 0.90,
    Duration = 1.00
  }
};

function Wired.UI.Animation.Flash:Constructor(application)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetApplication(application);
  self:SetParent(application:GetContext());
  self:SetIdle(application:GetIdle());
  
  -- components
  self:CreateBorder();
  self:CreateFlashAnimation();
  
  -- defaults
  self:ApplyFlashDefaults();
  
  -- global UI scaling
  Wire(UIParent);
  UIParent.Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function()
      self:LayoutBorder();
    end
  ));
end

-- application
function Wired.UI.Animation.Flash:GetApplication()
  return self.application;
end

function Wired.UI.Animation.Flash:SetApplication(value)
  self.application = value;
end

-- parent
function Wired.UI.Animation.Flash:GetParent()
  return self.parent;
end

function Wired.UI.Animation.Flash:SetParent(value)
  self.parent = value;
end

-- idle
function Wired.UI.Animation.Flash:GetIdle()
  return self.idle;
end

function Wired.UI.Animation.Flash:SetIdle(value)
  self.idle = value;
end

-- color
function Wired.UI.Animation.Flash:GetColor()
  if (not self.color) then
    self.color = Wired.UI.Animation.Flash.Defaults.Border.Color;
  end
  return self.color;
end

function Wired.UI.Animation.Flash:SetColor(value)
  self.color = value;
  
  -- update border
  self:SetBorderColor(self:GetColor());
end

-- size
function Wired.UI.Animation.Flash:GetSize()
  return self.size;
end

function Wired.UI.Animation.Flash:SetSize(value)
  self.size = value;
  
  -- layout
  self:LayoutBorder();
end

-- cycles
function Wired.UI.Animation.Flash:GetCycles()
  if (not self.cycles) then
    self.cycles = Wired.UI.Animation.Flash.Defaults.Animation.Cycles;
  end
  return self.cycles;
end

function Wired.UI.Animation.Flash:SetCycles(value)
  self.cycles = value;
  
  -- update animation
  self:GetFlashAnimation():SetCycles(self:GetCycles());
end

-- minimum
function Wired.UI.Animation.Flash:GetMinimum()
  if (not self.minimum) then
    self.minimum = Wired.UI.Animation.Flash.Defaults.Animation.Minimum;
  end
  return self.minimum;
end

function Wired.UI.Animation.Flash:SetMinimum(value)
  self.minimum = value;
  
  -- update animation steps
  for k, compositeStep in pairs(self:GetFlashAnimation():GetSteps()) do
    for l, step in pairs(compositeStep) do
      if (step:GetType() == Wired.UI.Animation.Step.Constants.Types.Timed) then
        step:GetScale():SetMinimum(self:GetMinimum());
      end
    end
  end
end

-- maximum
function Wired.UI.Animation.Flash:GetMaximum()
  if (not self.maximum) then
    self.maximum = Wired.UI.Animation.Flash.Defaults.Animation.Maximum;
  end
  return self.maximum;
end

function Wired.UI.Animation.Flash:SetMaximum(value)
  self.maximum = value;
  
  -- update animation steps
  for k, compositeStep in pairs(self:GetFlashAnimation():GetSteps()) do
    for l, step in pairs(compositeStep) do
      if (step:GetType() == Wired.UI.Animation.Step.Constants.Types.Timed) then
        step:GetScale():SetMaximum(self:GetMaximum());
      end
    end
  end
end

-- duration
function Wired.UI.Animation.Flash:GetDuration()
  if (not self.duration) then
    self.duration = Wired.UI.Animation.Flash.Defaults.Animation.Duration;
  end
  return self.duration;
end

function Wired.UI.Animation.Flash:SetDuration(value)
  self.duration = value;
  
  -- update animation
  self:GetFlashAnimation():SetDuration({self:GetDuration()});
end

-- border
function Wired.UI.Animation.Flash:GetBorder()
  return self.border;
end

function Wired.UI.Animation.Flash:SetBorder(value)
  self.border = value;
end

-- flash animation
function Wired.UI.Animation.Flash:GetFlashAnimation()
  return self.flashAnimation;
end

function Wired.UI.Animation.Flash:SetFlashAnimation(value)
  self.flashAnimation = value;
end

-- defaults
function Wired.UI.Animation.Flash:ApplyFlashDefaults()
  
  -- size
  self:SetSize(Wired.UI.Animation.Flash.Defaults.Border.Size);
end

-- create border
function Wired.UI.Animation.Flash:CreateBorder()
  local border = {};
  
  -- top
  border.top = Wired.UI.Trion.Frame("Flash_Border_Top", self:GetParent());
  border.top:SetVisible(false);
  
  -- bottom
  border.bottom = Wired.UI.Trion.Frame("Flash_Border_Bottom", self:GetParent());
  border.bottom:SetVisible(false);
  
  -- left
  border.left = Wired.UI.Trion.Frame("Flash_Border_Left", self:GetParent());
  border.left:SetVisible(false);
  
  -- right
  border.right = Wired.UI.Trion.Frame("Flash_Border_Right", self:GetParent());
  border.right:SetVisible(false);
  
  -- assign
  self:SetBorder(border);
end

-- create flash animation
function Wired.UI.Animation.Flash:CreateFlashAnimation()
  
  -- sequence
  self:SetFlashAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Flash",
    cycles = self:GetCycles(),
  }));
  
  -- steps
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset",
    value = self:GetMinimum(),
    effects = {Wired.UI.Animation.OpacityEffect({
      components = self:GetBorder()
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Show",
    value = true,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = self:GetBorder()
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade In",
    value = self:GetMinimum(),
    minimum = self:GetMinimum(),
    maximum = self:GetMaximum(),
    duration = self:GetDuration() / 2,
    direction = 1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = self:GetBorder()
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade Out",
    value = self:GetMaximum(),
    minimum = self:GetMinimum(),
    maximum = self:GetMaximum(),
    duration = self:GetDuration() / 2,
    direction = -1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = self:GetBorder()
    })}
  }));
  
  -- finished
  self:GetFlashAnimation().Wired.Event.Listener:Add(Wired.Event.Listener(
    "AnimationFinished",
    function()
      self:SetBorderVisible(false);
    end,
    true
  ));
end

-- layout border
function Wired.UI.Animation.Flash:LayoutBorder()
  
  -- border
  local border = self:GetBorder();
  
  -- top
  border.top:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  border.top:SetWidth(UIParent:GetWidth());
  border.top:SetHeight(self:GetSize());
  
  -- bottom
  border.bottom:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = UIParent:GetHeight() - self:GetSize()});
  border.bottom:SetWidth(UIParent:GetWidth());
  border.bottom:SetHeight(self:GetSize());
  
  -- left
  border.left:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = self:GetSize()});
  border.left:SetWidth(self:GetSize());
  border.left:SetHeight(UIParent:GetHeight() - (self:GetSize() * 2));
  
  -- right
  border.right:SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = UIParent:GetWidth() - self:GetSize(), y = self:GetSize()});
  border.right:SetWidth(self:GetSize());
  border.right:SetHeight(UIParent:GetHeight() - (self:GetSize() * 2));
end

-- border color
function Wired.UI.Animation.Flash:SetBorderColor(value)
  for k, frame in pairs(self:GetBorder()) do
    frame:SetColor(value);
  end
end

-- border visible
function Wired.UI.Animation.Flash:SetBorderVisible(value)
  for k, frame in pairs(self:GetBorder()) do
    frame:SetVisible(value);
  end
end

-- start
function Wired.UI.Animation.Flash:Start(data)
  
  -- update values
  self:SetColor(data.color);
  self:SetDuration(data.cycleDuration);
  self:SetCycles(data.cycles);
  self:SetMinimum(data.minimum);
  self:SetMaximum(data.maximum);
  
  -- start animation
  self:GetFlashAnimation():Start();
end
