-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Animation.Sequence = class();

function Wired.UI.Animation.Sequence:Constructor(data)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetIdle(data.idle);
  self:SetTitle(data.title);
  self:SetCycles(data.cycles);
  self:SetDuration(data.duration);
  self:SetContinuous(data.continuous);
  
  -- idle
  self:GetIdle().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Idle",
    function()
      self:Idle();
    end,
    true
  ));
end

-- idle
function Wired.UI.Animation.Sequence:GetIdle()
  return self.idle;
end

function Wired.UI.Animation.Sequence:SetIdle(value)
  self.idle = value;
end

-- title
function Wired.UI.Animation.Sequence:GetTitle()
  return self.title;
end

function Wired.UI.Animation.Sequence:SetTitle(value)
  self.title = value;
end

-- active
function Wired.UI.Animation.Sequence:GetActive()
  return self.active;
end

function Wired.UI.Animation.Sequence:SetActive(value)
  self.active = value;
end

-- started
function Wired.UI.Animation.Sequence:GetStarted()
  return self.started;
end

function Wired.UI.Animation.Sequence:SetStarted(value)
  self.started = value;
end

-- cycles
function Wired.UI.Animation.Sequence:GetCycles()
  return self.cycles;
end

function Wired.UI.Animation.Sequence:SetCycles(value)
  self.cycles = value;
end

-- duration
function Wired.UI.Animation.Sequence:GetDuration()
  return self.duration;
end

function Wired.UI.Animation.Sequence:SetDuration(value)
  self.duration = value;
end

-- continuous
function Wired.UI.Animation.Sequence:GetContinuous()
  return self.continuous;
end

function Wired.UI.Animation.Sequence:SetContinuous(value)
  self.continuous = value;
end

-- update duration
function Wired.UI.Animation.Sequence:UpdateDuration()
  
  -- exists
  if (self:GetDuration()) then
    
    -- duration
    local duration = self:GetDuration()[self:GetCycle()];
    if (not duration) then
      duration = self:GetDuration()[1];
    end
    
    -- update durations
    for k, compositeStep in pairs(self:GetSteps()) do
      for l, step in pairs(compositeStep) do
      
        -- timed step
        if (step:GetType() == Wired.UI.Animation.Step.Constants.Types.Timed) then
          step:SetDuration(
            (step:GetDuration() / self:GetTotalDuration()) * duration
          );
        end
      end
    end
    
    -- update total duration
    self:SetTotalDuration(duration);
  end
end

-- total duration
function Wired.UI.Animation.Sequence:GetTotalDuration()
  if (not self.totalDuration) then
    self.totalDuration = 0;
  end
  return self.totalDuration;
end

function Wired.UI.Animation.Sequence:SetTotalDuration(value)
  self.totalDuration = value;
end

-- cycle
function Wired.UI.Animation.Sequence:GetCycle()
  return self.cycle;
end

function Wired.UI.Animation.Sequence:SetCycle(value)
  self.cycle = value;
end

-- step count
function Wired.UI.Animation.Sequence:GetStepCount()
  if (not self.stepCount) then
    self.stepCount = 0;
  end
  return self.stepCount;
end

function Wired.UI.Animation.Sequence:SetStepCount(value)
  self.stepCount = value;
end


-- steps
function Wired.UI.Animation.Sequence:GetSteps()
  if (type(self.steps) ~= "table") then
    self.steps = {};
  end
  return self.steps;
end

function Wired.UI.Animation.Sequence:SetSteps(value)
  self.steps = value;
end

function Wired.UI.Animation.Sequence:AddStep(value)
  
  -- composite
  if (not value:GetComposite()) then
    
    -- update step count
    self:SetStepCount(self:GetStepCount() + 1);
    
    -- update total duration
    if (value:GetType() == Wired.UI.Animation.Step.Constants.Types.Timed) then
      self:SetTotalDuration(self:GetTotalDuration() + value:GetDuration());
    end
    
    -- order table
    self:GetSteps()[self:GetStepCount()] = {};
  end
  
  -- index by title
  self:GetSteps()[self:GetStepCount()][value:GetTitle()] = value;
end

-- step index
function Wired.UI.Animation.Sequence:GetStepIndex()
  return self.stepIndex;
end

function Wired.UI.Animation.Sequence:SetStepIndex(value)
  self.stepIndex = value;
end

-- step
function Wired.UI.Animation.Sequence:GetStep()
  local result = nil;
  if (self:GetStepIndex()) then
    result = self:GetSteps()[self:GetStepIndex()];
  end
  return result;
end

-- next step
function Wired.UI.Animation.Sequence:NextStep()
  
  -- index
  self:SetStepIndex(self:GetStepIndex() + 1);
  
  -- finished
  if (self:GetStepIndex() > self:GetStepCount()) then
    
    -- continuous
    if (self:GetContinuous()) then
      
      -- step
      self:SetStepIndex(1);
      
    -- not continuous
    else
      
      -- update cycle
      self:SetCycle(self:GetCycle() + 1);
      
      -- reset
      if (self:GetCycle() <= self:GetCycles()) then
        
        -- step
        self:SetStepIndex(1);
        
        -- duration
        self:UpdateDuration();
        
      -- finished
      else
        
        -- fire finished event
        if (type(self.Wired.Event.AnimationFinished) == "function") then
          self.Wired.Event:AnimationFinished();
        end
      end
    end
  end
end

-- start
function Wired.UI.Animation.Sequence:Start()

  -- stop
  self:Stop();
  
  -- reset
  self:SetCycle(1);
  self:SetStepIndex(1);
  self:UpdateDuration();
  self:SetStepActive(false);
  
  -- active
  self:SetActive(true);
  
  -- started
  self:SetStarted(true);
end

-- stop
function Wired.UI.Animation.Sequence:Stop()
  
  -- active
  self:SetActive(false);
  
  -- deactivate current step
  if (self:GetStep()) then
    self:SetStepActive(false);
  end
  
  -- claer index
  self:SetStepIndex();
end

-- step finished
function Wired.UI.Animation.Sequence:GetStepFinished()
  local result = true;
  for k, step in pairs(self:GetStep()) do
    result = result and step:GetFinished();
  end
  return result;
end

-- step active
function Wired.UI.Animation.Sequence:GetStepActive()
  local result = false;
  for k, step in pairs(self:GetStep()) do
    result = result or step:GetActive();
  end
  return result;
end

function Wired.UI.Animation.Sequence:SetStepActive(value)
  for k, step in pairs(self:GetStep()) do
    step:SetActive(value);
  end
end

-- update step
function Wired.UI.Animation.Sequence:UpdateStep(value)
  for k, step in pairs(self:GetStep()) do
    step:Update(value);
  end
end

-- update
function Wired.UI.Animation.Sequence:Update(value)
  
  -- step
  if (self:GetStep()) then
    
    -- activate
    if (not self:GetStepActive()) then
      
      self:SetStepActive(true);
      
    -- finished
    elseif (self:GetStepFinished()) then
      
      -- deactivate
      self:SetStepActive(false);
      
      -- next step
      self:NextStep();
      
    -- update
    else
      self:UpdateStep(value);
    end
  end
end

-- idle
function Wired.UI.Animation.Sequence:Idle(value)
  
  -- active
  if (self:GetActive()) then
    
    -- update animation
    self:Update(self:GetIdle():GetElapsed());
  end
end
