-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Animation.Ticker = class(Wired.UI.Trion.Mask);

-- defaults
Wired.UI.Animation.Ticker.Defaults = {
  Color = {
    Team = {
      Founders = Wired.UI.Color.Pink,
      Contributors = Wired.UI.Color.Orange,
      Translators = Wired.UI.Color.Cyan,
      Benefactors = Wired.UI.Color.Green
    }
  },
  Animation = {
    Position = {
      Minimum = 0.00,
      Maximum = 1.00,
      Duration = 8
    }
  },
  Spacing = 36,
  Font = {
    Size = 18
  }
};

function Wired.UI.Animation.Ticker:Constructor(name, parent, application)
  Wired.UI.Trion.Mask.Constructor(self, name, parent);
  
  -- parameters
  self:SetApplication(application);
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- label
  self:SetLabel(Wired.UI.Trion.Frame(self:GetName() .. "_Label", self));
  self:GetLabel():SetColor(Wired.UI.Color.Clear);
  self:GetLabel():SetVisible(true);
  
  -- defaults
  self:ApplyTickerDefaults();
  
  -- events
  
  -- mouse in
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function()
      self:GetTickerAnimation():SetActive(false);
    end
  ));
  
  -- mouse out
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function()
      self:GetTickerAnimation():SetActive(true);
    end
  ));
end

-- application
function Wired.UI.Animation.Ticker:GetApplication()
  return self.application;
end

function Wired.UI.Animation.Ticker:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.UI.Animation.Ticker:GetIdle()
  return self.idle;
end

function Wired.UI.Animation.Ticker:SetIdle(value)
  self.idle = value;
end

-- label
function Wired.UI.Animation.Ticker:GetLabel()
  return self.label;
end

function Wired.UI.Animation.Ticker:SetLabel(value)
  self.label = value;
end

-- data
function Wired.UI.Animation.Ticker:GetData()
  if (type(self.data) ~= "table") then
    self.data = {};
  end
  return self.data;
end

function Wired.UI.Animation.Ticker:SetData(value)
  self.data = value;
  
  -- update data
  self:UpdateData();
end

-- ticker animation
function Wired.UI.Animation.Ticker:GetTickerAnimation()
  return self.tickerAnimation;
end

function Wired.UI.Animation.Ticker:SetTickerAnimation(value)
  self.tickerAnimation = value;
end

-- defaults
function Wired.UI.Animation.Ticker:ApplyTickerDefaults()
  
end

-- create ticker animation
function Wired.UI.Animation.Ticker:CreateTickerAnimation(members)
  
  -- sequence
  self:SetTickerAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Tick",
    cycles = 1,
    continuous = true
  }));
    
  -- steps
  self:GetTickerAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Tick",
    value = Wired.UI.Animation.Ticker.Defaults.Animation.Position.Minimum,
    minimum = Wired.UI.Animation.Ticker.Defaults.Animation.Position.Minimum,
    maximum = Wired.UI.Animation.Ticker.Defaults.Animation.Position.Maximum,
    duration = Wired.UI.Animation.Ticker.Defaults.Animation.Position.Duration * members,
    direction = 1,
    effects = {Wired.UI.Animation.PositionEffect({
      components = {self:GetLabel()},
      position = {
        x = (self:GetWidth() / 2) + (self:GetLabel():GetWidth() / 2),
        y = 0
      },
      x = -(self:GetLabel():GetWidth() + self:GetWidth()),
      y = 0
    })}
  }));
end

-- update data
function Wired.UI.Animation.Ticker:UpdateData()
  
  -- founders
  local members = {};
  local nonFounders = {};
  
  -- categories
  for k, category in pairs(self:GetData()) do
    
    -- members
    for l, member in pairs(category) do
      
      -- data
      local memberData = {
        name = member,
        category = k,
        color = Wired.UI.Animation.Ticker.Defaults.Color.Team[k]
      }
      
      -- index
      if (k == "Founders") then
        table.insert(members, memberData);
      else
        table.insert(nonFounders, memberData);
      end
    end
  end
  
  -- founders
  local founders = #members;
  
  -- seed
  math.randomseed(Inspect.Time.Real());
  
  -- randomize
  for k, v in ipairs(nonFounders) do
    
    -- index
    local index = math.random(#nonFounders);
    
    -- swap
    local swap = nonFounders[index];
    nonFounders[index] = nonFounders[k];
    nonFounders[k] = swap;
  end
  
  -- add non founders
  for k, v in ipairs(nonFounders) do
    table.insert(members, v);
  end
  
  -- offset
  local offset = 0;
  local height = 0;
  for k, member in ipairs(members) do
    
    local label = Wired.UI.Trion.Text(self:GetName() .. "_" .. member.name, self:GetLabel());
    label:SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = offset, y = 0});
    label:SetFontSize(Wired.UI.Animation.Ticker.Defaults.Font.Size);
    label:SetFontColor(member.color);
    label:SetText(member.name);
    label:SetVisible(true);
    
    -- height
    height = math.max(height, label:GetHeight());
    
    -- offset
    offset = offset + label:GetWidth() + Wired.UI.Animation.Ticker.Defaults.Spacing;
  end
  
  -- width
  self:GetLabel():SetWidth(offset - Wired.UI.Animation.Ticker.Defaults.Spacing);
  self:GetLabel():SetHeight(height);
  self:SetHeight(self:GetLabel():GetHeight());
  
  -- create animation
  self:CreateTickerAnimation(#members);
end

-- start
function Wired.UI.Animation.Ticker:Start(data)
  
  -- update values
  
  -- start animation
  self:GetTickerAnimation():Start();
end
