-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Button = class(Wired.UI.Trion.Texture);

-- defaults
Wired.UI.Button.Defaults = {
  Width = 14,
  Height = 14,
  Enabled = true,
  MouseDown = false,
  MouseOver = false
};

function Wired.UI.Button:Constructor(name, parent)
  Wired.UI.Trion.Texture.Constructor(self, name, parent);
  
  -- defaults
  self:ApplyButtonDefaults();
  
  -- events
  
  -- left down
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftDown",
    function(sender, value)
      self:SetMouseDown(true);
    end
  ));
  
  -- left up
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftUp",
    function(sender, value)
      self:SetMouseDown(false);
    end
  ));
  
  -- left up outside
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftUpoutside",
    function(sender, value)
      self:SetMouseDown(false);
    end
  ));
  
  -- mouse in
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:SetMouseOver(true);
    end
  ));
  
  -- mosue out
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:SetMouseOver(false);
    end
  ));
end

-- mouse down
function Wired.UI.Button:GetMouseDown()
  return self.mouseDown;
end

function Wired.UI.Button:SetMouseDown(value)
  self.mouseDown = value;

  -- update texture
  self:UpdateTexture();
end

-- mouse over
function Wired.UI.Button:GetMouseOver()
  return self.mouseOver;
end

function Wired.UI.Button:SetMouseOver(value)
  self.mouseOver = value;

  -- update texture
  self:UpdateTexture();
end

-- enabled
function Wired.UI.Button:GetEnabled()
  return self.enabled;
end

function Wired.UI.Button:SetEnabled(value)
  self.enabled = value;

  -- update texture
  self:UpdateTexture();
end

-- textures
function Wired.UI.Button:GetTextures()
  return self.textures;
end

function Wired.UI.Button:SetTextures(value)
  self.textures = value;
  
  -- update texture
  self:UpdateTexture();
end

-- defaults
function Wired.UI.Button:ApplyButtonDefaults()
  
  -- size
  self:SetWidth(Wired.UI.Button.Defaults.Width);
  self:SetHeight(Wired.UI.Button.Defaults.Height);
  
  -- mouse down
  self:SetMouseDown(Wired.UI.Button.Defaults.MouseDown);
  
  -- mouse over
  self:SetMouseOver(Wired.UI.Button.Defaults.MouseOver);
  
  -- enabled
  self:SetEnabled(Wired.UI.Button.Defaults.Enabled);
end

-- update texture
function Wired.UI.Button:UpdateTexture()
  
  -- texture
  local texture = nil;
  
  -- textures
  if (self:GetTextures()) then
    
    -- disabled
    if (not self:GetEnabled()) then
      texture = self:GetTextures().Disabled;
      
    -- mouse down
    elseif (self:GetMouseDown()) then
      texture = self:GetTextures().Down;
      
    -- mouse over
    elseif (self:GetMouseOver()) then
      texture = self:GetTextures().Over;
      
    -- enabled
    else
      texture = self:GetTextures().Enabled;
    end
  end
  
  -- apply
  if (texture) then
    self:SetTexture(texture);
  end
end
