-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.CascadeBar = class(Wired.UI.Trion.Texture);

-- constants
Wired.UI.CascadeBar.Constants = {
  Persistence = {
    Size = "Size",
    Position = "Position",
    Locked = "Locked"
  },
  Texture = Wired.UI.Texture("Wired", "UI/Resources/gauge.png"),
  Size = {
    Min = 10,
    Max = 86,
    Ratio = 12
  }
};

-- defaults
Wired.UI.CascadeBar.Defaults = {
  Alpha = 0.50,
  Color = {
    Locked = Wired.UI.Color.Red,
    Unlocked = Wired.UI.Color.Yellow
  },
  FontColor = Wired.UI.Color.Grey,
  Position = {
    Left = UIParent:GetWidth() / 2,
    Top = UIParent:GetHeight() / 4
  },
  Size = 18
};

function Wired.UI.CascadeBar:Constructor(name, parent, defaultPosition)
  Wired.UI.Trion.Texture.Constructor(self, name, parent);
  
  -- parameters
  self:SetDefaultPosition(defaultPosition);
  
  -- hide
  self:SetVisible(false);
  
  -- persistence
  self.Wired.Persistence:Enable(self:GetName());
  self.Wired.Persistence:SetParent(self:GetParent():GetBase());
  
  -- drag
  self:GetBase().Wired.UI.Drag:Enable(true);
  
  -- texture
  self:SetTexture(Wired.UI.CascadeBar.Constants.Texture);
  
  -- title label
  self:SetTitleLabel(Wired.UI.TextShadow(self:GetName() .. "_TitleLabel", self));
  self:GetTitleLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetTitleLabel():SetVisible(true);
  
  -- defaults
  self:ApplyCascadeBarDefaults();
    
  -- load
  self:Load();
  
  -- events
  
  -- move
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function()
      self:Moved();
    end
  ));
  
  -- stationary click
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "StationaryClick",
    function()
      self:Lock();
    end,
    true
  ));
  
  -- right click
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function()
      self:Lock();
    end
  ));
  
  -- zoom back
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(1);
    end
  ));
  
  -- zoom forward
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-1);
    end
  ));
end

-- title Label
function Wired.UI.CascadeBar:GetTitleLabel()
  return self.titleLabel;
end

function Wired.UI.CascadeBar:SetTitleLabel(value)
  self.titleLabel = value;
end

-- title
function Wired.UI.CascadeBar:GetTitle()
  return self:GetTitleLabel():GetText();
end

function Wired.UI.CascadeBar:SetTitle(value)
  self:GetTitleLabel():SetText(value);
end

-- size
function Wired.UI.CascadeBar:GetSize()
  
  -- presistent value
  local result = self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Size];
  
  -- default value
  if (not result) then
    result = Wired.UI.CascadeBar.Defaults.Size;
  end
  
  -- return
  return result;
end

function Wired.UI.CascadeBar:SetSize(value)
  
  -- restrict
  local restricted = math.max(
    Wired.UI.CascadeBar.Constants.Size.Min,
    math.min(Wired.UI.CascadeBar.Constants.Size.Max, value)
  );
  
  -- save
  self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Size] = restricted;
  
  -- layout
  self:Layout();
end

-- locked
function Wired.UI.CascadeBar:GetLocked()
  
  -- presistent value
  local result = self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Locked];
  
  -- default value
  if (result == nil) then
    result = Wired.UI.CascadeBar.Defaults.Locked;
  end
  
  -- return
  return result;
end

function Wired.UI.CascadeBar:SetLocked(value)
  self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Locked] = value;
  
  -- layout
  self:Layout();
end

-- ratio
function Wired.UI.CascadeBar:GetRatio()
  if (not self.ratio) then
    self.ratio = Wired.UI.CascadeBar.Constants.Size.Ratio;
  end
  return self.ratio;
end

function Wired.UI.CascadeBar:SetRatio(value)
  self.ratio = value;
  
  -- layout
  self:Layout();
end

-- default position
function Wired.UI.CascadeBar:GetDefaultPosition()
  if (not self.defaultPosition) then
    self.defaultPosition = Wired.UI.CascadeBar.Defaults.Position;
  end
  return self.defaultPosition;
end

function Wired.UI.CascadeBar:SetDefaultPosition(value)
  self.defaultPosition = value;
end

-- defaults
function Wired.UI.CascadeBar:ApplyCascadeBarDefaults()
  
  -- display
  self:SetAlpha(Wired.UI.CascadeBar.Defaults.Alpha);
  self:GetTitleLabel():SetFontColor(Wired.UI.CascadeBar.Defaults.FontColor);
  
  -- values
  self:SetTitle(self:GetName());
  
  -- layout
  self:Layout();
end

-- layout
function Wired.UI.CascadeBar:Layout()
  
  -- font size
  self:GetTitleLabel():SetFontSize(self:GetSize());
  
  -- height
  self:SetHeight(self:GetTitleLabel():GetHeight());
  
  -- scale width by ratio
  self:SetWidth(self:GetHeight() * self:GetRatio());
  
  -- locked
  if (self:GetLocked()) then
    self:SetColor(Wired.UI.CascadeBar.Defaults.Color.Locked);
  else
    self:SetColor(Wired.UI.CascadeBar.Defaults.Color.Unlocked);
  end
end

-- set position
function Wired.UI.CascadeBar:Moved()
  self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Position] = {
    Left = self:GetLeft(),
    Top = self:GetTop()
  };
end

-- load
function Wired.UI.CascadeBar:Load()
  
  -- position
  local position = self.Wired.Persistence:GetPath()[Wired.UI.CascadeBar.Constants.Persistence.Position];
  if (not position) then
    position = self:GetDefaultPosition();
  end
  
  -- changed
  if (
    (position.Left ~= self:GetBase():GetLeft()) or
    (position.Top ~= self:GetBase():GetTop())
  ) then
    self:SetPosition(position.Left, position.Top);
    --self:SetPoint({child = "CENTER", parent = "TOPLEFT"}, {x = position.Left, y = position.Top});
  end
end

-- lock
function Wired.UI.CascadeBar:Lock()
  self:SetLocked(not self:GetLocked());
end

-- wheel
function Wired.UI.CascadeBar:Wheel(value)
  self:SetSize(self:GetSize() + value);
end
