-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

-- class
Wired.UI.Cell = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Cell.Constants = {
  Texture = Wired.UI.Texture("Wired", "UI/Resources/gauge.png")
};

-- defaults
Wired.UI.Cell.Defaults = {
  Title = "Cell",
  Width = 160,
  Height = 20,
  Color = {
    On = Wired.UI.Color.White,
    Off = Wired.UI.Color.Black,
    Base = {
      Disabled = Wired.UI.Color.Grey
    },
    TitleLabel = {
      Disabled = Wired.UI.Color.Magenta
    }
  },
  Active = false,
  Enabled = true
};

-- constructor
function Wired.UI.Cell:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- bar
  self:SetBar(Wired.UI.Trion.Texture(self:GetName() .. "_Bar", self));
  self:GetBar():SetVisible(true);
  self:GetBar():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetBar():SetTexture(Wired.UI.Cell.Constants.Texture);
  self:GetBar():SetColor(Wired.UI.Color.Clear);
  self:GetBar():SetLayer(1);
  
  -- label
  self:SetLabel(Wired.UI.Trion.Text(self:GetName() .. "_Label", self));
  self:GetLabel():SetVisible(true);
  self:GetLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetLabel():SetColor(Wired.UI.Color.Clear);
  self:GetLabel():SetLayer(2);
  
  -- defaults
  self:ApplyCellDefaults();
  
  -- layout
  self:LayoutCell();
  
  -- events
  
  -- left click
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      if (type(self.Wired.Event.LeftClick) == "function") then
        self.Wired.Event:LeftClick();
      end
    end
  ));
  
  -- mouse in
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:SetMouseOver(true);
    end
  ));
  
  -- mouse out
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:SetMouseOver(false);
    end
  ));
  
  -- size
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      self:LayoutCell();
    end
  ));
end

-- bar
function Wired.UI.Cell:GetBar()
  return self.bar;
end

function Wired.UI.Cell:SetBar(value)
  self.bar = value;
end

-- label
function Wired.UI.Cell:GetLabel()
  return self.label;
end

function Wired.UI.Cell:SetLabel(value)
  self.label = value;
end

-- list
function Wired.UI.Cell:GetList()
  return self.list;
end

function Wired.UI.Cell:SetList(value)
  self.list = value;
end

-- object
function Wired.UI.Cell:GetObject()
  return self.object;
end

function Wired.UI.Cell:SetObject(value)
  self.object = value;
end

-- mouse over
function Wired.UI.Cell:GetMouseOver()
  return self.mouseOver;
end

function Wired.UI.Cell:SetMouseOver(value)
  self.mouseOver = value;

  -- update
  self:Update();
end

-- active
function Wired.UI.Cell:GetActive()
  return self.active;
end

function Wired.UI.Cell:SetActive(value)
  self.active = value;

  -- update
  self:Update();
end

-- enabled
function Wired.UI.Cell:GetEnabled()
  return self.enabled;
end

function Wired.UI.Cell:SetEnabled(value)
  self.enabled = value;

  -- update
  self:Update();
end

-- on color
function Wired.UI.Cell:GetOnColor()
  return self.onColor;
end

function Wired.UI.Cell:SetOnColor(value)
  self.onColor = value;
  
  -- update
  self:Update();
end

-- off color
function Wired.UI.Cell:GetOffColor()
  return self.offColor;
end

function Wired.UI.Cell:SetOffColor(value)
  self.offColor = value;
  
  -- update
  self:Update();
end

-- title
function Wired.UI.Cell:GetTitle()
  return self:GetLabel():GetText();
end

function Wired.UI.Cell:SetTitle(value)
  self:GetLabel():SetText(value);
end

-- defaults
function Wired.UI.Cell:ApplyCellDefaults()
  
  -- colors
  self:SetOnColor(Wired.UI.Cell.Defaults.Color.On);
  self:SetOffColor(Wired.UI.Cell.Defaults.Color.Off);
  
  -- values
  self:SetTitle(Wired.UI.Cell.Defaults.Title);
  self:SetActive(Wired.UI.Cell.Defaults.Active);
  self:SetEnabled(Wired.UI.Cell.Defaults.Enabled);
  
  -- size
  self:SetWidth(Wired.UI.Cell.Defaults.Width);
  self:SetHeight(Wired.UI.Cell.Defaults.Height);
end

-- layout
function Wired.UI.Cell:LayoutCell()
  
  -- bar
  self:GetBar():SetWidth(self:GetWidth());
  self:GetBar():SetHeight(self:GetHeight());
  
  -- label
  self:GetLabel():ScaleFontSizeVertically(self);
end

-- update
function Wired.UI.Cell:Update()
  
  -- off
  local color = self:GetOffColor();
  local fontColor = self:GetOnColor();
  
  -- on
  if (self:GetMouseOver() or self:GetActive()) then
    color = self:GetOnColor();
    fontColor = self:GetOffColor();
    
  -- disabled
  elseif (not self:GetEnabled()) then
    color = Wired.UI.Cell.Defaults.Color.Base.Disabled;
    fontColor = Wired.UI.Cell.Defaults.Color.TitleLabel.Disabled;
  end
  
  -- update colors
  self:SetColor(color);
  self:GetLabel():SetFontColor(fontColor);
end
