-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.ColorPicker.Window = class(Wired.UI.Window);

-- constants
Wired.UI.ColorPicker.Window.Constants = {
  Title = "Choose Color",
  Texture = {
    Color = Wired.UI.Texture("Wired", "UI/ColorPicker/Resources/color.png"),
    None = Wired.UI.Texture("Wired", "UI/ColorPicker/Resources/none.png")
  },
  None = Wired.UI.Color.Grey
};

-- defaults
Wired.UI.ColorPicker.Window.Defaults = {
  Spacing = 2
};

function Wired.UI.ColorPicker.Window:Constructor(name, parent)
  Wired.UI.Window.Constructor(self, name, parent);
  
  -- layer
  self:GetBase():SetLayer(self:GetBase():GetLayer() + 100);
  self:GetBar():SetLayer(self:GetBar():GetLayer() + 110);
  
  -- title
  self:SetTitle(Wired.UI.ColorPicker.Window.Constants.Title);
  
  -- composite
  self.composite = Wired.UI.Trion.Texture(self:GetName() .. "_Composite", self:GetBase());
  self.composite:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = Wired.UI.ColorPicker.Window.Defaults.Spacing + self:GetBorderSize().top});
  self.composite:SetVisible(true);
  self.composite:SetWidth(260);
  self.composite:SetHeight(260);
  
  -- red
  self.red = Wired.UI.ColorPicker.Slider(self:GetName() .. "_SliderRed", self.composite, Wired.UI.ColorPicker.Component.Red);
  self.red:SetPoint({child = "TOPCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = 30 + Wired.UI.ColorPicker.Window.Defaults.Spacing});
  self.red:SetVisible(true);
  
  -- green
  self.green = Wired.UI.ColorPicker.Slider(self:GetName() .. "_SliderGreen", self.red, Wired.UI.ColorPicker.Component.Green);
  self.green:SetPoint({child = "TOPCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = Wired.UI.ColorPicker.Window.Defaults.Spacing});
  self.green:SetVisible(true);
  
  -- blue
  self.blue = Wired.UI.ColorPicker.Slider(self:GetName() .. "_SliderBlue", self.green, Wired.UI.ColorPicker.Component.Blue);
  self.blue:SetPoint({child = "TOPCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = Wired.UI.ColorPicker.Window.Defaults.Spacing});
  self.blue:SetVisible(true);
  
  -- ok
  self.ok = Wired.UI.Trion.Button("Ok", self:GetBase());
  self.ok:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -Wired.UI.ColorPicker.Window.Defaults.Spacing - self:GetBorderSize().right, y = -Wired.UI.ColorPicker.Window.Defaults.Spacing - self:GetBorderSize().bottom});
  self.ok:SetTitle("Ok");
  self.ok:SetColor(Wired.UI.Color.Clear);
  self.ok:SetVisible(true);
  
  -- none
  self.none = Wired.UI.Trion.Button("None", self.ok);
  self.none:SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMLEFT"}, {x = 0, y = 0});
  self.none:SetTitle("None");
  self.none:SetColor(Wired.UI.Color.Clear);
  self.none:SetVisible(true);
  
  -- width
  self:GetBase():SetWidth(self.none:GetWidth() + self.ok:GetWidth() + (2 * Wired.UI.ColorPicker.Window.Defaults.Spacing) + self:GetBorderSize().left + self:GetBorderSize().right);
  
  -- clicked ok
  self.ok:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      self:ClickedOk();
    end
  ));
  
  -- clicked none
  self.none:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      self:ClickedNone();
    end
  ));
  
  -- value changed red
  self.red.Wired.Event.Listener:Add(Wired.Event.Listener(
    "ValueChanged",
    function(sender, value)
      self:SliderValueChanged();
    end,
    true
  ));
  
  -- value changed green
  self.green.Wired.Event.Listener:Add(Wired.Event.Listener(
    "ValueChanged",
    function(sender, value)
      self:SliderValueChanged();
    end,
    true
  ));
  
  -- value changed blue
  self.blue.Wired.Event.Listener:Add(Wired.Event.Listener(
    "ValueChanged",
    function(sender, value)
      self:SliderValueChanged();
    end,
    true
  ));
  
  -- update
  self:Update();
end

-- color picker
function Wired.UI.ColorPicker.Window:GetColorPicker()
  return self.colorPicker;
end

function Wired.UI.ColorPicker.Window:SetColorPicker(value)
  self.colorPicker = value;
end

-- value
function Wired.UI.ColorPicker.Window:GetValue()
  return self.value;
end

function Wired.UI.ColorPicker.Window:SetValue(value)
  self.value = value;
  
  -- update
  self:Update();
end

-- selected
function Wired.UI.ColorPicker.Window:GetSelected()
  return self.selected;
end

function Wired.UI.ColorPicker.Window:SetSelected(value)
  self.selected = value;
  
  -- update selected
  self:UpdateSelected();
end

-- slider value changed
function Wired.UI.ColorPicker.Window:SliderValueChanged()
  self:SetValue(Wired.UI.Color(
    self.red:GetValue() / 255,
    self.green:GetValue() / 255,
    self.blue:GetValue() / 255,
    1
  ));
end

-- clicked ok
function Wired.UI.ColorPicker.Window:ClickedOk()
  
  -- hide
  self:SetVisible(false);
  
  -- set color
  self:GetColorPicker():SetColor(self:GetValue());
end


-- clicked none
function Wired.UI.ColorPicker.Window:ClickedNone()
  
  -- hide
  self:SetVisible(false);
  
  -- set color
  self:GetColorPicker():SetColor();
end

-- update
function Wired.UI.ColorPicker.Window:Update()
  
  -- color
  local color = self:GetValue();
  if (not color) then
    color = Wired.UI.ColorPicker.Window.Constants.None;
  end
  
  -- texture
  if (self:GetValue()) then
    self.composite:SetTexture(Wired.UI.ColorPicker.Window.Constants.Texture.Color);
  else
    self.composite:SetTexture(Wired.UI.ColorPicker.Window.Constants.Texture.None);
  end
  
  -- color
  self.composite:SetColor(color);
end

-- update selected
function Wired.UI.ColorPicker.Window:UpdateSelected()
  
  -- color
  local color = self:GetSelected();
  if (not color) then
    color = Wired.UI.ColorPicker.Window.Constants.None;
  end
  
  -- selected
  self.red:SetSelected(color:GetRed() * 255);
  self.green:SetSelected(color:GetGreen() * 255);
  self.blue:SetSelected(color:GetBlue() * 255);
  
  -- value
  self:SetValue(self:GetSelected());
  
  -- update
  self:Update();
end

--[[
-- import
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Imez.Event";
import "Imez.UI";

Window = class(Imez.UI.PaddedWindow);

-- constants
Window.Text = "Color";
Window.Padding = 5;

-- defaults
Window.Defaults = {
  NoneEnabled = false
};

function Window:Constructor()
  Imez.UI.PaddedWindow.Constructor(self);

  -- self
  self:SetText(Window.Text);

  -- red
  self.red = Imez.UI._Color.Slider:Red();
  self.red:SetParent(self:GetInner());
  self.red:AddLabel("Red", Imez.UI.Label.Anchors.Left);
  self.red:AddPadding(Window.Padding);
  self.red:SetVisible(true);
  self.red:AddListener(
    Imez.Event.Listener("ValueChanged", function() self:UpdateColor(); end)
  );

  -- green
  self.green = Imez.UI._Color.Slider:Green();
  self.green:SetParent(self:GetInner());
  self.green:AddLabel("Green", Imez.UI.Label.Anchors.Left);
  self.green:AddPadding(Window.Padding);
  self.green:SetVisible(true);
  self.green:AddListener(
    Imez.Event.Listener("ValueChanged", function() self:UpdateColor(); end)
  );

  -- blue
  self.blue = Imez.UI._Color.Slider:Blue();
  self.blue:SetParent(self:GetInner());
  self.blue:AddLabel("Blue", Imez.UI.Label.Anchors.Left);
  self.blue:AddPadding(Window.Padding);
  self.blue:SetVisible(true);
  self.blue:AddListener(
    Imez.Event.Listener("ValueChanged", function() self:UpdateColor(); end)
  );

  -- composite
  self.composite = Turbine.UI.Control();
  self.composite:SetParent(self:GetInner());
  self.composite:AddPadding(Window.Padding);
  self.composite:AddBorder();
  self.composite:SetVisible(true);

  -- ok button
  self.ok = Turbine.UI.Lotro.Button();
  self.ok:SetParent(self:GetInner());
  self.ok:SetText("Ok");
  self.ok:AddPadding(Window.Padding);
  self.ok:AddTooltip("Select this color.");
  self.ok:SetEnabled(true);
  self.ok:SetVisible(true);
  self.ok:SetWantsKeyEvents(true);
  self.ok:AddListener(
    Imez.Event.Listener("KeyDown", function(sender, args)
      if (
        (args.Action == Imez.Utility.Keys.Enter) and
        (self.ok:IsEnabled()) and
        (self:IsVisible())
      ) then
        self:ClickedOk(args);
      end
    end)
  );
  self.ok:AddListener(
    Imez.Event.Listener("Click", function(sender, args)
      self:ClickedOk(args);
    end)
  );

  -- none button
  self.none = Turbine.UI.Lotro.Button();
  self.none:SetParent(self:GetInner());
  self.none:SetText("None");
  self.none:AddPadding(Window.Padding);
  self.none:AddTooltip("Select no color.");
  self.none:SetVisible(true);
  self.none:AddListener(
    Imez.Event.Listener("Click", function() self:ClickedNone(); end)
  );

  -- defaults
  self:ApplyColorWindowDefaults();
  self:UpdateColor();

  -- layout
  self:LayoutColorWindow();
end

-- getters and setters
function Window:GetColor()
  return self.composite:GetBackColor();
end

function Window:SetColor(value)
  self.composite:SetBackColor(value);
end

function Window:IsNoneEnabled()
  return self.none:IsEnabled();
end

function Window:SetNoneEnabled(value)
  self.none:SetEnabled(value);
end

-- defaults
function Window:ApplyColorWindowDefaults()
  self.none:SetEnabled(Window.Defaults.NoneEnabled);
end

-- layout
function Window:LayoutColorWindow()

  -- components
  local line = math.max(
    self.red:GetRelativeWidth(),
    self.green:GetRelativeWidth(),
    self.blue:GetRelativeWidth()
  );
  self.red:SetRelativePosition(
    line - self.red:GetRelativeWidth(),
    0
  );
  self.green:SetRelativePosition(
    line - self.green:GetRelativeWidth(),
    self.red:GetRelativeBottom()
  );
  self.blue:SetRelativePosition(
    line - self.blue:GetRelativeWidth(),
    self.green:GetRelativeBottom()
  );

  -- composite
  self.composite:SetRelativeSize(
    self.blue:GetRelativeBottom() - self.red:GetRelativeTop(),
    self.blue:GetRelativeBottom() - self.red:GetRelativeTop()
  );
  self.composite:SetRelativePosition(self.red:GetRelativeRight(), 0);

  -- ok button
  self.ok:SetRelativeSize(
    self.composite:GetRelativeWidth(),
    self.red:GetRelativeHeight()
  );
  self.ok:SetRelativePosition(
    self.composite:GetRelativeLeft(),
    self.composite:GetRelativeBottom()
  );

  -- none button
  self.none:SetSize(self.ok:GetWidth(), self.ok:GetHeight());
  self.none:SetRelativePosition(
    self.ok:GetRelativeLeft() - self.none:GetRelativeWidth(),
    self.ok:GetRelativeTop()
  );

  -- inner
  self:GetInner():SetSize(
    self.ok:GetRelativeRight(),
    self.ok:GetRelativeBottom()
  );
end

-- input
function Window:Input(value)
  if (value) then
    self.red:Input(value.R);
    self.green:Input(value.G);
    self.blue:Input(value.B);
  else
    self.red:Input(0);
    self.green:Input(0);
    self.blue:Input(0);
  end
end

-- clicked ok
function Window:ClickedOk(sender, args)
  self:SetVisible(false);

  -- fire selected event
  if (type(self.SelectedColor) == "function") then
    self:SelectedColor(self:GetColor());
  end
end

-- clicked none
function Window:ClickedNone(sender, args)
  self:SetVisible(false);

  -- fire selected event
  if (type(self.SelectedColor) == "function") then
    self:SelectedColor();
  end
end

-- update color
function Window:UpdateColor()
  self:SetColor(Turbine.UI.Color(
    self.red:GetValue(),
    self.green:GetValue(),
    self.blue:GetValue()
  ));
end
--]]