-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.DragNew = {};

-- functions

-- left down
local LeftDown = function(sender, args)

  -- localization
  local data = sender.Wired.UI.DragNew.data;
  if (data.enabled) then

    -- start drag
    data.dragging = true;
    data.dragged = false;

    -- save relative mouse coordinates
    local mouse = Inspect.Mouse();
    data.x = mouse.x - ((sender:GetLeft() + (sender:GetWidth() / 2)) - (sender:GetParent():GetLeft() + (sender:GetParent():GetWidth() / 2)));
    data.y = mouse.y - ((sender:GetTop() + (sender:GetHeight() / 2)) - (sender:GetParent():GetTop() + (sender:GetParent():GetHeight() / 2)));
  end
end

-- left up
local LeftUp = function(sender, args)

  -- localization
  local data = sender.Wired.UI.DragNew.data;
  if (data.enabled) then

    -- stop drag
    data.dragging = false;
  end
end

-- left up outside
local LeftUpOutside = function(sender, args)

  -- localization
  local data = sender.Wired.UI.DragNew.data;
  if (data.enabled) then

    -- stop drag
    data.dragging = false;
  end
end

-- mouse move
local MouseMove = function(sender, args)

  -- localization
  local data = sender.Wired.UI.DragNew.data;
  if (data.enabled) then
    if (data.dragging) then
      data.dragged = true;
      
      -- apply mouse coordinates
      local mouse = Inspect.Mouse();
      sender:SetPoint(
        "CENTER",
        sender:GetParent(),
        "CENTER",
        math.min(
          ((sender:GetParent():GetWidth() / 2) - (sender:GetWidth() / 2)),
          math.max(
            -((sender:GetParent():GetWidth() / 2) - (sender:GetWidth() / 2)),
            mouse.x - data.x
          )
        ),
        math.min(
          ((sender:GetParent():GetHeight() / 2) - (sender:GetHeight() / 2)),
          math.max(
            -((sender:GetParent():GetHeight() / 2) - (sender:GetHeight() / 2)),
            mouse.y - data.y
          )
        )
      );
    end
  end
end

-- override LeftClick to differentiate between drag and click
local LeftClick = function(sender, args)
  if ((not sender.Wired.UI.DragNew.data.enabled) or (not sender.Wired.UI.DragNew.data.dragged)) then
    if (type(sender.Wired.Event.StationaryClick) == "function") then
      sender.Wired.Event:StationaryClick(args);
    end
  end
end

-- listeners
Wired.UI.DragNew.Listeners = {
  Wired.Event.Listener("LeftDown", LeftDown),
  Wired.Event.Listener("LeftUp", LeftUp),
  Wired.Event.Listener("LeftUpoutside", LeftUpOutside),
  Wired.Event.Listener("MouseMove", MouseMove),
  Wired.Event.Listener("LeftClick", LeftClick)
};

-- enable
local Enable = function(sender, value)

  -- add data
  if (not sender.data) then
    sender.data = {};
  end

  -- reset data
  if (value) then
    sender.data.enabled = true;
    sender.data.dragging = false;
    sender.data.dragged = false;
    sender.data.x = 0;
    sender.data.y = 0;
  else
    sender.data.enabled = false;
  end

  -- reset listeners
  for k, listener in pairs(Wired.UI.DragNew.Listeners) do
    if (value) then
      sender:GetComponent().Wired.Event.Listener:Add(listener);
    else
      sender:GetComponent().Wired.Event.Listener:Remove(listener);
    end
  end
end

-- add drag context to component
Wired.UI.DragNew.Wire = function(context, component)

  -- add context
  if (type(context.DragNew) ~= "table") then
    context.DragNew = {};
    Wired.Wire(context.DragNew, component);

    -- add drag functions
    context.DragNew.Enable = Enable;
  end
end
