-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

-- class
Wired.UI.Gauge = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Gauge.Constant = {
  Texture = {
    Cover = Wired.UI.Texture("Wired", "UI/Resources/gauge.png")
  }
};

-- defaults
Wired.UI.Gauge.Default = {
  Animation = {
    Opacity = {
      Minimum = 0.00,
      Maximum = 1.00,
      Duration = 2.00
    }
  },
  Ratio = 12,
  Percentage = true,
  Inverted = false,
  Flipped = false,
  FontSize = 18,
  Scale = {
    Minimum = 0,
    Maximum = 1,
    Value = 0
  },
  
  Color = {
    Self = Wired.UI.Color.Black,
    Cover = Wired.UI.Color.Clear,
    Font = Wired.UI.Color.White
  },
  Title = "Gauge Title"
};

-- constructor
function Wired.UI.Gauge:Constructor(name, parent, application)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- parameters
  self:SetApplication(application);
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- bar
  self:SetBar(Wired.UI.Trion.Frame(self:GetName() .. "_Bar", self));
  self:GetBar():SetLayer(1);
  self:GetBar():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 0, y = 0});
  self:GetBar():SetVisible(true);
  
  -- cover
  self:SetCover(Wired.UI.Trion.Texture(self:GetName() .. "_Cover", self));
  self:GetCover():SetLayer(2);
  self:GetCover():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetCover():SetTexture(Wired.UI.Gauge.Constant.Texture.Cover);
  self:GetCover():SetVisible(true);
  
  -- label
  self:SetLabel(Wired.UI.TextShadow(self:GetName() .. "_Label", self));
  self:GetLabel():SetLayer(3);
  self:GetLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetLabel():SetVisible(true);
  
  -- value label
  self:SetValueLabel(Wired.UI.TextShadow(self:GetName() .. "_ValueLabel", self));
  self:GetValueLabel():SetLayer(3);
  self:GetValueLabel():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = 0, y = 0});
  self:GetValueLabel():SetVisible(true);
  
  -- mark label
  self:SetMarkLabel(Wired.UI.TextShadow(self:GetName() .. "_MarkLabel", self));
  self:GetMarkLabel():SetLayer(3);
  self:GetMarkLabel():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 0, y = 0});
  self:GetMarkLabel():SetVisible(true);
  
  -- apply defaults
  self:ApplyGaugeDefaults();
end

-- application
function Wired.UI.Gauge:GetApplication()
  return self.application;
end

function Wired.UI.Gauge:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.UI.Gauge:GetIdle()
  return self.idle;
end

function Wired.UI.Gauge:SetIdle(value)
  self.idle = value;
end

-- bar
function Wired.UI.Gauge:GetBar()
  return self.bar;
end

function Wired.UI.Gauge:SetBar(value)
  self.bar = value;
end

-- cover
function Wired.UI.Gauge:GetCover()
  return self.cover;
end

function Wired.UI.Gauge:SetCover(value)
  self.cover = value;
end

-- label
function Wired.UI.Gauge:GetLabel()
  return self.label;
end

function Wired.UI.Gauge:SetLabel(value)
  self.label = value;
end

-- value label
function Wired.UI.Gauge:GetValueLabel()
  return self.valueLabel;
end

function Wired.UI.Gauge:SetValueLabel(value)
  self.valueLabel = value;
end

-- mark label
function Wired.UI.Gauge:GetMarkLabel()
  return self.markLabel;
end

function Wired.UI.Gauge:SetMarkLabel(value)
  self.markLabel = value;
end

-- flash animation
function Wired.UI.Gauge:GetFlashAnimation()
  if (not self.flashAnimation) then
    self:CreateFlashAnimation();
  end
  return self.flashAnimation;
end

function Wired.UI.Gauge:SetFlashAnimation(value)
  self.flashAnimation = value;
end

-- title
function Wired.UI.Gauge:GetTitle()
  return self:GetLabel():GetText();
end

function Wired.UI.Gauge:SetTitle(value)
  self:GetLabel():SetText(value);
end

-- mark
function Wired.UI.Gauge:GetMark()
  return self:GetMarkLabel():GetText();
end

function Wired.UI.Gauge:SetMark(value)
  self:GetMarkLabel():SetText(value);
end

-- tint
function Wired.UI.Gauge:GetTint()
  return self.tint;
end

function Wired.UI.Gauge:SetTint(value)
  self.tint = value;
  
  -- apply tint
  if (self:GetTint()) then
    self:GetBar():SetColor(value);
  end
end

-- font color
function Wired.UI.Gauge:GetFontColor()
  return self:GetLabel():GetFontColor();
end

function Wired.UI.Gauge:SetFontColor(value)
  self:GetLabel():SetFontColor(value);
  self:GetValueLabel():SetFontColor(value);
  self:GetMarkLabel():SetFontColor(value);
end

-- font size
function Wired.UI.Gauge:GetFontSize()
  return self:GetLabel():GetFontSize();
end

function Wired.UI.Gauge:SetFontSize(value)
  self:GetLabel():SetFontSize(value);
  self:GetValueLabel():SetFontSize(value);
  self:GetMarkLabel():SetFontSize(value);
  
  -- layout
  self:Layout(true);
end

-- percentage
function Wired.UI.Gauge:GetPercentage()
  return self.percentage;
end

function Wired.UI.Gauge:SetPercentage(value)
  self.percentage = value;
end

-- inverted
function Wired.UI.Gauge:GetInverted()
  return self.inverted
end

function Wired.UI.Gauge:SetInverted(value)
  self.inverted = value;
end

-- flipped
function Wired.UI.Gauge:GetFlipped()
  return self.flipped
end

function Wired.UI.Gauge:SetFlipped(value)
  self.flipped = value;
end

-- scale
function Wired.UI.Gauge:GetScale()
  return self.scale;
end

function Wired.UI.Gauge:SetScale(value)
  self.scale = value;
end

-- value
function Wired.UI.Gauge:GetValue()
  return self:GetScale():GetValue();
end

function Wired.UI.Gauge:SetValue(value)
  
  -- value changed
  if (value ~= self:GetValue()) then
    self:GetScale():SetValue(value);
    
    -- draw
    self:Draw();
    
    -- fire value changed event
    if (type(self.Wired.Event.ValueChanged) == "function") then
      self.Wired.Event:ValueChanged();
    end
  end
end

-- ratio
function Wired.UI.Gauge:GetRatio()
  return self.ratio;
end

function Wired.UI.Gauge:SetRatio(value)
  self.ratio = value;
end

-- defaults
function Wired.UI.Gauge:ApplyGaugeDefaults()
  
  -- scale
  self:SetScale(Wired.Math.Scale({
    minimum = Wired.UI.Gauge.Default.Scale.Minimum,
    maximum = Wired.UI.Gauge.Default.Scale.Maximum
  }));
  self:GetScale():SetValue(Wired.UI.Gauge.Default.Scale.Value);
  
  -- color
  self:SetColor(Wired.UI.Gauge.Default.Color.Self);
  self:GetCover():SetColor(Wired.UI.Gauge.Default.Color.Cover);
  
  -- font color
  self:SetFontColor(Wired.UI.Gauge.Default.Color.Font);
  
  -- values
  self:SetTitle(Wired.UI.Gauge.Default.Title);
  self:SetPercentage(Wired.UI.Gauge.Default.Percentage);
  self:SetInverted(Wired.UI.Gauge.Default.Inverted);
  self:SetFlipped(Wired.UI.Gauge.Default.Flipped);
  self:SetRatio(Wired.UI.Gauge.Default.Ratio);
  
  -- font size
  self:SetFontSize(Wired.UI.Gauge.Default.FontSize);
end

-- layout
function Wired.UI.Gauge:Layout(dynamic)
  
  -- dynamic
  if (dynamic) then
    self:SetHeight(self:GetLabel():GetHeight());
    self:SetWidth(math.ceil(self:GetHeight() * self:GetRatio()));
  end
  
  -- cover
  self:GetCover():SetWidth(self:GetWidth());
  self:GetCover():SetHeight(self:GetHeight());
  
  -- bar
  self:GetBar():SetHeight(self:GetHeight());
  
  -- draw
  self:Draw();
end

-- scale tint
function Wired.UI.Gauge:GetScaledRed()
  local scale = self:GetScale():GetMedian();
  return 1 - ((math.max(scale, self:GetValue()) - scale) / scale);
end

function Wired.UI.Gauge:GetScaledGreen()
  local scale = self:GetScale():GetMedian();
  return math.max(0, math.min(scale, self:GetValue()) / scale);
end

-- scale tint inverted
function Wired.UI.Gauge:GetScaledRedInverted()
  local scale = self:GetScale():GetMedian();
  return 1 - ((math.max(scale, ((scale * 2) - self:GetValue())) - scale) / scale);
end

function Wired.UI.Gauge:GetScaledGreenInverted()
  local scale = self:GetScale():GetMedian();
  return math.max(0, math.min(scale, ((scale * 2) - self:GetValue())) / scale);
end

function Wired.UI.Gauge:ScaleTint()
  
  -- inverted
  if (self:GetInverted()) then
    self:GetBar():SetColor(Wired.UI.Color(
      self:GetScaledGreenInverted(),
      self:GetScaledRedInverted(),
      0,
      1
    ));
    
  -- flipped
  elseif (self:GetFlipped()) then
    self:GetBar():SetColor(Wired.UI.Color(
      self:GetScaledGreen(),
      self:GetScaledRed(),
      0,
      1
    ));
    
  -- regular
  else
    self:GetBar():SetColor(Wired.UI.Color(
      self:GetScaledRed(),
      self:GetScaledGreen(),
      0,
      1
    ));
  end
end

-- scale width
function Wired.UI.Gauge:GetScaledBar()
  
  -- scaled width
  local result = math.ceil(self:GetScale():GetRatio() * self:GetWidth());
  
  -- flipped
  if (self:GetFlipped()) then
    
    -- invert
    result = self:GetWidth() - result;
  end
  
  -- return
  return result;
end

function Wired.UI.Gauge:ScaleBar()
  
  -- value
  local value = self:GetScaledBar();
  
  -- changed
  if (value ~= self:GetBar():GetWidth()) then
    
    -- apply
    self:GetBar():SetWidth(value);
  end
end

-- scale value
function Wired.UI.Gauge:GetScaledValue()
  
  -- result
  local result = 0;
  
  -- percentage
  if (self:GetPercentage()) then
    result = tostring(math.ceil(100 * self:GetScale():GetRatio()));
    
  -- value
  else
    result = Wired.UI.Format.Time(math.ceil(self:GetValue()));
  end
  
  -- return
  return result;
end

function Wired.UI.Gauge:ScaleValue()
  
  -- value
  local value = self:GetScaledValue();
  
  -- changed
  if (value ~= self:GetValueLabel():GetText()) then
    
    -- apply
    self:GetValueLabel():SetText(value);
  end
end

-- draw
function Wired.UI.Gauge:Draw()
  
  -- scale tint
  if (not self:GetTint()) then
    self:ScaleTint();
  end
  
  -- scale width
  self:ScaleBar();
  
  -- scale value
  self:ScaleValue();
end

-- create flash animation
function Wired.UI.Gauge:CreateFlashAnimation()
  
  -- sequence
  self:SetFlashAnimation(Wired.UI.Animation.Sequence({
    idle = self:GetIdle(),
    title = "Flash",
    cycles = 1
  }));
  
  -- steps
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Reset",
    value = Wired.UI.Gauge.Default.Animation.Opacity.Minimum,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.UntimedStep({
    title = "Show",
    value = true,
    effects = {Wired.UI.Animation.VisibleEffect({
      components = {self}
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade In",
    value = Wired.UI.Gauge.Default.Animation.Opacity.Minimum,
    minimum = Wired.UI.Gauge.Default.Animation.Opacity.Minimum,
    maximum = Wired.UI.Gauge.Default.Animation.Opacity.Maximum,
    duration = Wired.UI.Gauge.Default.Animation.Opacity.Duration / 8,
    direction = 1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  self:GetFlashAnimation():AddStep(Wired.UI.Animation.TimedStep({
    title = "Fade Out",
    value = Wired.UI.Gauge.Default.Animation.Opacity.Maximum,
    minimum = Wired.UI.Gauge.Default.Animation.Opacity.Minimum,
    maximum = Wired.UI.Gauge.Default.Animation.Opacity.Maximum,
    duration = Wired.UI.Gauge.Default.Animation.Opacity.Duration / 2,
    direction = -1,
    effects = {Wired.UI.Animation.OpacityEffect({
      components = {self}
    })}
  }));
  
  -- finished
  self:GetFlashAnimation().Wired.Event.Listener:Add(Wired.Event.Listener(
    "AnimationFinished",
    function()
      self:SetVisible(false);
    end,
    true
  ));
end

-- start flash animation
function Wired.UI.Gauge:StartFlashAnimation()
  self:GetFlashAnimation():Start();
end

-- stop flash animation
function Wired.UI.Gauge:StopFlashAnimation()
  self:GetFlashAnimation():Stop();
end
