-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.List = class(Wired.UI.Trion.Frame);

Wired.UI.List.Constants = {
  Orientation = {
    Vertical = 1,
    Horizontal = 2
  }
};

Wired.UI.List.Defaults = {
  Orientation = Wired.UI.List.Constants.Orientation.Vertical,
  CellSpacing = 2,
  Padding = 2,
  Size = 5,
  CellSize = {
    width = Wired.UI.Cell.Defaults.Width,
    height = Wired.UI.Cell.Defaults.Height
  }
};

function Wired.UI.List:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- defaults
  self:ApplyListDefaults();
  
  
end

-- orientation
function Wired.UI.List:GetOrientation()
  return self.orientation;
end

function Wired.UI.List:SetOrientation(value)
  self.orientation = value;
  
  -- update size
  self:UpdateSize();
  
  -- update cells
  self:UpdateCells();
end

-- size
function Wired.UI.List:GetSize()
  if (not self.size) then
    self.size = 1;
  end
  return self.size;
end

function Wired.UI.List:SetSize(value)
  self.size = value;
  
  -- update size
  self:UpdateSize();
  
  -- fire size changed event
  if (type(self.Wired.Event.SizeChanged) == "function") then
    self.Wired.Event:SizeChanged();
  end
end

-- padding
function Wired.UI.List:GetPadding()
  if (not self.padding) then
    self.padding = 0;
  end
  return self.padding;
end

function Wired.UI.List:SetPadding(value)
  self.padding = value;
  
  -- update size
  self:UpdateSize();
  
  -- update cells
  self:UpdateCells();
end

-- cell spacing
function Wired.UI.List:GetCellSpacing()
  if (not self.cellSpacing) then
    self.cellSpacing = 0;
  end
  return self.cellSpacing;
end

function Wired.UI.List:SetCellSpacing(value)
  self.cellSpacing = value;
  
  -- update size
  self:UpdateSize();
  
  -- update cells
  self:UpdateCells();
end

-- selected
function Wired.UI.List:GetSelected()
  return self.selected;
end

function Wired.UI.List:SetSelected(value)
  self.selected = value;
  
  -- fire selected changed event
  if (type(self.Wired.Event.SelectedChanged) == "function") then
    self.Wired.Event:SelectedChanged();
  end
end

-- cells
function Wired.UI.List:GetCells()
  if (type(self.cells) ~= "table") then
    self.cells = {};
  end
  return self.cells;
end

function Wired.UI.List:SetCells(value)
  
  -- hide old cells
  self:SetCellsVisible(false);
  self:SetCellsActive(false);
  
  -- set cells
  self.cells = value;
  
  -- parenting
  for k, cell in pairs(self:GetCells()) do
    cell:SetList(self);
  end
  
  -- update cells
  self:UpdateCells();
  
  -- fire cells changed event
  if (type(self.Wired.Event.CellsChanged) == "function") then
    self.Wired.Event:CellsChanged();
  end
end

function Wired.UI.List:AddCell(value)
  
  -- parenting
  value:SetList(self);
  
  -- add
  table.insert(self:GetCells(), value);
  
  -- update cells
  self:UpdateCells();
end

-- cell size
function Wired.UI.List:GetCellSize()
  return self.cellSize;
end

function Wired.UI.List:SetCellSize(value)
  self.cellSize = value;
end

-- defaults
function Wired.UI.List:ApplyListDefaults()
  
  -- cell size
  self:SetCellSize(Wired.UI.List.Defaults.CellSize);
  
  -- orientation
  self:SetOrientation(Wired.UI.List.Defaults.Orientation);
  
  -- padding
  self:SetPadding(Wired.UI.List.Defaults.Padding);
  
  -- spacing
  self:SetCellSpacing(Wired.UI.List.Defaults.CellSpacing);
  
  -- size
  self:SetSize(Wired.UI.List.Defaults.Size);
end

-- set cells visible
function Wired.UI.List:SetCellsVisible(value)
  for k, cell in pairs(self:GetCells()) do
    cell:SetVisible(value);
  end
end

-- set cells active
function Wired.UI.List:SetCellsActive(value)
  for k, cell in pairs(self:GetCells()) do
    cell:SetActive(value);
  end
end

-- layout cells
function Wired.UI.List:LayoutCells();
  
  -- position
  local position = {
    left = self:GetPadding(),
    top = self:GetPadding()
  };
  
  for k, cell in pairs(self:GetCells()) do
    cell:SetPosition(position.left, position.top);
    
    -- vertical
    if (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Vertical) then
      
      -- size
      cell:SetWidth(self:GetWidth() - (self:GetPadding() * 2));
      
      -- position
      position.top = position.top + cell:GetHeight() + self:GetCellSpacing()
      
    -- horizontal
    elseif (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Horizontal) then
      
      -- size
      cell:SetHeight(self:GetHeight() - (self:GetPadding() * 2));
      
      -- position
      position.left = position.left + cell:GetWidth() + self:GetCellSpacing()
    end
  end
end

-- scale cells
function Wired.UI.List:ScaleCells();
  
  -- cells
  local cells = table.getn(self:GetCells());
  
  -- dimension
  local dimension = self:GetWidth();
  if (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Vertical) then
    dimension = self:GetHeight();
  end
  
  -- size
  local size = 0;
  if (cells > 0) then
    size = (dimension - (self:GetPadding() * 2) - (self:GetCellSpacing() * (cells - 1))) / cells;
  end
  
  -- dimension
  for k, cell in pairs(self:GetCells()) do
    if (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Horizontal) then
      cell:SetWidth(size);
    else
      cell:SetHeight(size);
    end
  end
  
  -- layout cells
  self:LayoutCells();
end

-- update size
function Wired.UI.List:UpdateSize()
  
  -- size
  local size = {
    width = self:GetCellSize().width,
    height = self:GetCellSize().height,
  };
  
  -- vertical
  if (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Vertical) then
    size.height = (
      (size.height * self:GetSize()) +
      (self:GetCellSpacing() * (self:GetSize() - 1))
    );
    
  -- horizontal
  elseif (self:GetOrientation() == Wired.UI.List.Constants.Orientation.Horizontal) then
    size.width = (
      (size.width * self:GetSize()) +
      (self:GetCellSpacing() * (self:GetSize() - 1))
    );
  end
  
  -- size
  self:SetWidth(size.width + (self:GetPadding() * 2));
  self:SetHeight(size.height + (self:GetPadding() * 2));
end

-- update cells
function Wired.UI.List:UpdateCells();
  
  -- layout cells
  self:LayoutCells();
  
  -- show new cells
  self:SetCellsVisible(true);
end

-- select
function Wired.UI.List:Select(value);
  
  -- unselect old
  if (self:GetSelected()) then
    self:GetSelected():SetActive(false);
  end
  
  -- select new
  if (value) then
    self:SetSelected(value);
    value:SetActive(true);
  end
end

-- create cell
function Wired.UI.List:CreateCell(value)
  return Wired.UI.Cell(self:GetName() .. "_Cell_" .. value, self);
end

-- populate
function Wired.UI.List:Populate(action)
  
  -- cells
  local cells = {};
  
  -- create cells
  for i = 1, self:GetSize() do
    cells[i] = Wired.UI.Cell(self:GetName() .. "_Cell_" .. i, self);
    
    -- left click
    cells[i].Wired.Event.Listener:Add(Wired.Event.Listener(
      "LeftClick",
      function()
        if (type(action) == "function") then
          action(cells[i], i);
        end
      end,
      true
    ));
  end
  
  -- add
  self:SetCells(cells);
  self:SetCellsVisible(false);
end
