-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Scroll.Bar = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Scroll.Bar.Constants = {
  Orientation = {
    Horizontal = 1,
    Vertical = 2
  },
  Scale = {
    Minimum = -1,
    Maximum = 1
  },
  Texture = {
    Horizontal = {
      Decrease = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Decrease/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Decrease/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Decrease/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Decrease/Disabled.png")
      },
      Increase = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Increase/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Increase/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Increase/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Increase/Disabled.png")
      }
    },
    Vertical = {
      Decrease = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Decrease/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Decrease/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Decrease/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Decrease/Disabled.png")
      },
      Increase = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Increase/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Increase/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Increase/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Increase/Disabled.png")
      }
    }
  }
};

-- defaults
Wired.UI.Scroll.Bar.Defaults = {
  Size = Wired.UI.Button.Defaults.Width,
  Scale = {
    Value = -1
  },
  Rate = 0.1,
  Orientation = Wired.UI.Scroll.Bar.Constants.Orientation.Vertical,
  Color = {
    DragLane = Wired.UI.Color.Black;
  }
};

function Wired.UI.Scroll.Bar:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- components
  
  -- scale
  self:SetScale(Wired.Math.Scale({
    minimum = Wired.UI.Scroll.Bar.Constants.Scale.Minimum,
    maximum = Wired.UI.Scroll.Bar.Constants.Scale.Maximum
  }));
  
  -- decrease button
  self:SetDecreaseButton(Wired.UI.Button(self:GetName() .. "_DecreaseButton", self));
  self:GetDecreaseButton():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  self:GetDecreaseButton():SetVisible(true);
  
  -- increase button
  self:SetIncreaseButton(Wired.UI.Button(self:GetName() .. "_IncreaseButton", self));
  self:GetIncreaseButton():SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = 0, y = 0});
  self:GetIncreaseButton():SetVisible(true);
  
  -- drag lane
  self:SetDragLane(Wired.UI.Trion.Frame(self:GetName() .. "_DragLane", self));
  self:GetDragLane():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetDragLane():SetVisible(true);
  
  -- drag button
  self:SetDragButton(Wired.UI.Scroll.Slider(self:GetName() .. "_DragButton", self:GetDragLane()));
  self:GetDragButton():SetVisible(true);
  
  -- drag
  self:GetDragButton():GetBase().Wired.UI.DragNew:Enable(true);
  
  -- defaults
  self:ApplyScrollBarDefaults()
  
  -- events
  
  -- lane
  self:GetDragLane():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:DragLaneClicked();
    end
  ));
  
  -- decrease
  self:GetDecreaseButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:SetValue(self:GetValue() - Wired.UI.Scroll.Bar.Defaults.Rate);
    end
  ));
  
  -- increase
  self:GetIncreaseButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:SetValue(self:GetValue() + Wired.UI.Scroll.Bar.Defaults.Rate);
    end
  ));
  
  -- zoom back
  self:GetDragLane():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:SetValue(self:GetValue() + Wired.UI.Scroll.Bar.Defaults.Rate);
    end
  ));
  
  -- zoom forward
  self:GetDragLane():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:SetValue(self:GetValue() - Wired.UI.Scroll.Bar.Defaults.Rate);
    end
  ));
  
  -- dragged
  self:GetDragButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function()
      self:DragButtonMoved();
    end
  ));
  
  -- layout
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function()
      self:LayoutScrollBar();
    end
  ));
end

-- orientation
function Wired.UI.Scroll.Bar:GetOrientation()
  return self.orientation;
end

function Wired.UI.Scroll.Bar:SetOrientation(value)
  self.orientation = value;
  
  -- update drag button
  self:GetDragButton():SetOrientation(self:GetOrientation());
  
  -- layout
  self:LayoutScrollBar();
  
  -- update textures
  self:UpdateTextures();
end

-- scale
function Wired.UI.Scroll.Bar:GetScale()
  return self.scale;
end

function Wired.UI.Scroll.Bar:SetScale(value)
  self.scale = value;
end

-- value
function Wired.UI.Scroll.Bar:GetValue()
  return self:GetScale():GetValue();
end

function Wired.UI.Scroll.Bar:SetValue(value, input)
  
  -- last value
  local lastValue = self:GetValue();
  
  -- set value
  self:GetScale():SetValue(value);
  
  -- changed
  if (lastValue ~= self:GetValue()) then
    
    -- update
    if (not input) then
      self:Update();
    end
    
    -- update buttons
    self:UpdateButtons();
    
    -- fire value changed event
    if (type(self.Wired.Event.ValueChanged) == "function") then
      self.Wired.Event:ValueChanged();
    end
  end
end

-- decrease button
function Wired.UI.Scroll.Bar:GetDecreaseButton()
  return self.decreaseButton;
end

function Wired.UI.Scroll.Bar:SetDecreaseButton(value)
  self.decreaseButton = value;
end

-- increase button
function Wired.UI.Scroll.Bar:GetIncreaseButton()
  return self.increaseButton;
end

function Wired.UI.Scroll.Bar:SetIncreaseButton(value)
  self.increaseButton = value;
end

-- drag lane
function Wired.UI.Scroll.Bar:GetDragLane()
  return self.dragLane;
end

function Wired.UI.Scroll.Bar:SetDragLane(value)
  self.dragLane = value;
end

-- drag button
function Wired.UI.Scroll.Bar:GetDragButton()
  return self.dragButton;
end

function Wired.UI.Scroll.Bar:SetDragButton(value)
  self.dragButton = value;
end

-- defaults
function Wired.UI.Scroll.Bar:ApplyScrollBarDefaults()
  
  -- color
  self:GetDragLane():SetColor(Wired.UI.Scroll.Bar.Defaults.Color.DragLane);
  
  -- value
  self:SetValue(Wired.UI.Scroll.Bar.Defaults.Scale.Value);
  
  -- orientation
  self:SetOrientation(Wired.UI.Scroll.Bar.Defaults.Orientation);
end

-- layout
function Wired.UI.Scroll.Bar:LayoutScrollBar()
  
  -- size
  local width = self:GetDecreaseButton():GetWidth();
  local height = self:GetDecreaseButton():GetHeight();
  
  -- orientation
  if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
    width = self:GetWidth() - self:GetDecreaseButton():GetWidth() - self:GetIncreaseButton():GetWidth();
  elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
    height = self:GetHeight() - self:GetDecreaseButton():GetHeight() - self:GetIncreaseButton():GetHeight();
  end
  
  -- apply
  self:GetDragLane():SetWidth(width);
  self:GetDragLane():SetHeight(height);
  
  -- button
  if (self:GetParent():GetComponent()) then
    if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
      self:GetDragButton():SetWidth(math.max((self:GetDecreaseButton():GetWidth() + self:GetIncreaseButton():GetWidth()),
        self:GetDragLane():GetWidth() *
        (self:GetParent():GetMask():GetWidth() / self:GetParent():GetComponent():GetWidth())
      ));
      self:GetDragButton():SetHeight(self:GetDecreaseButton():GetHeight());
    elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
      self:GetDragButton():SetWidth(self:GetDecreaseButton():GetWidth());
      self:GetDragButton():SetHeight(math.max((self:GetDecreaseButton():GetHeight() + self:GetIncreaseButton():GetHeight()),
        self:GetDragLane():GetHeight() *
        (self:GetParent():GetMask():GetHeight() / self:GetParent():GetComponent():GetHeight())
      ));
    end
  end
  
  -- update
  self:Update();
end

-- update
function Wired.UI.Scroll.Bar:Update()
  
  -- offset
  local offset = {
    x = 0,
    y = 0
  };
  
  -- orientation
  if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
    offset.x = (
      self:GetValue() *
      ((self:GetDragLane():GetWidth() - self:GetDragButton():GetWidth()) / 2)
    );
  elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
    offset.y = (
      self:GetValue() *
      ((self:GetDragLane():GetHeight() - self:GetDragButton():GetHeight()) / 2)
    );
  end
  
  -- position
  self:GetDragButton():SetPoint({child = "CENTER", parent = "CENTER"}, offset);
end

-- drag button moved
function Wired.UI.Scroll.Bar:DragButtonMoved()
  
  -- offset
  local button = 0;
  local lane = 0;
  local size = 1;
  
  -- orientation
  if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
    button = self:GetDragButton():GetLeft() + (self:GetDragButton():GetWidth() / 2);
    lane = self:GetDragLane():GetLeft() + (self:GetDragLane():GetWidth() / 2);
    size = (self:GetDragLane():GetWidth() - self:GetDragButton():GetWidth()) / 2;
  elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
    button = self:GetDragButton():GetTop() + (self:GetDragButton():GetHeight() / 2);
    lane = self:GetDragLane():GetTop() + (self:GetDragLane():GetHeight() / 2);
    size = (self:GetDragLane():GetHeight() - self:GetDragButton():GetHeight()) / 2;
  end
  
  -- value
  self:SetValue((button - lane) / size, true);
end

-- drag lane clicked
function Wired.UI.Scroll.Bar:DragLaneClicked()
  
  -- direction
  local direction = 1;
  
  -- mouse
  local mouse = Inspect.Mouse();
  
  -- orientation
  if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
    if (mouse.x < self:GetDragButton():GetLeft()) then
      direction = -1;
    end
  elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
    if (mouse.y < self:GetDragButton():GetTop()) then
      direction = -1;
    end
  end
  
  -- value
  self:SetValue(self:GetValue() + (direction * Wired.UI.Scroll.Bar.Defaults.Rate * 3));
end

-- update buttons
function Wired.UI.Scroll.Bar:UpdateButtons()
  
  -- at minimum
  if (self:GetValue() == Wired.UI.Scroll.Bar.Constants.Scale.Minimum) then
    self:GetDecreaseButton():SetEnabled(false);
    self:GetIncreaseButton():SetEnabled(true);
    
  -- at maximum
  elseif (self:GetValue() == Wired.UI.Scroll.Bar.Constants.Scale.Maximum) then
    self:GetDecreaseButton():SetEnabled(true);
    self:GetIncreaseButton():SetEnabled(false);
    
  -- somewhere else
  else
    self:GetDecreaseButton():SetEnabled(true);
    self:GetIncreaseButton():SetEnabled(true);
  end
end

-- update textures
function Wired.UI.Scroll.Bar:UpdateTextures()
  
  -- textures
  local textures = nil;
  
  -- horizontal
  if (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal) then
    textures = Wired.UI.Scroll.Bar.Constants.Texture.Horizontal;
    
  -- vertical
  elseif (self:GetOrientation() == Wired.UI.Scroll.Bar.Constants.Orientation.Vertical) then
    textures = Wired.UI.Scroll.Bar.Constants.Texture.Vertical;
  end
  
  -- apply
  self:GetDecreaseButton():SetTextures(textures.Decrease);
  self:GetIncreaseButton():SetTextures(textures.Increase);
end
