-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Scroll.DropList = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Scroll.DropList.Constants = {
  Message = {
    Select = "Select Value"
  }
};

-- defaults
Wired.UI.Scroll.DropList.Defaults = {
  Padding = 2
};

function Wired.UI.Scroll.DropList:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- components
  
  -- button
  self:SetButton(Wired.UI.Button(self:GetName() .. "_Button", self));
  self:GetButton():SetTextures(Wired.UI.Scroll.Bar.Constants.Texture.Vertical.Increase);
  self:GetButton():SetVisible(true);
  
  -- cell
  self:SetCell(Wired.UI.Cell(self:GetName() .. "_Cell", self));
  self:GetCell():SetTitle(Wired.UI.Scroll.DropList.Constants.Message.Select);
  self:GetCell():SetVisible(true);
  
  -- list
  self:SetList(Wired.UI.Scroll.List(self:GetName() .. "_List", self));
  self:GetList():SetVisible(false);
  
  -- defaults
  self:ApplyDropListDefaults();
  
  -- events
  
  -- left click button
  self:GetButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:SetDeployed(not self:GetDeployed());
    end
  ));
  
  -- left click cell
  self:GetCell().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:SetDeployed(not self:GetDeployed());
    end,
    true
  ));
  
  -- left up
  self:GetList():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftUpoutside",
    function(sender, value)
      self:SetDeployed(false);
    end
  ));
  
  -- size
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      self:LayoutDropList();
    end
  ));
  
  -- selected changed
  self:GetList():GetList().Wired.Event.Listener:Add(Wired.Event.Listener(
    "SelectedChanged",
    function(sender, value)
      self:UpdateSelected();
      
      -- fire selected changed event
      if (type(self.Wired.Event.SelectedChanged) == "function") then
        self.Wired.Event:SelectedChanged();
      end
    end,
    true
  ));
  
  -- cells changed
  self:GetList():GetList().Wired.Event.Listener:Add(Wired.Event.Listener(
    "CellsChanged",
    function(sender, value)
      
      -- layout
      self:LayoutDropList();
      
      -- scroll
      self:GetList():Scroll();
    end,
    true
  ));
end

-- button
function Wired.UI.Scroll.DropList:GetButton()
  return self.button;
end

function Wired.UI.Scroll.DropList:SetButton(value)
  self.button = value;
end

-- cell
function Wired.UI.Scroll.DropList:GetCell()
  return self.cell;
end

function Wired.UI.Scroll.DropList:SetCell(value)
  self.cell = value;
end

-- list
function Wired.UI.Scroll.DropList:GetList()
  return self.list;
end

function Wired.UI.Scroll.DropList:SetList(value)
  self.list = value;
end

-- padding
function Wired.UI.Scroll.DropList:GetPadding()
  if (not self.padding) then
    self.padding = Wired.UI.Scroll.DropList.Defaults.Padding;
  end
  return self.padding;
end

function Wired.UI.Scroll.DropList:SetPadding(value)
  self.padding = value;
  
  -- layout
  self:LayoutDropList();
end

-- selected
function Wired.UI.Scroll.DropList:GetSelected()
  return self:GetList():GetList():GetSelected();
end

function Wired.UI.Scroll.DropList:SetSelected(value)
  self:GetList():GetList():SetSelected(value);
end

-- cells
function Wired.UI.Scroll.DropList:GetCells()
  return self:GetList():GetList():GetCells();
end

function Wired.UI.Scroll.DropList:SetCells(value)
  self:GetList():GetList():SetCells(value);
end

-- size
function Wired.UI.Scroll.DropList:GetSize()
  return self:GetList():GetList():GetSize();
end

function Wired.UI.Scroll.DropList:SetSize(value)
  self:GetList():GetList():SetSize(value);
end

-- display size
function Wired.UI.Scroll.DropList:GetDisplaySize()
  return self:GetList():GetDisplaySize();
end

function Wired.UI.Scroll.DropList:SetDisplaySize(value)
  self:GetList():SetDisplaySize(value);
end

-- deployed
function Wired.UI.Scroll.DropList:GetDeployed()
  return self.deployed;
end

function Wired.UI.Scroll.DropList:SetDeployed(value)
  self.deployed = value;
  
  -- update deployed
  self:UpdateDeployed();
end

-- defaults
function Wired.UI.Scroll.DropList:ApplyDropListDefaults()
  
end

-- layout
function Wired.UI.Scroll.DropList:LayoutDropList()
  
  -- anchors
  self:GetButton():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = -self:GetPadding(), y = 0});
  self:GetCell():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = self:GetPadding(), y = 0});
  self:GetList():SetPoint({child = "TOPCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = -self:GetPadding()});
  
  -- cell
  self:GetCell():SetWidth(self:GetWidth() - self:GetButton():GetWidth() - (self:GetPadding() * 3));
  self:GetCell():SetHeight(self:GetHeight() - (self:GetPadding() * 2));
  
  -- list
  self:GetList():SetWidth(self:GetWidth());
end

-- populate
function Wired.UI.Scroll.DropList:Populate(value)
  self:GetList():GetList():Populate(value);
end

-- select
function Wired.UI.Scroll.DropList:Select(value)
  self:GetList():GetList():Select(value);
  
  -- deployed
  self:SetDeployed(false);
end

-- layout
function Wired.UI.Scroll.DropList:UpdateDeployed()
  self:GetList():SetVisible(self:GetDeployed());
  self:GetButton():SetEnabled(not self:GetDeployed());
end

-- update selected
function Wired.UI.Scroll.DropList:UpdateSelected()
  
  -- undeploy
  self:SetDeployed(false);
  
  -- selected
  local selected = self:GetSelected();
  if (selected) then
    self:GetCell():SetTitle(selected:GetTitle());
  else
    self:GetCell():SetTitle(Wired.UI.Scroll.DropList.Constants.Message.Select);
  end
end
