-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Scroll.Frame = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Scroll.Frame.Constants = {
  
};

-- defaults
Wired.UI.Scroll.Frame.Defaults = {
  Padding = 2,
  Rate = 0.1
};

function Wired.UI.Scroll.Frame:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- components
  
  -- horizontal bar
  self:SetHorizontalBar(Wired.UI.Scroll.Bar(self:GetName() .. "_HorizontalBar", self));
  self:GetHorizontalBar():SetPoint({child = "BOTTOMLEFT", parent = "BOTTOMLEFT"}, {x = Wired.UI.Scroll.Frame.Defaults.Padding, y = -Wired.UI.Scroll.Frame.Defaults.Padding});
  self:GetHorizontalBar():SetHeight(Wired.UI.Scroll.Bar.Defaults.Size);
  self:GetHorizontalBar():SetOrientation(Wired.UI.Scroll.Bar.Constants.Orientation.Horizontal);
  self:GetHorizontalBar():SetVisible(true);
  
  -- vertical bar
  self:SetVerticalBar(Wired.UI.Scroll.Bar(self:GetName() .. "_VerticalBar", self));
  self:GetVerticalBar():SetPoint({child = "TOPRIGHT", parent = "TOPRIGHT"}, {x = -Wired.UI.Scroll.Frame.Defaults.Padding, y = Wired.UI.Scroll.Frame.Defaults.Padding});
  self:GetVerticalBar():SetWidth(Wired.UI.Scroll.Bar.Defaults.Size);
  self:GetVerticalBar():SetOrientation(Wired.UI.Scroll.Bar.Constants.Orientation.Vertical);
  self:GetVerticalBar():SetVisible(true);
  
  -- mask
  self:SetMask(Wired.UI.Scroll.Mask(self:GetName() .. "_Mask", self));
  self:GetMask():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = Wired.UI.Scroll.Frame.Defaults.Padding, y = Wired.UI.Scroll.Frame.Defaults.Padding});
  self:GetMask():SetVisible(true);
  
  -- defaults
  self:ApplyScrollBarDefaults()
  
  -- events
  
  -- horizontal bar value changed
  self:GetHorizontalBar().Wired.Event.Listener:Add(Wired.Event.Listener(
    "ValueChanged",
    function(sender, value)
      self:Scroll();
    end,
    true
  ));
  
  -- vertical bar value changed
  self:GetVerticalBar().Wired.Event.Listener:Add(Wired.Event.Listener(
    "ValueChanged",
    function(sender, value)
      self:Scroll();
    end,
    true
  ));
  
  -- zoom back
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(1);
    end
  ));
  
  -- zoom forward
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-1);
    end
  ));
  
  -- size
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      self:LayoutScrollFrame();
    end
  ));
  
  -- test
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      self:SetVisible(false);
    end
  ));
end

-- horizontal bar
function Wired.UI.Scroll.Frame:GetHorizontalBar()
  return self.horizontalBar;
end

function Wired.UI.Scroll.Frame:SetHorizontalBar(value)
  self.horizontalBar = value;
end

-- vertical bar
function Wired.UI.Scroll.Frame:GetVerticalBar()
  return self.verticalBar;
end

function Wired.UI.Scroll.Frame:SetVerticalBar(value)
  self.verticalBar = value;
end

-- mask
function Wired.UI.Scroll.Frame:GetMask()
  return self.mask;
end

function Wired.UI.Scroll.Frame:SetMask(value)
  self.mask = value;
end

-- component scale
function Wired.UI.Scroll.Frame:GetComponentScale()
  if (type(self.componentScale) ~= "table") then
    self.componentScale = {
      horizontal = false,
      vertical = false
    };
  end
  return self.componentScale;
end

function Wired.UI.Scroll.Frame:SetComponentScale(value)
  self.componentScale = value;
  
  -- layout
  self:LayoutScrollFrame();
end

-- component
function Wired.UI.Scroll.Frame:GetComponent()
  return self.component;
end

function Wired.UI.Scroll.Frame:SetComponent(value)
  self.component = value;
  
  -- size
  self.component:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      
      -- layout
      self:LayoutScrollFrame();
    end
  ));
  
  -- layout
  self:LayoutScrollFrame();
end

-- orientation
function Wired.UI.Scroll.Frame:GetOrientation()
  if (not self.orientation) then
    self.orientation = Wired.UI.Scroll.Frame.Defaults.Orientation;
  end
  return self.orientation;
end

function Wired.UI.Scroll.Frame:SetOrientation(value)
  self.orientation = value;
  
  -- layout
  self:LayoutScrollFrame();
end

-- padding
function Wired.UI.Scroll.Frame:GetPadding()
  if (not self.padding) then
    self.padding = Wired.UI.Scroll.Frame.Defaults.Padding;
  end
  return self.padding;
end

function Wired.UI.Scroll.Frame:SetPadding(value)
  self.padding = value;
  
  -- layout
  self:LayoutScrollFrame();
end

-- defaults
function Wired.UI.Scroll.Frame:ApplyScrollBarDefaults()
  
end

-- layout
function Wired.UI.Scroll.Frame:LayoutScrollFrame()
  
  -- size
  local horizontalSize = self:GetHorizontalBar():GetWidth();
  local verticalSize = self:GetVerticalBar():GetHeight();
  
  -- mask
  self:GetMask():SetWidth(self:GetWidth() - (self:GetPadding() * 2));
  self:GetMask():SetHeight(self:GetHeight() - (self:GetPadding() * 2));
  
  -- scale
  local scale = self:GetComponentScale();
  
  -- over
  local width = 0;
  local height = 0;
  if (self:GetComponent()) then
    
    -- horizontal
    if (not scale.horizontal) then
      width = self:GetComponent():GetWidth() - self:GetMask():GetWidth();
    end
    
    -- vertical
    if (not scale.vertical) then
      height = self:GetComponent():GetHeight() - self:GetMask():GetHeight();
    end
  end
  
  -- too big
  if (
    ((width > 0) and (height > 0)) or
    ((width > 0) and (height + self:GetHorizontalBar():GetHeight() + self:GetPadding() > 0) and (not scale.vertical)) or
    ((width + self:GetVerticalBar():GetWidth() + self:GetPadding() > 0) and (height > 0) and (not scale.horizontal))
  ) then
    
    -- bar
    self:GetHorizontalBar():SetWidth(self:GetWidth() - (self:GetPadding() * 3) - self:GetVerticalBar():GetWidth());
    self:GetVerticalBar():SetHeight(self:GetHeight() - (self:GetPadding() * 3) - self:GetHorizontalBar():GetHeight());
    
    -- mask
    self:GetMask():SetWidth(self:GetWidth() - (self:GetPadding() * 3) - self:GetVerticalBar():GetWidth());
    self:GetMask():SetHeight(self:GetHeight() - (self:GetPadding() * 3) - self:GetHorizontalBar():GetHeight());
    
    -- layout
    self:GetHorizontalBar():LayoutScrollBar();
    self:GetVerticalBar():LayoutScrollBar();
    
    -- visible
    self:GetHorizontalBar():SetVisible(true);
    self:GetVerticalBar():SetVisible(true);
    
  -- too wide
  elseif (width > 0) then
    
    -- bar
    self:GetHorizontalBar():SetWidth(self:GetWidth() - (self:GetPadding() * 2));
    
    -- mask
    self:GetMask():SetWidth(self:GetWidth() - (self:GetPadding() * 2));
    self:GetMask():SetHeight(self:GetHeight() - (self:GetPadding() * 3) - self:GetHorizontalBar():GetHeight());
    
    -- visible
    self:GetHorizontalBar():SetVisible(true);
    self:GetVerticalBar():SetVisible(false);
  
  -- too tall
  elseif (height > 0) then
    
    -- bar
    self:GetVerticalBar():SetHeight(self:GetHeight() - (self:GetPadding() * 2));
    
    -- mask
    self:GetMask():SetWidth(self:GetWidth() - (self:GetPadding() * 3) - self:GetVerticalBar():GetWidth());
    self:GetMask():SetHeight(self:GetHeight() - (self:GetPadding() * 2));
    
    -- visible
    self:GetHorizontalBar():SetVisible(false);
    self:GetVerticalBar():SetVisible(true);
    
  -- fits
  else
    
    -- visible
    self:GetHorizontalBar():SetVisible(false);
    self:GetVerticalBar():SetVisible(false);
  end
  
  -- scale
  if (self:GetComponent()) then
    if (scale.horizontal) then
      self:GetComponent():SetWidth(self:GetMask():GetWidth());
    end
    if (scale.vertical) then
      self:GetComponent():SetHeight(self:GetMask():GetHeight());
    end
  end
  
  -- bar size
  if (horizontalSize == self:GetHorizontalBar():GetWidth()) then
    self:GetHorizontalBar():LayoutScrollBar();
  end
  if (verticalSize == self:GetVerticalBar():GetHeight()) then
    self:GetVerticalBar():LayoutScrollBar();
  end
  
  -- redraw
  if (
    (self:GetHorizontalBar():GetValue() == -1) and
    (self:GetVerticalBar():GetValue() == -1)
  ) then
    self:Scroll();
  
  -- scroll to top
  else
    self:GetHorizontalBar():SetValue(-1);
    self:GetVerticalBar():SetValue(-1);
  end
end

-- scroll
function Wired.UI.Scroll.Frame:Scroll()
  
  -- exists
  if (self:GetComponent()) then
    
    -- width
    local width = self:GetComponent():GetWidth() - self:GetMask():GetWidth();
    
    -- height
    local height = self:GetComponent():GetHeight() - self:GetMask():GetHeight();
    
    -- offset
    local offset = {
      x = 0,
      y = 0
    };
    
    if (width > 0) then
      offset.x = - ((width / 2) * self:GetHorizontalBar():GetValue());
    end
    
    if (height > 0) then
      offset.y = - ((height / 2) * self:GetVerticalBar():GetValue());
    end
    
    -- position
    self:GetComponent():SetPoint({child = "CENTER", parent = "CENTER"}, offset);
  end
end

-- wheel
function Wired.UI.Scroll.Frame:Wheel(value)
  if (self:GetVerticalBar():GetVisible()) then
    self:GetVerticalBar():SetValue(self:GetVerticalBar():GetValue() + (value * Wired.UI.Scroll.Frame.Defaults.Rate));
  elseif (self:GetHorizontalBar():GetVisible()) then
    self:GetHorizontalBar():SetValue(self:GetHorizontalBar():GetValue() + (value * Wired.UI.Scroll.Frame.Defaults.Rate));
  end
end
