-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

-- class
Wired.UI.Scroll.List = class(Wired.UI.Scroll.Frame);

-- defaults
Wired.UI.Scroll.List.Defaults = {
  Size = {
    Display = 5
  }
};

-- constructor
function Wired.UI.Scroll.List:Constructor(name, parent)
  Wired.UI.Scroll.Frame.Constructor(self, name, parent);
  
  -- components
  
  -- list
  self:SetList(Wired.UI.List(self:GetName() .. "_List", self:GetMask()));
  self:GetList():SetPadding(0);
  self:GetList():SetVisible(true);
  
  -- component
  self:SetComponent(self:GetList());
  self:SetComponentScale({
    horizontal = true,
    vertical = false
  });
  
  -- defaults
  self:ApplyScrollListDefaults();
  
  -- events
  
  -- size
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      self:GetList():LayoutCells();
      self:Scroll();
    end
  ));
  
  -- size changed
  self:GetList().Wired.Event.Listener:Add(Wired.Event.Listener(
    "SizeChanged",
    function(sender, value)
      
      -- layout
      self:LayoutScrollList();
    end,
    true
  ));
  
  
  -- cells changed
  self:GetList().Wired.Event.Listener:Add(Wired.Event.Listener(
    "CellsChanged",
    function(sender, value)
      
      -- layout
      self:LayoutScrollFrame();
      
      -- select
      self:Select(self:GetCells()[1]);
    end,
    true
  ));
  
end

-- list
function Wired.UI.Scroll.List:GetList()
  return self.list;
end

function Wired.UI.Scroll.List:SetList(value)
  self.list = value;
end

-- selected
function Wired.UI.Scroll.List:GetSelected()
  return self:GetList():GetSelected();
end

function Wired.UI.Scroll.List:SetSelected(value)
  self:GetList():SetSelected(value);
end

-- cells
function Wired.UI.Scroll.List:GetCells()
  return self:GetList():GetCells();
end

function Wired.UI.Scroll.List:SetCells(value)
  self:GetList():SetCells(value);
end

-- size
function Wired.UI.Scroll.List:GetSize()
  return self:GetList():GetSize();
end

function Wired.UI.Scroll.List:SetSize(value)
  self:GetList():SetSize(value);
end

-- display size
function Wired.UI.Scroll.List:GetDisplaySize()
  if (not self.displaySize) then
    self.displaySize = Wired.UI.Scroll.List.Defaults.Size.Display;
  end
  return self.displaySize;
end

function Wired.UI.Scroll.List:SetDisplaySize(value)
  self.displaySize = value;
  
  -- layout
  self:LayoutScrollList();
end

-- defaults
function Wired.UI.Scroll.List:ApplyScrollListDefaults()
  
end

-- layout
function Wired.UI.Scroll.List:LayoutScrollList()
  
  -- cell height
  local height = 0;
  if (self:GetCells()) then
    if (self:GetCells()[1]) then
      height = self:GetCells()[1]:GetHeight();
    end
  end
  
  -- size
  local size = math.min(self:GetDisplaySize(), self:GetList():GetSize());
  self:SetHeight((height * size) + (self:GetPadding() * (size + 1)));
end

-- select
function Wired.UI.Scroll.List:Select(value)
  self:GetList():Select(value);
end