-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Scroll.Slider = class(Wired.UI.Button);

-- constants
Wired.UI.Scroll.Slider.Constants = {
  Orientation = {
    Horizontal = 1,
    Vertical = 2
  },
  Texture = {
    Horizontal = {
      Decrease = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Decrease/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Decrease/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Decrease/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Decrease/Disabled.png")
      },
      Increase = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Increase/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Increase/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Increase/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Increase/Disabled.png")
      },
      Middle = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Middle/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Middle/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Middle/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Horizontal/Slider/Middle/Disabled.png")
      }
    },
    Vertical = {
      Decrease = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Decrease/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Decrease/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Decrease/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Decrease/Disabled.png")
      },
      Increase = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Increase/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Increase/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Increase/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Increase/Disabled.png")
      },
      Middle = {
        Down = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Middle/Down.png"),
        Over = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Middle/Over.png"),
        Enabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Middle/Enabled.png"),
        Disabled = Wired.UI.Texture("Wired", "UI/Scroll/Resources/Vertical/Slider/Middle/Disabled.png")
      }
    }
  }
};

-- defaults
Wired.UI.Scroll.Slider.Defaults = {
  Size = Wired.UI.Button.Defaults.Width,
  Orientation = Wired.UI.Scroll.Slider.Constants.Orientation.Vertical
};

function Wired.UI.Scroll.Slider:Constructor(name, parent)
  Wired.UI.Button.Constructor(self, name, parent);
  
  -- components
  
  -- decrease button
  self:SetDecreaseButton(Wired.UI.Trion.Texture(self:GetName() .. "_DecreaseButton", self));
  self:GetDecreaseButton():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  self:GetDecreaseButton():SetLayer(2);
  self:GetDecreaseButton():SetColor(Wired.UI.Color.Clear);
  self:GetDecreaseButton():SetVisible(true);
  
  -- increase button
  self:SetIncreaseButton(Wired.UI.Trion.Texture(self:GetName() .. "_IncreaseButton", self));
  self:GetIncreaseButton():SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = 0, y = 0});
  self:GetIncreaseButton():SetLayer(2);
  self:GetIncreaseButton():SetColor(Wired.UI.Color.Clear);
  self:GetIncreaseButton():SetVisible(true);
  
  -- drag button
  self:SetDragButton(Wired.UI.Trion.Texture(self:GetName() .. "_DragButton", self));
  self:GetDragButton():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetDragButton():SetLayer(1);
  self:GetDragButton():SetColor(Wired.UI.Color.Clear);
  self:GetDragButton():SetVisible(true);
  
  -- defaults
  self:ApplyScrollSliderDefaults()
  
  -- events
  
  -- layout
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function()
      self:LayoutScrollSlider();
    end
  ));
end

-- orientation
function Wired.UI.Scroll.Slider:GetOrientation()
  return self.orientation;
end

function Wired.UI.Scroll.Slider:SetOrientation(value)
  self.orientation = value;
  
  -- update textures
  self:UpdateTextures();
end

-- decrease button
function Wired.UI.Scroll.Slider:GetDecreaseButton()
  return self.decreaseButton;
end

function Wired.UI.Scroll.Slider:SetDecreaseButton(value)
  self.decreaseButton = value;
end

-- increase button
function Wired.UI.Scroll.Slider:GetIncreaseButton()
  return self.increaseButton;
end

function Wired.UI.Scroll.Slider:SetIncreaseButton(value)
  self.increaseButton = value;
end

-- drag button
function Wired.UI.Scroll.Slider:GetDragButton()
  return self.dragButton;
end

function Wired.UI.Scroll.Slider:SetDragButton(value)
  self.dragButton = value;
end

-- defaults
function Wired.UI.Scroll.Slider:ApplyScrollSliderDefaults()
  
  -- orientation
  self:SetOrientation(Wired.UI.Scroll.Slider.Defaults.Orientation);
end

-- layout
function Wired.UI.Scroll.Slider:LayoutScrollSlider()
  
  -- decrease
  self:GetDecreaseButton():SetWidth(Wired.UI.Scroll.Slider.Defaults.Size);
  self:GetDecreaseButton():SetHeight(Wired.UI.Scroll.Slider.Defaults.Size);
  
  -- increase
  self:GetIncreaseButton():SetWidth(Wired.UI.Scroll.Slider.Defaults.Size);
  self:GetIncreaseButton():SetHeight(Wired.UI.Scroll.Slider.Defaults.Size);
  
  -- horizontal
  if (self:GetOrientation() == Wired.UI.Scroll.Slider.Constants.Orientation.Horizontal) then
    self:GetDragButton():SetWidth(math.max(self:GetDecreaseButton():GetWidth(), self:GetWidth() - ((self:GetDecreaseButton():GetWidth() + self:GetIncreaseButton():GetWidth()) / 2)));
    self:GetDragButton():SetHeight(Wired.UI.Scroll.Slider.Defaults.Size);
    
  -- vertical
  elseif (self:GetOrientation() == Wired.UI.Scroll.Slider.Constants.Orientation.Vertical) then
    self:GetDragButton():SetWidth(Wired.UI.Scroll.Slider.Defaults.Size);
    self:GetDragButton():SetHeight(math.max(self:GetDecreaseButton():GetHeight(), self:GetHeight() - ((self:GetDecreaseButton():GetHeight() + self:GetIncreaseButton():GetHeight()) / 2)));
  end
end

-- update textures
function Wired.UI.Scroll.Slider:UpdateTextures()
  
  -- textures
  local textures = nil;
  
  -- horizontal
  if (self:GetOrientation() == Wired.UI.Scroll.Slider.Constants.Orientation.Horizontal) then
    textures = Wired.UI.Scroll.Slider.Constants.Texture.Horizontal;
    
  -- vertical
  elseif (self:GetOrientation() == Wired.UI.Scroll.Slider.Constants.Orientation.Vertical) then
    textures = Wired.UI.Scroll.Slider.Constants.Texture.Vertical;
  end
  
  -- apply
  self:SetTextures(textures);
end

-- update texture
function Wired.UI.Scroll.Slider:UpdateTexture()
  
  -- texture
  local texture = nil;
  
  -- disabled
  if (not self:GetEnabled()) then
    texture = "Disabled";
    
  -- mouse down
  elseif (self:GetMouseDown()) then
    texture = "Down";
    
  -- mouse over
  elseif (self:GetMouseOver()) then
    texture = "Over";
    
  -- enabled
  else
    texture = "Enabled";
  end
  
  -- apply
  if (self:GetTextures() and texture) then
    self:GetDecreaseButton():SetTexture(self:GetTextures().Decrease[texture]);
    self:GetIncreaseButton():SetTexture(self:GetTextures().Increase[texture]);
    self:GetDragButton():SetTexture(self:GetTextures().Middle[texture]);
  end
end
