-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Slider = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Slider.Constants = {
  
};

-- defaults
Wired.UI.Slider.Defaults = {
  Size = {
    Base = {
      Width = 100,
      Height = 20
    },
    Label = {
      Width = 30,
      Height = 20
    }
  },
  Scale = 255,
  Value = 127,
  Selected = 127,
  Percentage = false
};

function Wired.UI.Slider:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- self
  self:SetColor(Wired.UI.Color.Clear);
  
  -- bar
  self:SetBar(Wired.UI.Trion.Frame(self:GetName() .. "_Bar", self));
  self:GetBar():SetVisible(true);
  self:GetBar():SetPoint({child = "CENTERLEFT", parent = "CENTERLEFT"}, {x = 0, y = 0});
  
  -- label
  self:SetLabel(Wired.UI.Trion.Text(self:GetName() .. "_Label", self));
  self:GetLabel():SetVisible(true);
  self:GetLabel():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {x = 0, y = 0});
  self:GetLabel():SetText("100");
  self:GetLabel():SetFontColor(Wired.UI.Color.White);
  self:GetLabel():SetColor(Wired.UI.Color.Black);
  
  -- left click Bar
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:Clicked();
    end
  ));
  
  -- mouse move Bar
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseMove",
    function(sender, value)
      self:Explored();
    end
  ));
  
  -- mouse out Bar
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:Exited();
    end
  ));
  
  -- defaults
  self:ApplySliderDefaults();
end

-- Bar
function Wired.UI.Slider:GetBar()
  return self.Bar;
end

function Wired.UI.Slider:SetBar(value)
  self.Bar = value;
end

-- label
function Wired.UI.Slider:GetLabel()
  return self.label;
end

function Wired.UI.Slider:SetLabel(value)
  self.label = value;
end

-- padding
function Wired.UI.Slider:GetPadding()
  if (not self.padding) then
    self.padding = 0;
  end
  return self.padding;
end

function Wired.UI.Slider:SetPadding(value)
  self.padding = value;
  
  -- update size
  self:UpdateSize();
end

-- scale
function Wired.UI.Slider:GetScale()
  return self.scale;
end

function Wired.UI.Slider:SetScale(value)
  self.scale = value;
  
  -- update
  self:Update();
end

-- value
function Wired.UI.Slider:GetValue()
  if (not self.value) then
    self.value = Wired.UI.Slider.Defaults.Scale;
  end
  return self.value;
end

function Wired.UI.Slider:SetValue(value)
  self.value = value;
  
  -- update
  self:Update();
  
  -- value changed
  if (type(self.Wired.Event.ValueChanged) == "function") then
    self.Wired.Event:ValueChanged();
  end
end

-- selected
function Wired.UI.Slider:GetSelected()
  return self.selected;
end

function Wired.UI.Slider:SetSelected(value)
  self.selected = value;
  
  -- value
  self:SetValue(value);
end

-- percentage
function Wired.UI.Slider:GetPercentage()
  return self.percentage;
end

function Wired.UI.Slider:SetPercentage(value)
  self.percentage = value;
end

-- defaults
function Wired.UI.Slider:ApplySliderDefaults()
  
  -- size
  self:GetBar():SetWidth(198);
  self:GetBar():SetHeight(40);
  
  -- self
  self:SetWidth(260);
  self:SetHeight(40);
  
  -- label
  self:GetLabel():ScaleFontSizeVertically(self);
  
  -- percentage
  self:SetPercentage(Wired.UI.Slider.Defaults.Percentage);
  
  -- scale
  self:SetScale(Wired.UI.Slider.Defaults.Scale);
  
  -- value
  self:SetValue(Wired.UI.Slider.Defaults.Value);
  
  -- selected
  self:SetSelected(Wired.UI.Slider.Defaults.Selected);
end

-- mouse value
function Wired.UI.Slider:GetMouseValue()
  return math.max(0, math.min(1,
    (Inspect.Mouse().x - self:GetBar():GetLeft()) /
    (self:GetBar():GetWidth() - 1)
  )) * self:GetScale();
end

-- clicked
function Wired.UI.Slider:Clicked()
  self:SetSelected(self:GetMouseValue());
end

-- explored
function Wired.UI.Slider:Explored()
  self:SetValue(self:GetMouseValue());
end

-- exited
function Wired.UI.Slider:Exited()
  self:SetValue(self:GetSelected());
end

-- update size
function Wired.UI.Slider:UpdateSize()
  
end

-- update
function Wired.UI.Slider:Update()
  
  -- scale color
  self:ScaleColor();
  
  -- scale label
  self:ScaleLabel();
end

-- scale color
function Wired.UI.Slider:GetScaledRed()
  local scale = self:GetScale() / 2;
  return 1 - ((math.max(scale, self:GetValue()) - scale) / scale);
end

function Wired.UI.Slider:GetScaledGreen()
  local scale = self:GetScale() / 2;
  return math.max(0, math.min(scale, self:GetValue()) / scale);
end

function Wired.UI.Slider:ScaleColor()
  self:GetBar():SetColor(Wired.UI.Color(
    self:GetScaledRed(),
    self:GetScaledGreen(),
    0,
    1
  ));
end

-- scale label
function Wired.UI.Slider:GetScaledLabel()
  local result = 0;
  if (self:GetPercentage()) then
    result = math.ceil(
      (self:GetValue() * 100) / self:GetScale()
    );
  else
    result = math.ceil(math.max(0, math.min(
      self:GetScale(),
      self:GetValue()
    )));
  end
  return result;
end

function Wired.UI.Slider:ScaleLabel()
  local count = self:GetScaledLabel();
  if (count ~= self:GetLabel():GetText()) then
    self:GetLabel():SetText(tostring(count));
    self:GetLabel():SetWidth(60);
  end
end

--[[

Slider = class(Turbine.UI.Control);

-- constants
Slider.Alignment = Imez.UI.Alignments.MiddleCenter;
Slider.Padding = {left = 0, right = 10, top = 0, bottom = 0};
Slider.Scale = {Color = 255, Value = 1};

-- defaults
Slider.Defaults = {
  DisplaySize = {width = 100, height = 20},
  InputSize = {width = 42, height = 22},
  Scale = 100,
  Value = Slider.Scale.Value / 2
};

function Slider:Constructor()
  Turbine.UI.Control.Constructor(self);

  -- display
  self.display = Turbine.UI.Control();
  self.display:SetParent(self);
  self.display:AddBorder();
  self.display:AddPadding(Slider.Padding);
  self.display:SetVisible(true);
  self.display:AddListener(
    Imez.Event.Listener("MouseLeave", function(sender, args)
      self:DisplayExited(args);
    end)
  );
  self.display:AddListener(
    Imez.Event.Listener("MouseMove", function(sender, args)
      self:DisplayExplored(args);
    end)
  );
  self.display:AddListener(
    Imez.Event.Listener("MouseClick", function(sender, args)
      self:DisplayClicked(args);
    end)
  );

  -- input
  self.input = Imez.UI.UpdatingTextBar();
  self.input:SetParent(self);
  self.input:GetTextStyle():SetAlignment(Slider.Alignment);
  self.input:SetVisible(true);
  self.input:AddListener(
    Imez.Event.Listener("TextUpdated", function(sender, args)
      self:InputUpdated();
    end)
  );

  -- layout
  self:AddListener(
    Imez.Event.Listener("SizeChanged", self.Layout)
  );

  -- defaults
  self:ApplyDefaults();
end

-- getters and setters
function Slider:GetScale()
  return self.scale;
end

function Slider:SetScale(value)
  self.scale = value;
end

function Slider:GetValue()
  return self.value;
end

function Slider:SetValue(value)
  self.value = value;

  -- scale color
  self:ScaleColor();

  -- fire value changed event
  if (type(self.ValueChanged) == "function") then
    self:ValueChanged(value);
  end
end

function Slider:GetSelected()
  return self.selected;
end

function Slider:SetSelected(value)
  self.selected = value;

  -- determine input state
  if (self:NotExternal()) then

    -- fire value selected event
    if (type(self.ValueSelected) == "function") then
      self:ValueSelected(value);
    end

    -- persistence
    self:PromptSave();
  end
end

function Slider:GetColor()
  return self.display:GetBackColor();
end

function Slider:SetColor(value)
  self.display:SetBackColor(value);
end

-- defaults
function Slider:ApplyDefaults()

  -- scale
  self:SetScale(Slider.Defaults.Scale);

  -- initialize value
  self:SetRelayExternal(false);
  self:SetRelayInternal(false);
  self:Input(Slider.Defaults.Value);

  -- size components
  self.input:SetSize(
    Slider.Defaults.InputSize.width,
    Slider.Defaults.InputSize.height
  );
  self.display:SetSize(
    Slider.Defaults.DisplaySize.width,
    Slider.Defaults.DisplaySize.height
  );
  self:SetSize(
    self.input:GetRelativeWidth() + self.display:GetRelativeWidth(),
    self.input:GetHeight()
  );
end

-- layout
function Slider:Layout()
  self.display:SetRelativePosition(0, 0);
  self.input:SetRelativePosition(self.display:GetRelativeRight(), 0);
end

-- input
function Slider:Input(value)
  self.input:SetText(Imez.Math.Round(value * self:GetScale()));
end

function Slider:InputUpdated()

  -- convert to number
  local number = tonumber(self.input:GetText());
  if (not number) then

    -- clean up text
    self:SetRelayInternal(false);
    self.input:SetText(0);
  else

    -- restrict number
    local restricted = Imez.Math.Round(
      Imez.Math.Restrict(number, 0, self:GetScale())
    );
    if (restricted ~= number) then

      -- clean up text
      self:SetRelayInternal(false);
      self.input:SetText(restricted);
    else

      -- scale value
      self:SetValue(restricted / self:GetScale());

      -- determine input state
      if (self:NotInternal()) then
        self:SetSelected(self:GetValue());
      end
    end
  end
end

-- display events
function Slider:DisplayExited(args)
  self:InternalInput(self:GetSelected());
end

function Slider:DisplayExplored(args)
  self:InternalInput(
    (args.X / (self.display:GetWidth() - 1)) * Slider.Scale.Value
  );
end

function Slider:DisplayClicked(args)
  self:SetSelected(self:GetValue());
end

-- color
function Slider:GetScaledRed()
  local scale = Slider.Scale.Value / 2;
  return 1 - ((math.max(scale, self:GetValue()) - scale) / scale);
end

function Slider:GetScaledGreen()
  local scale = Slider.Scale.Value / 2;
  return math.min(scale, self:GetValue()) / scale;
end

function Slider:ScaleColor()
  self:SetColor(Turbine.UI.Color(
    self:GetScaledRed(),
    self:GetScaledGreen(),
    0
  ));
end

-- persistence override
function Slider:Load(data)
  if (data) then
    self:Input(data);
  end
end

function Slider:Save()
  return self:GetValue();
end
--]]