-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

-- class
Wired.UI.Tab.List = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.Tab.List.Constants = {
  Orientation = {
    Vertical = 1,
    Horizontal = 2
  }
};

-- defaults
Wired.UI.Tab.List.Defaults = {
  Spacing = 2,
  Padding = 2
};

function Wired.UI.Tab.List:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- values
  self:SetColor(Wired.UI.Color.Clear);
  
  -- list
  self:SetList(Wired.UI.List(self:GetName() .. "_List", self));
  self:GetList():SetOrientation(Wired.UI.List.Constants.Orientation.Horizontal);
  self:GetList():SetColor(Wired.UI.Color.Clear);
  self:GetList():SetPadding(0);
  self:GetList():SetVisible(true);
  
  -- frame
  self:SetFrame(Wired.UI.Tab.Frame(self:GetName() .. "_Frame", self));
  self:GetFrame():SetColor(Wired.UI.Color.Clear);
  self:GetFrame():SetVisible(true);
  
  -- defaults
  self:ApplyTabListDefaults();
  
  -- size
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function()
      self:LayoutTabList();
    end
  ));
end

-- list
function Wired.UI.Tab.List:GetList()
  return self.list;
end

function Wired.UI.Tab.List:SetList(value)
  self.list = value;
end

-- frame
function Wired.UI.Tab.List:GetFrame()
  return self.frame;
end

function Wired.UI.Tab.List:SetFrame(value)
  self.frame = value;
end

-- spacing
function Wired.UI.Tab.List:GetSpacing()
  if (not self.spacing) then
    self.spacing = Wired.UI.Tab.List.Defaults.Spacing;
  end
  return self.spacing;
end

function Wired.UI.Tab.List:SetSpacing(value)
  self.spacing = value;
end

-- padding
function Wired.UI.Tab.List:GetPadding()
  if (not self.padding) then
    self.padding = Wired.UI.Tab.List.Defaults.Padding;
  end
  return self.padding;
end

function Wired.UI.Tab.List:SetPadding(value)
  self.padding = value;
end

-- tabs
function Wired.UI.Tab.List:GetTabs()
  if (type(self.tabs) ~= "table") then
    self.tabs = {};
  end
  return self.tabs;
end

function Wired.UI.Tab.List:SetTabs(value)
  self.tabs = value;
end

function Wired.UI.Tab.List:AddTab(tab)
  
  -- index
  self:GetTabs()[tab:GetTitle()] = tab;
  
  -- anchor
  tab:SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = 0});
  
  -- create cell
  tab:SetTabCell(Wired.UI.Cell(tab:GetTitle() .. "_TabCell", self:GetList()));
  tab:GetTabCell():SetTitle(tab:GetTitle());
  tab:GetTabCell():SetVisible(true);
  
  -- left click
  tab:GetTabCell().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      self:Select(tab);
    end,
    true
  ));
  
  -- add cell
  self:GetList():AddCell(tab:GetTabCell());
  
  -- scale cells
  self:GetList():ScaleCells();
end

-- defaults
function Wired.UI.Tab.List:ApplyTabListDefaults()
  
end

-- layout
function Wired.UI.Tab.List:LayoutTabList()
  
  -- list
  self:GetList():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = self:GetPadding(), y = self:GetPadding()});
  self:GetList():SetWidth(self:GetWidth() - (self:GetPadding() * 2));
  
  -- frame
  self:GetFrame():SetPoint({child = "BOTTOMRIGHT", parent = "BOTTOMRIGHT"}, {x = -self:GetPadding(), y = -self:GetPadding()});
  self:GetFrame():SetWidth(self:GetWidth() - (self:GetPadding() * 2));
  self:GetFrame():SetHeight(self:GetHeight() - (self:GetPadding() * 2) - self:GetSpacing() - self:GetList():GetHeight());
  
  -- scale cells
  self:GetList():ScaleCells();
end

-- select
function Wired.UI.Tab.List:Select(value)
  for k, tab in pairs(self:GetTabs()) do
    if (value:GetTitle() == tab:GetTitle()) then
      tab:SetVisible(true);
      self:GetList():Select(tab:GetTabCell());
    else
      tab:SetVisible(false);
    end
  end
end
