-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

-- constructor
Wired.UI.TargetGauge = class(Wired.UI.Trion.Frame);

-- constants
Wired.UI.TargetGauge.Constant = {
  Ratio = {
    Cast = 10 / 18,
    Health = 18 / 18,
    Resource = 1 / 18
  }
};

-- defaults
Wired.UI.TargetGauge.Default = {
  Color = {
    Self = Wired.UI.Color.Clear
  },
  FontSize = 18
};

-- constructor
function Wired.UI.TargetGauge:Constructor(name, parent, application)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- parameters
  self:SetApplication(application);
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- health
  self:SetHealth(Wired.UI.Gauge(self:GetName() .. "_Health", self, self:GetApplication()));
  self:GetHealth():SetVisible(true);
  
  -- resource
  self:SetResource(Wired.UI.Gauge(self:GetName() .. "_Resource", self, self:GetApplication()));
  self:GetResource():SetVisible(false);
  
  -- cast
  self:SetCast(Wired.UI.Gauge(self:GetName() .. "_Cast", self, self:GetApplication()));
  self:GetCast():SetVisible(false);
  
  -- apply defaults
  self:ApplyTargetGaugeDefaults();
  
  -- layout
  self:LayoutTargetGauge();
  
  -- events
  
  -- drag
  self:GetBase().Wired.UI.Drag:Enable(true);
  
  -- zoom forward
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function()
      self:RightClick();
    end
  ));
  
  -- zoom back
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(1);
    end
  ));
  
  -- zoom forward
  self:GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-1);
    end
  ));
  
  -- visible
  self:GetResource().Wired.Event.Listener:Add(Wired.Event.Listener(
    "VisibleChanged",
    function(sender, value)
      self:LayoutTargetGauge();
    end,
    true
  ));
  
  -- visible
  self:GetCast().Wired.Event.Listener:Add(Wired.Event.Listener(
    "VisibleChanged",
    function(sender, value)
      self:LayoutTargetGauge();
    end,
    true
  ));
end

-- application
function Wired.UI.TargetGauge:GetApplication()
  return self.application;
end

function Wired.UI.TargetGauge:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.UI.TargetGauge:GetIdle()
  return self.idle;
end

function Wired.UI.TargetGauge:SetIdle(value)
  self.idle = value;
end

-- health
function Wired.UI.TargetGauge:GetHealth()
  return self.health;
end

function Wired.UI.TargetGauge:SetHealth(value)
  self.health = value;
end

-- resource
function Wired.UI.TargetGauge:GetResource()
  return self.resource;
end

function Wired.UI.TargetGauge:SetResource(value)
  self.resource = value;
end

-- cast
function Wired.UI.TargetGauge:GetCast()
  return self.cast;
end

function Wired.UI.TargetGauge:SetCast(value)
  self.cast = value;
end

-- mark
function Wired.UI.TargetGauge:GetMark()
  return self:GetHealth():GetMark();
end

function Wired.UI.TargetGauge:SetMark(value)
  self:GetHealth():SetMark(value);
end

-- locked
function Wired.UI.TargetGauge:GetLocked()
  return self.locked
end

function Wired.UI.TargetGauge:SetLocked(value)
  self.locked = value;
  
  -- drag
  self:GetBase().Wired.UI.Drag.data.enabled = not self.locked;
  
  -- fire locked event
  if (type(self.Wired.Event.LockedChanged) == "function") then
    self.Wired.Event:LockedChanged();
  end
end

-- font size
function Wired.UI.TargetGauge:GetFontSize()
  return self:GetHealth():GetFontSize();
end

function Wired.UI.TargetGauge:SetFontSize(value)
  self:GetCast():SetFontSize(value * Wired.UI.TargetGauge.Constant.Ratio.Cast);
  self:GetHealth():SetFontSize(value * Wired.UI.TargetGauge.Constant.Ratio.Health);
  self:GetResource():SetFontSize(value * Wired.UI.TargetGauge.Constant.Ratio.Resource);
  
  -- layout
  self:LayoutTargetGauge();
end

-- acquired
function Wired.UI.TargetGauge:GetAcquired()
  if (not self.acquired) then
    self.acquired = Inspect.Time.Frame();
  end
  return self.acquired;
end

function Wired.UI.TargetGauge:SetAcquired(value)
  self.acquired = value;
end

-- defaults
function Wired.UI.TargetGauge:ApplyTargetGaugeDefaults()
  
  -- color
  self:SetColor(Wired.UI.TargetGauge.Default.Color.Self);
  
  -- font size
  self:SetFontSize(Wired.UI.TargetGauge.Default.FontSize);
  
  -- cast
  self:GetCast():GetValueLabel():SetVisible(false);
  self:GetCast():GetMarkLabel():SetVisible(false);
  
  -- resource
  self:GetResource():GetLabel():SetVisible(false);
  self:GetResource():GetValueLabel():SetVisible(false);
  self:GetResource():GetMarkLabel():SetVisible(false);
end

-- layout
function Wired.UI.TargetGauge:LayoutTargetGauge()
  
  -- height
  local height = 0;
  
  -- cast
  if (self:GetCast():GetVisible()) then
    self:GetCast():SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = height});
    height = height + self:GetCast():GetHeight();
  end
  
  -- health
  if (self:GetHealth():GetVisible()) then
    self:GetHealth():SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = height});
    height = height + self:GetHealth():GetHeight();
  end
  
  -- resource
  if (self:GetResource():GetVisible()) then
    self:GetResource():SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = height});
    height = height + self:GetResource():GetHeight();
  end
  
  -- width
  self:SetWidth(self:GetHealth():GetWidth());
  self:GetCast():SetWidth(self:GetWidth());
  self:GetResource():SetWidth(self:GetWidth());
  
  -- height
  if (self:GetLocked() and (height ~= self:GetHeight())) then
    self:SetPosition(self:GetLeft(), self:GetTop() - (height - self:GetHeight()));
  end
  self:SetHeight(height);
  
  -- draw
  self:GetCast():Layout();
  self:GetResource():Layout();
end

-- right click
function Wired.UI.TargetGauge:RightClick()
  self:SetLocked(not self:GetLocked());
end

-- wheel
function Wired.UI.TargetGauge:Wheel(value)
  if (not self:GetLocked()) then
    self:SetFontSize(self:GetFontSize() + value);
  end
end
