-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.TextShadow = class(Wired.UI.Trion.Frame);

-- defaults
Wired.UI.TextShadow.Defaults = {
  Color = {
    Font = {
      Label = Wired.UI.Color.White,
      Shadow = Wired.UI.Color.Black,
    }
  },
  Colors = {
    Background = Wired.UI.Color.Clear,
    Font = {
      LabelShadow = Wired.UI.Color.Black
    }
  },
  Offset = {
    Left = 1,
    Top = 1
  }
};

function Wired.UI.TextShadow:Constructor(name, parent)
  Wired.UI.Trion.Frame.Constructor(self, name, parent);
  
  -- create label shadow
  self:SetLabelShadow(Wired.UI.Trion.Text(self:GetName() .. "_LabelShadow", self));
  self:GetLabelShadow():SetPoint({child = "CENTER", parent = "CENTER"}, {x = Wired.UI.TextShadow.Defaults.Offset.Left, y = Wired.UI.TextShadow.Defaults.Offset.Top});
  self:GetLabelShadow():SetLayer(1);
  self:GetLabelShadow():SetVisible(true);
  
  -- create label
  self:SetLabel(Wired.UI.Trion.Text(self:GetName() .. "_Label", self));
  self:GetLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  self:GetLabel():SetLayer(2);
  self:GetLabel():SetVisible(true);
  
  -- defaults
  self:ApplyTextShadowDefaults();
end

-- label shadow
function Wired.UI.TextShadow:GetLabelShadow()
  return self.labelShadow;
end

function Wired.UI.TextShadow:SetLabelShadow(value)
  self.labelShadow = value;
end

-- label
function Wired.UI.TextShadow:GetLabel()
  return self.label;
end

function Wired.UI.TextShadow:SetLabel(value)
  self.label = value;
end

-- text
function Wired.UI.TextShadow:GetText()
  return self:GetLabel():GetText();
end

function Wired.UI.TextShadow:SetText(value)
  self:GetLabelShadow():SetText(value);
  self:GetLabel():SetText(value);
  
  -- layout
  self:LayoutTextShadow();
end

-- font color
function Wired.UI.TextShadow:GetFontColor()
  if (not self.fontColor) then
    self.fontColor = Wired.UI.TextShadow.Defaults.Color.Font.Label;
  end
  return self.fontColor;
end

function Wired.UI.TextShadow:SetFontColor(value)
  self.fontColor = value;
  
  -- update
  self:GetLabel():SetFontColor(self:GetFontColor());
end

-- font size
function Wired.UI.TextShadow:GetFontSize()
  return self:GetLabel():GetFontSize();
end

function Wired.UI.TextShadow:SetFontSize(value)
  self:GetLabelShadow():SetFontSize(value);
  self:GetLabel():SetFontSize(value);
  
  -- layout
  self:LayoutTextShadow();
end

-- width
function Wired.UI.TextShadow:SetWidth(value)
  self:GetLabelShadow():SetWidth(value);
  self:GetLabel():SetWidth(value);
  
  -- layout
  self:LayoutTextShadow();
end

-- height
function Wired.UI.TextShadow:SetHeight(value)
  self:GetLabelShadow():SetHeight(value);
  self:GetLabel():SetHeight(value);
  
  -- layout
  self:LayoutTextShadow();
end

-- wrap
function Wired.UI.TextShadow:GetWrapping()
  return self:GetLabel():GetWrapping();
end

function Wired.UI.TextShadow:SetWrapping(value)
  self:GetLabelShadow():SetWrapping(value);
  self:GetLabel():SetWrapping(value);
end

-- defaults
function Wired.UI.TextShadow:ApplyTextShadowDefaults()
  
  -- color
  self:SetColor(Wired.UI.TextShadow.Defaults.Colors.Background);
  self:GetLabelShadow():SetColor(Wired.UI.TextShadow.Defaults.Colors.Background);
  self:GetLabel():SetColor(Wired.UI.TextShadow.Defaults.Colors.Background);
  
  -- font color
  self:GetLabelShadow():SetFontColor(Wired.UI.TextShadow.Defaults.Colors.Font.LabelShadow);
end

-- layout
function Wired.UI.TextShadow:LayoutTextShadow()
  self:GetBase():SetWidth(self:GetLabel():GetWidth() + Wired.UI.TextShadow.Defaults.Offset.Left);
  self:GetBase():SetHeight(self:GetLabel():GetHeight() + Wired.UI.TextShadow.Defaults.Offset.Top);
end

-- scale font size vertically
function Wired.UI.TextShadow:ScaleFontSizeVertically(component)
  self:GetLabelShadow():ScaleFontSizeVertically(component);
  self:GetLabel():ScaleFontSizeVertically(component);
end
