-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Trion.Base = class();

-- constants
Wired.UI.Trion.Base.Constants = {
  Anchor = "TOPLEFT",
  Types = {
    Frame = "Frame",
    Mask = "Mask",
    Text = "Text",
    Window = "RiftWindow",
    Texture = "Texture",
    Check = "RiftCheckbox",
    Button = "RiftButton"
  }
};

-- defaults
Wired.UI.Trion.Base.Defaults = {
  Visible = false,
  Color = Wired.UI.Color.Black,
  Alpha = 1
};

function Wired.UI.Trion.Base:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
end

-- name
function Wired.UI.Trion.Base:GetName()
  return self.name;
end

function Wired.UI.Trion.Base:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Trion.Base:GetParent()
  return self.parent;
end

function Wired.UI.Trion.Base:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Trion.Base:GetBase()
  return self.base;
end

function Wired.UI.Trion.Base:SetBase(value)
  self.base = value;
end

-- color
function Wired.UI.Trion.Base:GetColor()
  return self.color;
end

function Wired.UI.Trion.Base:SetColor(value)
  self.color = value;
  
  -- update base
  self:GetBase():SetBackgroundColor(
    value:GetRed(),
    value:GetGreen(),
    value:GetBlue(),
    value:GetAlpha()
  );
end

-- alpha
function Wired.UI.Trion.Base:GetAlpha()
  return self:GetBase():GetAlpha();
end

function Wired.UI.Trion.Base:SetAlpha(value)
  self:GetBase():SetAlpha(value);
end

-- visible
function Wired.UI.Trion.Base:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.Trion.Base:SetVisible(value)
  self:GetBase():SetVisible(value);
  
  -- fire visible changed
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- layer
function Wired.UI.Trion.Base:GetLayer()
  return self:GetBase():GetLayer();
end

function Wired.UI.Trion.Base:SetLayer(value)
  self:GetBase():SetLayer(value);
end

-- width
function Wired.UI.Trion.Base:GetWidth()
  return self:GetBase():GetWidth();
end

function Wired.UI.Trion.Base:SetWidth(value)
  self:GetBase():SetWidth(value);
end

-- height
function Wired.UI.Trion.Base:GetHeight()
  return self:GetBase():GetHeight();
end

function Wired.UI.Trion.Base:SetHeight(value)
  self:GetBase():SetHeight(value);
end

-- mouse masking
function Wired.UI.Trion.Base:GetMouseMasking()
  return self:GetBase():GetMouseMasking();
end

function Wired.UI.Trion.Base:SetMouseMasking(value)
  self:GetBase():SetMouseMasking(value);
end

-- secure mode
function Wired.UI.Trion.Base:GetSecureMode()
  return self:GetBase():GetSecureMode();
end

function Wired.UI.Trion.Base:SetSecureMode(value)
  self:GetBase():SetSecureMode(value);
end

-- left
function Wired.UI.Trion.Base:GetLeft()
  return self:GetBase():GetLeft();
end

-- right
function Wired.UI.Trion.Base:GetRight()
  return self:GetBase():GetRight();
end

-- top
function Wired.UI.Trion.Base:GetTop()
  return self:GetBase():GetTop();
end

-- bottom
function Wired.UI.Trion.Base:GetBottom()
  return self:GetBase():GetBottom();
end

-- point

--[[
function Wired.UI.Trion.Base:GetLastPoint()
  return self.lastPoint;
end
--]]

function Wired.UI.Trion.Base:SetPoint(anchor, offset)
  
  --[[
  -- clear
  if (self:GetLastPoint()) then
    if (anchor.child ~= self:GetLastPoint()) then
      self:GetBase():ClearPoint(self:GetLastPoint());
    end
  end
  
  -- last point
  self.lastPoint = anchor.child;
  --]]
  
  -- set
  self:GetBase():SetPoint(
    anchor.child,
    self:GetParent():GetBase(),
    anchor.parent,
    offset.x,
    offset.y
  );
end

-- position
function Wired.UI.Trion.Base:SetPosition(x, y)
  self:GetBase():SetPoint(
    Wired.UI.Trion.Base.Constants.Anchor,
    self:GetParent():GetBase(),
    Wired.UI.Trion.Base.Constants.Anchor,
    x,
    y
  );
end

-- defaults
function Wired.UI.Trion.Base:ApplyBaseDefaults()
  self:SetVisible(Wired.UI.Trion.Base.Defaults.Visible);
  self:SetColor(Wired.UI.Trion.Base.Defaults.Color);
  self:SetAlpha(Wired.UI.Trion.Base.Defaults.Alpha);
  
  -- wire
  Wire(self:GetBase());
  
  -- secure mode
  self:SetSecureMode(self:GetParent():GetBase():GetSecureMode());
end
