-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Window = class();

Wired.UI.Window.Constants = {
  Texture = {
    CloseButton = {
      On = Wired.UI.Texture("Wired", "UI/Resources/close-on.png"),
      Off = Wired.UI.Texture("Wired", "UI/Resources/close-off.png")
    }
  },
  Offset = {
    CloseButton = {
      Left = 2,
      Top = 5
    }
  }
};

Wired.UI.Window.Defaults = {
  Visible = {
    Base = false,
    Bar = false,
    CloseButton = true
  },
  Color = {
    Base = Wired.UI.Color.Clear,
    Bar = Wired.UI.Color.Clear,
    CloseButton = Wired.UI.Color.Clear
  },
  Position = {
    Left = 100,
    Top = 100
  }
};

function Wired.UI.Window:Constructor(name, parent)

  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- base
  self:SetBase(Wired.UI.Trion.Window(self:GetName() .. "_Base", self:GetParent()));
  self:GetBase():SetVisible(false);
  self:GetBase():SetLayer(parent:GetBase():GetLayer() + 1);
  self:GetBase():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  
  -- bar
  self:SetBar(Wired.UI.Trion.Frame(self:GetName() .. "_Bar", self:GetParent()));
  self:GetBar():SetVisible(false);
  self:GetBar():SetLayer(parent:GetBase():GetLayer() + 2);
  self:GetBar():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  
  -- close button
  self:SetCloseButton(Wired.UI.Trion.Texture(self:GetName() .. "_CloseButton", self:GetBar()));
  self:GetCloseButton():SetVisible(false);
  self:GetCloseButton():SetPoint({child = "CENTERRIGHT", parent = "CENTERRIGHT"}, {
    x = Wired.UI.Window.Constants.Offset.CloseButton.Left,
    y = Wired.UI.Window.Constants.Offset.CloseButton.Top
  });
  
  -- drag
  self:GetBar():GetBase().Wired.UI.Drag:Enable(true);
  
  -- persistence
  self:GetBar():GetBase().Wired.Persistence:Enable(self:GetName());
  
  -- x
  self.x = Wired.Persistence.PersistentVariable(
    "Left", self:GetBar():GetBase(), 100
  );
  
  -- y
  self.y = Wired.Persistence.PersistentVariable(
    "Top", self:GetBar():GetBase(), 100
  );
  
  -- defaults
  self:ApplyWindowDefaults();
  
  -- load
  self:Load();
  
  -- resize
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Size",
    function(sender, value)
      self:UpdateSize();
    end
  ));
  
  -- move
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function(sender, value)
      self:UpdatePosition();
    end
  ));
  
  -- left click close button
  self:GetCloseButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:Close();
    end
  ));
  
  -- mouse in close button
  self:GetCloseButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:GetCloseButton():SetTexture(Wired.UI.Window.Constants.Texture.CloseButton.On);
    end
  ));
  
  -- mosue out close button
  self:GetCloseButton():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:GetCloseButton():SetTexture(Wired.UI.Window.Constants.Texture.CloseButton.Off);
    end
  ));
end

-- name
function Wired.UI.Window:GetName()
  return self.name;
end

function Wired.UI.Window:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Window:GetParent()
  return self.parent;
end

function Wired.UI.Window:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Window:GetBase()
  return self.base;
end

function Wired.UI.Window:SetBase(value)
  self.base = value;
end

-- bar
function Wired.UI.Window:GetBar()
  return self.bar;
end

function Wired.UI.Window:SetBar(value)
  self.bar = value;
end

-- close button
function Wired.UI.Window:GetCloseButton()
  return self.closeButton;
end

function Wired.UI.Window:SetCloseButton(value)
  self.closeButton = value;
end

-- visible
function Wired.UI.Window:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.Window:SetVisible(value)
  self:GetBase():SetVisible(value);
  self:GetBar():SetVisible(value);
  
  -- fire visible changed event
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- title
function Wired.UI.Window:GetTitle()
  return self:GetBase():GetTitle();
end

function Wired.UI.Window:SetTitle(value)
  self:GetBase():SetTitle(value);
end

-- width
function Wired.UI.Window:GetWidth()
  return self:GetBase():GetWidth();
end

function Wired.UI.Window:SetWidth(value)
  self:GetBase():SetWidth(value);
end

-- height
function Wired.UI.Window:GetHeight()
  return self:GetBase():GetHeight();
end

function Wired.UI.Window:SetHeight(value)
  self:GetBase():SetHeight(value);
end

-- border size
function Wired.UI.Window:GetBorderSize()
  return self:GetBase():GetBorderSize();
end

-- defaults
function Wired.UI.Window:ApplyWindowDefaults()
  
  -- color
  self:GetBase():SetColor(Wired.UI.Window.Defaults.Color.Base);
  self:GetBar():SetColor(Wired.UI.Window.Defaults.Color.Bar);
  self:GetCloseButton():SetColor(Wired.UI.Window.Defaults.Color.CloseButton);
  
  -- size
  self:GetBar():SetHeight(self:GetBorderSize().top);
  
  -- texture
  self:GetCloseButton():SetTexture(Wired.UI.Window.Constants.Texture.CloseButton.Off);
  
  -- update size
  self:UpdateSize();
  
  -- visible
  self:GetBase():SetVisible(Wired.UI.Window.Defaults.Visible.Base);
  self:GetBar():SetVisible(Wired.UI.Window.Defaults.Visible.Bar);
  self:GetCloseButton():SetVisible(Wired.UI.Window.Defaults.Visible.CloseButton);
end

-- load
function Wired.UI.Window:Load()
  
  -- position
  if (
    (self.x:GetValue() ~= self:GetBar():GetBase():GetLeft()) or
    (self.y:GetValue() ~= self:GetBar():GetBase():GetTop())
  ) then
    self:GetBar():SetPosition(self.x:GetValue(), self.y:GetValue());
  end
  
  -- update position
  self:UpdatePosition();
end

-- position
function Wired.UI.Window:SetPosition(left, top)
  self:GetBar():SetPosition(left + self:GetBorderSize().left, top);
end

-- update position
function Wired.UI.Window:UpdatePosition()
  
  -- base
  self:GetBase():SetPosition(
    self:GetBar():GetLeft() - self:GetBorderSize().left,
    self:GetBar():GetTop()
  );
  
  -- persistence
  self.x:SetValue(self:GetBar():GetBase():GetLeft());
  self.y:SetValue(self:GetBar():GetBase():GetTop());
end

-- update size
function Wired.UI.Window:UpdateSize()
  self:GetBar():SetWidth(
    self:GetWidth() - self:GetBorderSize().left - self:GetBorderSize().right
  );
end

-- display
function Wired.UI.Window:DisplayAtMouse()
  local mouse = Inspect.Mouse();
  self:SetPosition(
    mouse.x - (self:GetBase():GetWidth() / 2),
    mouse.y - (self:GetBase():GetHeight() / 2)
  );
  self:SetVisible(true);
end

-- close
function Wired.UI.Window:Close()
  self:SetVisible(false);
end
