-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.Utility.Application = class();

-- constants
Wired.Utility.Application.Constants = {
  
};

-- defaults
Wired.Utility.Application.Defaults = {
  Enabled = true,
  Debug = false
};

function Wired.Utility.Application:Constructor(name, version, command, date, iconTrayTextures)

  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetVersion(version);
  self:SetCommand(command);
  self:SetDate(date);
  self:SetIconTrayTextures(iconTrayTextures);
    
  -- events
  table.insert(Event.Addon.Load.End, {
    function(value)
      self:Loaded(value);
    end,
    self:GetName(),
    self:GetName() .. " Loaded"
  });
  
  -- register
  WiredApplicationManager:RegisterApplication(self);
end

-- name
function Wired.Utility.Application:GetName()
  return self.name;
end

function Wired.Utility.Application:SetName(value)
  self.name = value;
end

-- version
function Wired.Utility.Application:GetVersion()
  return self.version;
end

function Wired.Utility.Application:SetVersion(value)
  self.version = value;
end

-- command
function Wired.Utility.Application:GetCommand()
  return self.command;
end

function Wired.Utility.Application:SetCommand(value)
  self.command = value;
end

-- date
function Wired.Utility.Application:GetDate()
  return self.date;
end

function Wired.Utility.Application:SetDate(value)
  self.date = value;
end

-- enabled
function Wired.Utility.Application:GetEnabled()
  return self.enabled:GetValue();
end

function Wired.Utility.Application:SetEnabled(value)
  self.enabled:SetValue(value);
end

-- debug
function Wired.Utility.Application:GetDebug()
  return self.debug:GetValue();
end

function Wired.Utility.Application:SetDebug(value)
  self.debug:SetValue(value);
end

-- persistence
function Wired.Utility.Application:GetPersistence()
  return self.persistence;
end

function Wired.Utility.Application:SetPersistence(value)
  self.persistence = value;
  
  -- persistence loaded
  self:PersistenceLoaded();
end

-- context
function Wired.Utility.Application:GetContext()
  return self.context;
end

function Wired.Utility.Application:SetContext(value)
  self.context = value;
end

-- idle
function Wired.Utility.Application:GetIdle()
  return self.idle;
end

function Wired.Utility.Application:SetIdle(value)
  self.idle = value;
end

-- command manager
function Wired.Utility.Application:GetCommandManager()
  return self.commandManager;
end

function Wired.Utility.Application:SetCommandManager(value)
  self.commandManager = value;
end

-- options
function Wired.Utility.Application:GetOptions()
  return self.options;
end

function Wired.Utility.Application:SetOptions(value)
  self.options = value;
end

-- application tray icon textures
function Wired.Utility.Application:GetIconTrayTextures()
  return self.iconTrayTextures;
end

function Wired.Utility.Application:SetIconTrayTextures(value)
  self.iconTrayTextures = value;
end

-- cascade bars
function Wired.Utility.Application:GetCascadeBars()
  if (type(self.cascadeBars) ~= "table") then
    self.cascadeBars = {};
  end
  return self.cascadeBars;
end

function Wired.Utility.Application:SetCascadeBars(value)
  self.cascadeBars = value;
end

-- register cascade bar
function Wired.Utility.Application:RegisterCascadeBar(value);
  
  -- index by name
  self:GetCascadeBars()[value:GetName()] = value;
end

-- unregister cascade bar
function Wired.Utility.Application:UnregisterCascadeBar(value);
  
  -- deindex by name
  self:GetCascadeBars()[value:GetName()] = nil;
end

-- welcome
function Wired.Utility.Application:Welcome()
  
  -- enabled
  local enabled = "Enabled";
  if (not self:GetEnabled()) then
    enabled = "Disabled";
  end
  
  -- output
  print("v" .. self:GetVersion() .. " [" .. enabled .. "] /" .. self:GetCommand() .. " for commands");
end

-- loaded
function Wired.Utility.Application:Loaded(value)
  if (value == self:GetName()) then
    
    -- fire loaded event
    if (type(self.Wired.Event.Loaded) == "function") then
      self.Wired.Event:Loaded();
    end
  end
end

-- persistence loaded
function Wired.Utility.Application:PersistenceLoaded()

  -- create context
  self:SetContext(Wired.UI.Trion.Context(self));
  
  -- create idle
  self:SetIdle(Wired.Utility.Idle(self));
  
  -- enabled
  self.enabled = Wired.Persistence.PersistentVariable(
    "Enabled",
     self:GetContext():GetBase(),
     Wired.Utility.Application.Defaults.Enabled
  );

  -- debug
  self.debug = Wired.Persistence.PersistentVariable(
    "Debug",
     self:GetContext():GetBase(),
     Wired.Utility.Application.Defaults.Debug
  );
  
  -- create command manager
  self:CreateCommandManager();
end

-- create command manager
function Wired.Utility.Application:CreateCommandManager()
  
  -- create
  self:SetCommandManager(Wired.Utility.CommandManager(self, self:GetCommand()));
  
  -- clear
  self:GetCommandManager():AddCommand(Wired.Utility.Command(
    self,
    "clear",
    "Clears any persistent data",
    function(value)
      self.persistence.Root = {};
      print(self:GetName() .. "'s persistent data cleared.");
      print("You must /reloadui for this change to take effect.");
    end
  ));
  
  -- debug
  self:GetCommandManager():AddCommand(Wired.Utility.Command(
    self,
    "debug",
    "Enables or disables " .. self:GetName() .. "'s debug mode",
    function(value)
      self:SetDebug(not self:GetDebug());
      if (self:GetDebug()) then
        print(self:GetName() .. "'s debug mode is now enabled.");
      else
        print(self:GetName() .. "'s debug mode is now disabled.");
      end
    end
  ));
  
  -- enable
  self:GetCommandManager():AddCommand(Wired.Utility.Command(
    self,
    "enable",
    "Enables or disables " .. self:GetName() .. "'s functionality",
    function(value)
      self:SetEnabled(not self:GetEnabled());
      if (self:GetEnabled()) then
        print(self:GetName() .. " is now enabled.");
      else
        print(self:GetName() .. " is now disabled.");
      end
    end
  ));
  
  -- options
  self:GetCommandManager():AddCommand(Wired.Utility.Command(
    self,
    "options",
    "Shows " .. self:GetName() .. "'s options window",
    function(value)
      if (self:GetOptions()) then
        self:GetOptions():SetVisible(true);
        print("Showing " .. self:GetName() .. "'s options window.");
      else
        print("No options window exists for " .. self:GetName() .. ".");
      end
    end
  ));
end

-- debug
function Wired.Utility.Application:Debug(value)
  if (self:GetDebug()) then
    print(value);
  end
end

-- translation

-- language
function Wired.Utility.Application:GetLanguage()
  if (not self.language) then
    self.language = Inspect.System.Language();
  end
  return self.language;
end

function Wired.Utility.Application:SetLanguage(value)
  self.language = value;
end

-- translation
function Wired.Utility.Application:GetTranslations()
  if (type(self.translations) ~= "table") then
    self.translations = {};
  end
  return self.translations;
end

function Wired.Utility.Application:SetTranslations(value)
  self.translations = value;
end

function Wired.Utility.Application:AddTranslation(value)
  self:GetTranslations()[value:GetLanguage()] = value;
end

-- default translation
function Wired.Utility.Application:GetDefaultTranslation()
  return self.defaultTranslation;
end

function Wired.Utility.Application:SetDefaultTranslation(value)
  self.defaultTranslation = value;
  
  -- add tranlsation
  self:AddTranslation(self:GetDefaultTranslation());
end

-- default translate
function Wired.Utility.Application:DT(value)
  
  --  default translation
  local result = self:GetDefaultTranslation():Translate(value);
  
  -- default value
  if (not result) then
    result = value;
  end
  
  -- return
  return result;
end

-- translate
function Wired.Utility.Application:T(value)
  
  -- result
  local result = nil;
  
  -- translation
  local translation = self:GetTranslations()[self:GetLanguage()];
  
  -- default translation
  if (not translation) then
    translation = self:GetDefaultTranslation();
  end
  
  -- translated value
  result = translation:Translate(value);
  
  -- default translated value
  if (not result) then
    result = self:DT(value);
  end
  
  -- return
  return result;
end
